#!/bin/sh

if [ "$buildpath" = "" ]; then
	buildpath="`pwd`"
fi

dio87d_dir="$buildpath/../src/dio87d"
dio87c_dir="$buildpath/../src/dio87c"

PathDio87d="$dio87d_dir/dio87d"
PathDio87c="$dio87c_dir/dio87c"

PATH="$dio87d_dir:$dio87c_dir:$PATH"
export PATH

DAEMON_OPTION=""

dayDate=`date +"%Y-%m-%d %H:%M:%S"`
export DG87_USER="admin"
export DG87_PASSWORD=""

JOBPATH=$buildpath/$srcdir/jobs
DATAPATH=$buildpath/$srcdir/data

# check data directory
if [ ! -d $DATAPATH ]; then
	# create directory
	mkdir $DATAPATH | exit 1
else
	# remove previous data files
	rm $DATAPATH/* -f >/dev/null 2>&1
fi
# create 'dio87d' configuration file
cat > $DATAPATH/dio87d.conf << EOF
IPPORT=9993
PID_FILE=$DATAPATH/dio87d.pid
DATA_DIR=$DATAPATH
SAVE_TEMPO=300
LOG_LEVEL=ERROR
EOF
# create 'dio87c' configuration file
cat > $DATAPATH/dio87c.conf << EOF
ADDRESS=localhost
IPPORT=9993
EOF

################################################################################
checkNoDaemon ()
{
n="`ps ax | awk '{print \$5}' | grep \"[d]io87d\" | wc -l | awk '{print \$1}'`"
if [ "$n" -ne 0 ]; then
	echo "ERROR : diogene87 daemon is already running !"; 
	exit 1
fi
}
################################################################################
stopDio87d ()
{
echo "Stop dio87d"
pid="`ps ax | awk '{print \$1 \" \" \$5}' | grep [d]io87d | awk '{print \$1}'`"
if [ "$pid" != "" ] ; then
	kill -s TERM $pid
fi
sleep 1
n="`ps ax | awk '{print \$5}' | grep \"[d]io87d\" | wc -l | awk '{print \$1}'`"
if [ "$n" -ne 0 ]; then
	echo "ERROR : dio87d is not stopped !"; exit 1 
fi
}
################################################################################
startDio87d ()
{
if [ ! -f $PathDio87d ] ; then
	echo "Can't find dio87d : $PathDio87d"
	return 1
fi
# initialize previous database
rm $DATAPATH/dio87d.data >/dev/null 2>&1
$PathDio87d -C $DATAPATH/dio87d.conf -I >/dev/null 2>&1

echo "Start dio87d"
$PathDio87d -C $DATAPATH/dio87d.conf -d $DAEMON_OPTION
sleep 1
dio87c -- version > /dev/null
if [ $? != 0 ] ; then
	echo "Can't communicate with dio87d"
	return 1;
fi
}
################################################################################
restartDio87d ()
{
stopDio87d
if [ ! -f $PathDio87d ] ; then
	echo "Can't find dio87d : $PathDio87d"
	return 1
fi
$PathDio87d -d -C $DATAPATH/dio87d.conf
if [ $? -ne 0 ] ; then
	echo "can't restart dio87d !" ; exit 1
fi
sleep 1
dio87c -- version > /dev/null
if [ $? != 0 ] ; then
	echo "Can't communicate with dio87d"
	return 1;
fi
}

################################################################################
waitMessageSending ()
{
retry=0
#echo -n " Wait messages sending "
while test $retry -lt 6
do
	sleep 1
	if [ "`eval dio87c mc`" -eq 0 ]; then
		# ok all messages are sended
		#echo "OK"
		return 0
	fi
	#echo -n "."
done
echo "Messages are not sended after 5 seconds ! timeout error !"
# timeout error
exit 1
}
################################################################################
waitretry ()
{
	max=$1
	cmd=$2
	value=$3
	res=""
	n=0
	#echo -n "  wait "
	while [ $n -ne $max ]
	do
		res="`eval $cmd`"
		if [ "$verbose" = "yes" ]; then
			echo $res
		fi
		if [ "$res" = "$value" ]; then
			return 0
		fi
		sleep 1
		n=$(($n+1))
	done
	echo "ERROR"
	echo "  - command:  [$cmd]"
	echo "  - result:   [$res]"
	echo "  - attempted:[$value]"
	return 1
}
################################################################################
