#!/bin/sh
#
# file tst02.sh 
# (test dio87d functions)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test referential commands"

startDio87d || exit 1

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

################################################## TARGET 
echo "Test target commands"
mem1=`dio87c -- memory`
# add a target 
dio87c -- ta tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a target"; exit 1; fi
# verify target count = 2
n=`dio87c -- tc`
if [ $n != 2 ] ; then echo "Target count isn't 2"; exit 1; fi
# change target description
dio87c -- tsd tg1 NewDescr >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change the target description!"; exit 1; fi
# verify description change
if [ x`dio87c -- ti tg1 | grep Description | awk '{print $2}'` != xNewDescr ] ; then
	echo "Description change of Target failed"; exit 1;
fi
# change target ipport 
dio87c -- tsp tg1 6666 >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change ipport of Target"; exit 1; fi
# verify ipport change
if [ x`dio87c -- ti tg1 | grep "IP Port" | awk '{print $3}'` != x6666 ] ; then
	echo "IPPort change of Target failed"; exit 1;
fi

# try to add a second target with same name
dio87c -- ta tg1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add two target with same name"; exit 1; fi
# delete target 
dio87c -- tk tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the target"; exit 1; fi
# verify target count = 1
n=`dio87c -- tc`
if [ $n != 1 ] ; then echo "Target count isn't 1"; exit 1; fi
# try to delete a target which doesn't exist
dio87c -- tk tg1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete the target which doesn't exist"; exit 1; fi

mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after target $mem1 -> $mem2"
	exit 1
fi

################################################## QUEUE 
echo "Test queue commands"
# add a target 
dio87c -- ta tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a target"; exit 1; fi
# add a queue 
dio87c -- qa tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a queue"; exit 1; fi
# change queue description
dio87c -- qsd tg1 q1 NewDescr >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change description of Queue"; exit 1; fi
# verify description change
if [ x`dio87c -- qi tg1 q1 | grep Description |awk '{print $2}'` != xNewDescr ] ; then
	echo "Description change of Queue failed"; exit 1;
fi
# verify queue count = 2
n=`dio87c -- qc tg1`
if [ $n != 2 ] ; then echo "Queue count isn't 2"; exit 1; fi
# try to add a second queue with same name
dio87c -- qa tg1 q1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add two queue with same name"; exit 1; fi
# delete queue 
dio87c -- qk tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the queue"; exit 1; fi
# verify queue count = 0
n=`dio87c -- qc tg1`
if [ $n != 1 ] ; then echo "Queue count isn't 1"; exit 1; fi
# try to delete a queue which doesn't exist
dio87c -- qk tg1 q1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete a queue which doesn't exist"; exit 1; fi
# delete the target
dio87c -- tk tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a target"; exit 1; fi

mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after queue $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a target with queues
mem1=`dio87c -- memory`
dio87c -- ta tg1 > /dev/null 2>&1
dio87c -- qa tg1 q1 > /dev/null 2>&1
dio87c -- qa tg1 q2 > /dev/null 2>&1
dio87c -- qa tg1 q3 > /dev/null 2>&1
dio87c -- tk tg1 > /dev/null 2>&1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error delete target with queues $mem1 -> $mem2"
	exit 1
fi


################################################## REF
echo "Test referential commands"
# add a ref
dio87c -- ra r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a ref"; exit 1; fi
# verify ref count = 1
n=`dio87c -- rc`
if [ $n != 1 ] ; then echo "Referential count isn't 1"; exit 1; fi
# try to add a second ref with same name
dio87c -- ra r1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add two ref with same name"; exit 1; fi
# delete ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the ref"; exit 1; fi
# verify ref count = 0
n=`dio87c -- rc`
if [ $n != 0 ] ; then echo "Referential count isn't 0"; exit 1; fi
# try to delete a ref which doesn't exist
dio87c -- rk r1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete the ref which doesn't exist"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after ref $mem1 -> $mem2"
	exit 1
fi

################################################## TASK 
echo "Test task commands"
# add a ref 
dio87c -- ra r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a ref"; exit 1; fi
# add a task
dio87c -- rta r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a task"; exit 1; fi
# verify ref count = 1
n=`dio87c -- rtc r1`
if [ $n != 1 ] ; then echo "task count isn't 1"; exit 1; fi
# verify day name
if [ "`dio87c -- rti -S! r1 t1 | grep \"Referential Name\"`" != "Referential Name!r1" ] ; then
	echo "Task ref is not set." ; exit 1
fi
# change task description
dio87c -- rtsd r1 t1 NewDescr >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change description of Task"; exit 1; fi
# verify description change
if [ x`dio87c -- rti r1 t1 | grep Description |awk '{print $2}'` != xNewDescr ] ; then
	echo "Description change of Task failed"; exit 1;
fi

# verify that enabled property is true
if [ x`dio87c -- rti r1 t1 | grep Enabled |awk '{print $2}'` != xtrue ] ; then
	echo "enabled property of task isn't true"; exit 1;
fi
# change enabled property
dio87c -- rtse r1 t1 false
if [ $? != 0 ] ; then echo "Can't change enabled property"; exit 1; fi
# verify that property is changed
if [ x`dio87c -- rti r1 t1 | grep Enabled |awk '{print $2}'` != xfalse ] ; then
	echo "Enabled property of task change failed"; exit 1;
fi

# verify that waitAllDependencies is true
if [ x`dio87c -- rti r1 t1 | grep "All Dep" |awk '{print $4}'` != xtrue ] ; then
	echo "waitAllDependencies property of task isn't true"; exit 1;
fi
# change waitAllDependencies property
dio87c -- rtswad r1 t1 false
if [ $? != 0 ] ; then echo "Can't change waitAllDependencies property";exit 1;fi
# verify that property is changed
if [ x`dio87c -- rti r1 t1 | grep "All Dep" |awk '{print $4}'` != xfalse ] ; then
	echo "waitAllDependencies property of task change failed"; exit 1;
fi

# verify that onErrorContinue is true
if [ x`dio87c -- rti r1 t1 | grep "On Error" |awk '{print $4}'` != xtrue ] ; then
	echo "onErrorContinue property of task isn't true"; exit 1;
fi
# change OnErrorContinue property
dio87c -- rtsoec r1 t1 false
if [ $? != 0 ] ; then echo "Can't change onErrorContinue property";exit 1;fi
# verify that property is changed
if [ x`dio87c -- rti r1 t1 | grep "On Error" |awk '{print $4}'` != xfalse ] ; then
	echo "OnErrorContinue property of task change failed"; exit 1;
fi

# try to add a second task with same name
dio87c -- rta r1 t1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add two tasks with same name"; exit 1; fi
# delete task 
dio87c -- rtk r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the task"; exit 1; fi
# verify task count = 0
n=`dio87c -- rtc r1`
if [ $n != 0 ] ; then echo "Task count isn't 0"; exit 1; fi
# try to delete a task which doesn't exist
dio87c -- rtk r1 t1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete a task which doesn't exist"; exit 1; fi
# delete the ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after task $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a ref with tasks 
mem1=`dio87c -- memory`
dio87c -- ra r1 > /dev/null 2>&1
dio87c -- rta r1 t1 > /dev/null 2>&1
dio87c -- rta r1 t2 > /dev/null 2>&1
dio87c -- rta r1 t3 > /dev/null 2>&1
dio87c -- rk r1 > /dev/null 2>&1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error deleting ref with tasks $mem1 -> $mem2"
	exit 1
fi

################################################## JOB
echo "Test job commands"
# add a ref 
dio87c -- ra r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a ref"; exit 1; fi
# add a task
dio87c -- rta r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a task"; exit 1; fi
# add a job 
dio87c -- rja r1 t1 j1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a job"; exit 1; fi
# verify ref count = 1
n=`dio87c -- rjc r1 t1`
if [ $n != 1 ] ; then echo "job count isn't 1"; exit 1; fi
# verify task 
if [ "`dio87c -- rji -S! r1 t1 j1 | grep \"Task Name\"`" != "Task Name!t1" ] ; then
	echo "Job task is not set." ; exit 1
fi
# verify ref 
if [ "`dio87c -- rji -S! r1 t1 j1 | grep \"Referential Name\"`" != "Referential Name!r1" ] ; then
	echo "Job ref is not set." ; exit 1
fi
# change job description
dio87c -- rjsd r1 t1 j1 NewDescr >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change description of job"; exit 1; fi
# verify description change
if [ x`dio87c -- rji r1 t1 j1 |grep Description |awk '{print $2}'` != xNewDescr ]; 
then echo "Description change of job failed"; exit 1; fi
# change job path 
dio87c -- rjsp r1 t1 j1 ls >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change path of job"; exit 1; fi
# verify description change
if [ x`dio87c -- rji r1 t1 j1 |grep Path |awk '{print $2}'` != xls ]; 
then echo "Path change of job failed"; exit 1; fi
# change job argument 
dio87c -- rjsa r1 t1 j1 fic >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change argument of job"; exit 1; fi
# verify description change
if [ x`dio87c -- rji r1 t1 j1 |grep Argument |awk '{print $2}'` != xfic ]; 
then echo "Argument change of job failed"; exit 1; fi
# change job tempo 
dio87c -- rjst r1 t1 j1 66 67 >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change tempo of job"; exit 1; fi
# verify tempo change
if [ x`dio87c -- rji r1 t1 j1 |grep "Start Tempo" |awk '{print $3}'` != x66 ]; 
then echo "Start tempo change of job failed"; exit 1; fi
if [ x`dio87c -- rji r1 t1 j1 |grep "End Tempo" |awk '{print $3}'` != x67 ]; 
then echo "End tempo change of job failed"; exit 1; fi

# verify that waitAllDependencies is true
if [ x`dio87c -- rji r1 t1 j1 | grep "All Dep" |awk '{print $4}'` != xtrue ] ; then
	echo "waitAllDependencies property of job isn't true"; exit 1;
fi
# change waitAllDependencies property
dio87c -- rjswad r1 t1 j1 false
if [ $? != 0 ] ; then echo "Can't change waitAllDependencies property";exit 1;fi
# verify that property is changed
if [ x`dio87c -- rji r1 t1 j1 | grep "All Dep" |awk '{print $4}'` != xfalse ] ; then
	echo "waitAllDependencies property of job change failed"; exit 1;
fi

# verify that onErrorContinue is true
if [ x`dio87c -- rji r1 t1 j1 | grep "On Error" |awk '{print $4}'` != xtrue ] ; then
	echo "OnErrorContinue property of job isn't true"; exit 1;
fi
# change OnErrorContinue property
dio87c -- rjsoec r1 t1 j1 false
if [ $? != 0 ] ; then echo "Can't change OnErrorContinue property";exit 1;fi
# verify that property is changed
if [ x`dio87c -- rji r1 t1 j1 | grep "On Error" |awk '{print $4}'` != xfalse ] ; then
	echo "OnErrorContinue property of job change failed"; exit 1;
fi

# try to add a second job with same name
dio87c -- rja r1 t1 j1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add two jobs with same name"; exit 1; fi
# delete job 
dio87c -- rjk r1 t1 j1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the job"; exit 1; fi
# verify task count = 0
n=`dio87c -- rjc r1 t1`
if [ $n != 0 ] ; then echo "Job count isn't 0"; exit 1; fi
# try to delete a job which doesn't exist
dio87c -- rjk r1 t1 j1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete a job which doesn't exist"; exit 1; fi
# delete the task 
dio87c -- rtk r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a task"; exit 1; fi
# delete the ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after job $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a ref with jobs
mem1=`dio87c -- memory`
dio87c -- ra r1 > /dev/null 2>&1
dio87c -- rta r1 t1 > /dev/null 2>&1
dio87c -- rja r1 t1 j1 > /dev/null 2>&1
dio87c -- rja r1 t1 j2 > /dev/null 2>&1
dio87c -- rja r1 t1 j3 > /dev/null 2>&1
dio87c -- rk r1 > /dev/null 2>&1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error deleting ref with jobs $mem1 -> $mem2"
	exit 1
fi

################################################## TASK SET QUEUE 
echo "Test Task-Queue commands"
mem1=`dio87c -- memory`
dio87c -- ra r1 > /dev/null 2>&1
dio87c -- rta r1 t1 > /dev/null 2>&1
dio87c -- ta tg1 > /dev/null 2>&1
dio87c -- qa tg1 q1 > /dev/null 2>&1
# set queue for task 
dio87c -- rtst r1 t1 tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't set queue"; exit 1; fi
# verify that queue is set
if [ x`dio87c -- rti r1 t1 | sed -ne "s/Queue:.\(.*\)/\1/p"` != xq1 ] ; then
	echo "Queue is not set"
	exit 1
fi
# verify that target is set
if [ x`dio87c -- rti r1 t1 | sed -ne "s/Target:.\(.*\)/\1/p"` != xtg1 ] ; then
	echo "Target is not set"
	exit 1
fi
# verify that task is in task list of target
if [ x`dio87c -- qrtc tg1 q1` != x1 ] ; then
	echo "Task list count is not 1."
	exit 1
fi

if [ x`dio87c -- qrtl tg1 q1 | awk '{print $2}'` != xt1 ] ; then
	echo "Task is not in task list of target."
	exit 1
fi
########## unset task
# set queue for task to none
dio87c -- rtst r1 t1 none > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't set queue to none"; exit 1; fi
# verify that queue is unset
if [ x`dio87c -- rti r1 t1 | sed -ne "s/Queue:.\(.*\)/\1/p"` != xnone ] ; then
	echo "Queue is not unset"
	exit 1
fi
# verify that target is unset
if [ x`dio87c -- rti r1 t1 | sed -ne "s/Target:.\(.*\)/\1/p"` != xnone ] ; then
	echo "Target is not unset"
	exit 1
fi
# verify that task is not in task list of target
if [ x`dio87c -- qrtc tg1 q1` != x0 ] ; then
	echo "Task list count is not 0."
	exit 1
fi
######## delete task
# set queue for task 
dio87c -- rtst r1 t1 tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't set queue"; exit 1; fi
# verify that task is in task list of target
if [ x`dio87c -- qrtc tg1 q1` != x1 ] ; then
	echo "Task list count is not 1."
	exit 1
fi
dio87c -- rtk r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete task"; exit 1; fi
# verify that task is not in task list of target
if [ x`dio87c -- qrtc tg1 q1` != x0 ] ; then
	echo "Task list count for queue is not 0."
	exit 1
fi
######## delete queue 
dio87c -- rta r1 t1 > /dev/null 2>&1
dio87c -- rtst r1 t1 tg1 q1 > /dev/null 2>&1
# verify that queue is set
if [ x`dio87c -- rti r1 t1 | sed -ne "s/Queue:.\(.*\)/\1/p"` != xq1 ] ; then
	echo "Queue is not set"
	exit 1
fi
# delete queue
dio87c -- qk tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete queue"; exit 1; fi
# verify that queue is set to none for task
if [ x`dio87c -- rti r1 t1 | sed -ne "s/Queue:.\(.*\)/\1/p"` != xnone ] ; then
	echo "Queue is not unset"
	exit 1
fi
# cleanup
dio87c -- rk r1 > /dev/null 2>&1
dio87c -- tk tg1 > /dev/null 2>&1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## TASK DEPENDENCES 
echo "Test task dependencies commands" 
mem1=`dio87c -- memory`
dio87c -- ra r1 > /dev/null 2>&1
dio87c -- rta r1 t1 > /dev/null 2>&1
dio87c -- rta r1 t2 > /dev/null 2>&1
dio87c -- rta r1 t3 > /dev/null 2>&1
dio87c -- rta r1 t4 > /dev/null 2>&1
dio87c -- rta r1 t5 > /dev/null 2>&1

# try to add a dep with a wrong type
dio87c -- rtda r1 t1 t2 zzz > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a taskDep with wrong type"; exit 1; fi
# try to add a dep with an unknown ref 
dio87c -- rtda rzzz1 t1 t2> /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a taskDep with unknown ref"; exit 1; fi
# try to add a dep with an unknown task 
dio87c -- rtda r1 tzzz1 t2> /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a taskDep with unknown task"; exit 1; fi

# add a dep with different type
dio87c -- rtda r1 t1 t2 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a taskDep"; exit 1; fi
dio87c -- rtda r1 t2 t3 always > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a taskDep with type always"; exit 1; fi
dio87c -- rtda r1 t3 t4 onerror  > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a taskDep with type onerror"; exit 1; fi
dio87c -- rtda r1 t3 t5 onsuccess > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a taskDep with type onsucces"; exit 1; fi

# verify dep count for ref 
if [ x`dio87c -- rtdc r1` != x4 ] ; then
	echo "TaskDep count is not 4."
	exit 1
fi
# verify taskdep list command
if [ x`dio87c -- rtdl r1 | wc -l | awk '{print $1}'` != x4 ] ; then
	echo "Incorrect taskdep list."
	exit 1
fi

# delete a task dep
mem3=`dio87c -- memory`
dio87c -- rtda r1 t1 t5 always > /dev/null 2>&1
dio87c -- rtdk r1 t1 t5 always > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a taskdep"; exit 1; fi
if [ x`dio87c -- rtdl r1 | wc -l | awk '{print $1}'` != x4 ] ; then
	echo "Incorrect taskdep count."
	exit 1
fi
mem4=`dio87c -- memory`
if [ $mem3 != $mem4 ] ; then
	echo "dio87d : memory error after taskdep delete $mem1 -> $mem2"
	exit 1
fi

# delete a task 
dio87c -- rta r1 t6 > /dev/null 2>&1
dio87c -- rtda r1 t1 t6 > /dev/null 2>&1
# try to delete a task witch has dependencies
dio87c -- rtk r1 t6 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete a task with taskdep 1"; exit 1; fi
dio87c -- rtdk r1 t1 t6 always > /dev/null 2>&1
dio87c -- rtk r6 t1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete a task with taskdep 2"; exit 1; fi
dio87c -- rk r1 > /dev/null 2>&1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## JOB DEPENDENCES 
echo "Test job dependencies commands" 
mem1=`dio87c -- memory`

dio87c -- ra r1 > /dev/null 2>&1
dio87c -- rta r1 t1 > /dev/null 2>&1
dio87c -- rja r1 t1 j1 > /dev/null 2>&1
dio87c -- rja r1 t1 j2 > /dev/null 2>&1
dio87c -- rja r1 t1 j3 > /dev/null 2>&1
dio87c -- rja r1 t1 j4 > /dev/null 2>&1
dio87c -- rja r1 t1 j5 > /dev/null 2>&1

# try to add a dep with a wrong type
dio87c -- rjda r1 t1 j1 j2 zzz > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a jobDep with wrong type"; exit 1; fi

# try to add a dep with an unknown ref 
dio87c -- rjda rzzz1 t1 j1 j2> /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a jobDep with unknown ref"; exit 1; fi
# try to add a dep with an unknown task 
dio87c -- rjda r1 tzzz1 j1 j2> /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a jobDep with unknown task"; exit 1; fi
# try to add a dep with an unknown job 
dio87c -- rjda r1 t1 jzzz1 j2> /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a jobDep with unknown job"; exit 1; fi
# try to add a dep with an unknown type 
dio87c -- rjda r1 t1 j1 j2 zzz > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a jobDep with unknown type"; exit 1; fi

# add a dep with different type
dio87c -- rjda r1 t1 j1 j2 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a jobDep"; exit 1; fi
dio87c -- rjda r1 t1 j2 j3 always > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a jobDep with type always"; exit 1; fi
dio87c -- rjda r1 t1 j3 j4 onerror  > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a jobDep with type onerror"; exit 1; fi
dio87c -- rjda r1 t1 j3 j5 onsuccess > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a jobDep with type onsucces"; exit 1; fi

# verify dep count for task 
if [ x`dio87c -- rjdc r1 t1` != x4 ] ; then
	echo "JobDep count is not 4."
	exit 1
fi
# verify jobdep list command
if [ x`dio87c -- rjdl r1 t1 | wc -l | awk '{print $1}'` != x4 ] ; then
	echo "Incorrect jobdep list."
	exit 1
fi

# delete a job dep
mem3=`dio87c -- memory`
dio87c -- rjda r1 t1 j1 j5 always > /dev/null 2>&1
# verify dep count for task 
if [ x`dio87c -- rjdc r1 t1` != x5 ] ; then
	echo "JobDep count is incorrect before delete."
	exit 1
fi
dio87c -- rjdk r1 t1 j1 j5 always > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a jobdep"; exit 1; fi
# verify dep count for task 
if [ x`dio87c -- rjdc r1 t1` != x4 ] ; then
	echo "JobDep count is incorrect after delete."
	exit 1
fi

mem4=`dio87c -- memory`
if [ $mem3 != $mem4 ] ; then
	echo "dio87d : memory error after jobdep delete $mem1 -> $mem2"
	exit 1
fi

# delete a job 
dio87c -- rja r1 t1 j6 > /dev/null 2>&1
dio87c -- rjda r1 t1 j1 j6 > /dev/null 2>&1
dio87c -- rjda r1 t1 j6 j2 > /dev/null 2>&1
# verify dep count for task 
if [ x`dio87c -- rjdc r1 t1` != x6 ] ; then
	echo "JobDep count is incorrect after delete."
	exit 1
fi
# delete a job witch has dependencies
dio87c -- rjk r1 t1 j6 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can' delete a job with jobdep 1"; exit 1; fi
# verify dep count for task 
if [ x`dio87c -- rjdc r1 t1` != x4 ] ; then
	echo "JobDep count is incorrect after delete."
	exit 1
fi
# verify job count
if [ x`dio87c -- rjc r1 t1` != x5 ] ; then
	echo "Job count is incorrect after delete."
	exit 1
fi

# delete ref
dio87c -- rk r1 > /dev/null 2>&1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi


################################################## CONDITIONS
echo "Test conditions commands" 
mem1=`dio87c -- memory`

dio87c -- ra r1 > /dev/null 2>&1
dio87c -- rta r1 t1 > /dev/null 2>&1

# add a condition with an unknown ref
dio87c -- rtca rzzz1 t1 manual "Ready?" > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can't add a condition with unknown ref"; exit 1; fi
# add a condition with an unknown task 
dio87c -- rtca r1 tzzz1 manual "Ready?" > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can't add a condition with unknown task"; exit 1; fi
# add a condition with an unknown type 
dio87c -- rtca r1 t1 zzz "Ready?" > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can't add a condition with unknown type"; exit 1; fi
# add a condition without data argument
dio87c -- rtca r1 t1 manual > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can't add a condition with without data"; exit 1; fi

# add a manual condition 
dio87c -- rtca r1 t1 manual "Ready?" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a manual condition"; exit 1; fi
# add a file condition 
dio87c -- rtca r1 t1 file "/tmp/fictop" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a file condition"; exit 1; fi
# add conditions
dio87c -- rtca r1 t1 hmin "20:30" 0 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmin condition"; exit 1; fi
dio87c -- rtca r1 t1 hmax "23:30" 0 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmax condition"; exit 1; fi
dio87c -- rtca r1 t1 hlate "22:30" 0 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hlate condition"; exit 1; fi
dio87c -- rtca r1 t1 hmin "06:30" 1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmin condition"; exit 1; fi
dio87c -- rtca r1 t1 hmax "07:30" 2 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmax condition"; exit 1; fi
dio87c -- rtca r1 t1 hlate "08:30" 3 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hlate condition"; exit 1; fi
# verify condition count for task 
if [ x`dio87c -- rtcc r1 t1` != x8 ] ; then
	echo "Conditions count is incorrect."
	exit 1
fi
# verify condition list 
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x8 ] ; then
	echo "Condition list is incorrect";  exit 1
fi

# add and remove a manual condition 
mem3=`dio87c -- memory`
dio87c -- rtca r1 t1 manual "OK?" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a manual condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x9 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
dio87c -- rtck r1 t1 manual "OK?" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a manual condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x8 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
mem4=`dio87c -- memory`
if [ $mem3 != $mem4 ] ; then
	echo "dio87d : memory error after manual condition delete $mem3 -> $mem4"
	exit 1
fi


# add and remove a file condition 
mem3=`dio87c -- memory`
dio87c -- rtca r1 t1 file "/tmp/fic" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a file condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x9 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
dio87c -- rtck r1 t1 file "/tmp/fic" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a file condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x8 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
mem4=`dio87c -- memory`
if [ $mem3 != $mem4 ] ; then
	echo "dio87d : memory error after file condition delete $mem3 -> $mem4"
	exit 1
fi

# add and remove a hmin condition 
mem3=`dio87c -- memory`
dio87c -- rtca r1 t1 hmin "11:00" 0 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmin condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x9 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
dio87c -- rtck r1 t1 hmin "11:00" 0 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a hmin condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x8 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
mem4=`dio87c -- memory`
if [ $mem3 != $mem4 ] ; then
	echo "dio87d : memory error after hmin condition delete $mem3 -> $mem4"
	exit 1
fi

# add and remove a hmax condition 
mem3=`dio87c -- memory`
dio87c -- rtca r1 t1 hmax "11:00" 0 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmax condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x9 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
dio87c -- rtck r1 t1 hmax "11:00" 0 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a hmax condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x8 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
mem4=`dio87c -- memory`
if [ $mem3 != $mem4 ] ; then
	echo "dio87d : memory error after hmax condition delete $mem3 -> $mem4"
	exit 1
fi


# add and remove a hlate condition 
mem3=`dio87c -- memory`
dio87c -- rtca r1 t1 hlate "11:00" 0 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hlate condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x9 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
dio87c -- rtck r1 t1 hlate "11:00" 0 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a hlate condition"; exit 1; fi
if [ x`dio87c -- rtcl r1 t1 | wc -l | awk '{print $1}'` != x8 ] ; then
	echo "Condition list is incorrect !"; exit 1
fi
mem4=`dio87c -- memory`
if [ $mem3 != $mem4 ] ; then
	echo "dio87d : memory error after hlate condition delete $mem3 -> $mem4"
	exit 1
fi

dio87c -- rk r1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## REF COPY 
echo "Test referential copy" 

# create a target and a queue
dio87c -- ta tg1 > /dev/null 2>&1
dio87c -- qa tg1 q1 > /dev/null 2>&1

mem1=`dio87c -- memory`
# create a base ref
dio87c -- ra r1 > /dev/null 2>&1

dio87c -- rta r1 t1 "task1" > /dev/null 2>&1
dio87c -- rta r1 t2 "task2" > /dev/null 2>&1
dio87c -- rta r1 t3 "task3" > /dev/null 2>&1
dio87c -- rta r1 t4 "task4" > /dev/null 2>&1
dio87c -- rtda r1 t1 t2 onerror > /dev/null 2>&1
dio87c -- rtda r1 t1 t3 always > /dev/null 2>&1
dio87c -- rtda r1 t3 t4 onsuccess > /dev/null 2>&1
dio87c -- rtswad r1 t2 false || exit 1
dio87c -- rtswad r1 t3 true || exit 1
dio87c -- rtsoec r1 t1 false || exit 1
dio87c -- rtsoec r1 t4 true || exit 1

dio87c -- rja r1 t1 j1 "task1-job1" > /dev/null 2>&1
dio87c -- rja r1 t1 j2 "task1-job2" > /dev/null 2>&1
dio87c -- rja r1 t1 j3 "task1-job3" > /dev/null 2>&1
dio87c -- rja r1 t1 j4 "task1-job4" > /dev/null 2>&1
dio87c -- rjda r1 t1 j1 j2 onerror > /dev/null 2>&1
dio87c -- rjda r1 t1 j1 j3 always > /dev/null 2>&1
dio87c -- rjda r1 t1 j3 j4 onsuccess > /dev/null 2>&1
dio87c -- rjsp r1 t1 j1 "ls"
dio87c -- rjsa r1 t1 j1 "arg"
dio87c -- rjst r1 t1 j1 10 20
dio87c -- rjswad r1 t1 j2 false || exit 1
dio87c -- rjswad r1 t1 j3 true || exit 1
dio87c -- rjsoec r1 t1 j1 false || exit 1
dio87c -- rjsoec r1 t1 j4 true || exit 1

dio87c -- rja r1 t2 j1 "task2-job1" > /dev/null 2>&1
dio87c -- rja r1 t2 j2 "task2-job2" > /dev/null 2>&1
dio87c -- rjda r1 t2 j1 j2 always > /dev/null 2>&1

dio87c -- rja r1 t3 j1 "task3-job1" > /dev/null 2>&1
dio87c -- rja r1 t3 j2 "task3-job2" > /dev/null 2>&1
dio87c -- rjda r1 t3 j1 j2 always > /dev/null 2>&1

dio87c -- rja r1 t4 j1 "task4-job1" > /dev/null 2>&1
dio87c -- rja r1 t4 j2 "task4-job2" > /dev/null 2>&1
dio87c -- rjda r1 t4 j1 j2 onerror > /dev/null 2>&1

dio87c -- rtca r1 t1 hmin "11:00" 0 
dio87c -- rtca r1 t1 hmax "12:00" 0 
dio87c -- rtca r1 t2 manual "ok?"
dio87c -- rtca r1 t4 file "/tmp/fic"
dio87c -- rtst r1 t1 tg1 q1 > /dev/null 2>&1
dio87c -- rtst r1 t2 none > /dev/null 2>&1

# make a copy of the ref
mem2=`dio87c -- memory`
dio87c -- rd r1 r2
if [ $? != 0 ] ; then echo "Can't copy a ref"; exit 1; fi
mem3=`dio87c -- memory`

# verify memory 
delta1=$(( $mem2 - $mem1))
delta2=$(( $mem3 - $mem2))
if [ $delta1 -ne $delta2 ] ; then 
	echo "memory delta different after ref duplicate"
	exit 1
fi

# verify task copy
nbr=`dio87c -- rtc r2`
if [ $nbr -ne 4 ] ; then echo "Wrong task count"; exit 1; fi
# verify condition count
nbr=`dio87c -- rtcc r2 t1` 
if [ $nbr -ne 2 ] ; then echo "Wrong condition count"; exit 1; fi
# verify task dependence count
nbr=`dio87c -- rtdc r2` 
if [ $nbr -ne 3 ] ; then echo "Wrong task dependence count"; exit 1; fi
# verify job count
nbr=`dio87c -- rjc r2 t1`
if [ $nbr -ne 4 ] ; then echo "Wrong job count"; exit 1; fi
# verify job dependence count
nbr=`dio87c -- rjdc r2 t1`
if [ $nbr -ne 3 ] ; then echo "Wrong job dependence count"; exit 1; fi
# verify a task
if [ x`dio87c -- rtl r2 | grep t1` != "xt1" ] ; then
	echo "Can't find a task."; exit 1
fi
if [ x`dio87c -- rti r2 t1 | grep Description | awk '{print $2;}'` != "xtask1" ] 
then
	echo "Wrong task description."; exit 1
fi
# verify task which queue is set 
if [ x`dio87c -- rti r2 t1 | grep Queue | awk '{print $2;}'` != "xq1" ] ; then
	echo "Queue is not set"; exit 1
fi
# verify task which queue is not set 
if [ x`dio87c -- rti r2 t2 | grep Queue | awk '{print $2;}'` != "xnone" ] ; then
	echo "Queue is set"; exit 1
fi
# verify task waitAllDependencies property 
if [ x`dio87c -- rti r2 t2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ] ; 
then
	echo "waitAllDependencies property is not false"; exit 1
fi
if [ x`dio87c -- rti r2 t3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ] ; 
then
	echo "waitAllDependencies property is not true"; exit 1
fi
# verify task OnErrorContinue property 
if [ x`dio87c -- rti r2 t1 | grep "On Error"| awk '{print $4;}'` != "xfalse" ] ; 
then
	echo "OnErrorContinue property is not false"; exit 1
fi
if [ x`dio87c -- rti r2 t4 | grep "On Error"| awk '{print $4;}'` != "xtrue" ] ; 
then
	echo "OnErrorContinue property is not true"; exit 1
fi

# verify a job 
if [ x`dio87c -- rjl r2 t1 | grep j1` != "xj1" ] ; then
	echo "Can't find a job."; exit 1
fi
if [ x`dio87c -- rji r2 t1 j1 | grep Description | awk '{print $2}'` != "xtask1-job1" ] ; then
	echo "Wrong job description."; exit 1
fi
if [ x`dio87c -- rji r2 t1 j1 | grep Path | awk '{print $2}'` != "xls" ] ; then
	echo "Wrong job path."; exit 1
fi
if [ x`dio87c -- rji r2 t1 j1 | grep Argument | awk '{print $2}'` != "xarg" ] ; then
	echo "Wrong job argument."; exit 1
fi
if [ x`dio87c -- rji r2 t1 j1 | grep Start | awk '{print $3}'` != "x10" ] ; then
	echo "Wrong start tempo."; exit 1
fi
if [ x`dio87c -- rji r2 t1 j1 | grep End | awk '{print $3}'` != "x20" ] ; then
	echo "Wrong end tempo."; exit 1
fi
# verify job waitAllDependencies property 
if [ x`dio87c -- rji r2 t1 j2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ] ; then
	echo "waitAllDependencies property of job is not false"; exit 1
fi
if [ x`dio87c -- rji r2 t1 j3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ] ; then
	echo "waitAllDependencies property of job is not true"; exit 1
fi
# verify job OnErrorContinue property 
if [ x`dio87c -- rji r2 t1 j1 | grep "On Error"| awk '{print $4;}'` != "xfalse" ] ; then
	echo "OnErrorContinue property of job is not false"; exit 1
fi
if [ x`dio87c -- rji r2 t1 j4 | grep "On Error"| awk '{print $4;}'` != "xtrue" ] ; then
	echo "OnErrorContinue property of job is not true"; exit 1
fi

# verify task dependence
if [ "`dio87c -- rtdl -S! -t r2 | grep onerror`" != "t1!t2!onerror" ] ; then
	echo "Incorrect task dependence (onerror)"; exit 1
fi
if [ "`dio87c -- rtdl -S! -t r2 | grep always`" != "t1!t3!always" ] ; then
	echo "Incorrect task dependence (always)"; exit 1
fi
if [ "`dio87c -- rtdl -S! -t r2 | grep onsuccess`" != "t3!t4!onsuccess" ] ; then
	echo "Incorrect task dependence (onsuccess)"; exit 1
fi

# verify job dependence
if [ "`dio87c -- rjdl -S! -t r2 t1 | grep onerror`" != "j1!j2!onerror" ] ; then
	echo "Incorrect job dependence (onerror)"; exit 1
fi
if [ "`dio87c -- rjdl -S! -t r2 t1 | grep always`" != "j1!j3!always" ] ; then
	echo "Incorrect job dependence (always)"; exit 1
fi
if [ "`dio87c -- rjdl -S! -t r2 t1 | grep onsuccess`" != "j3!j4!onsuccess" ] ; then
	echo "Incorrect job dependence (onsuccess)"; exit 1
fi
# verify condition
if [ "`dio87c -- rtcl -S! r2 t1 | grep hmin`" != "hmin!11:00!+0" ] ; then
	echo "Incorrect hmin condition"; exit 1
fi
if [ "`dio87c -- rtcl -S! r2 t1 | grep hmax`" != "hmax!12:00!+0" ] ; then
	echo "Incorrect hmax condition"; exit 1
fi
if [ "`dio87c -- rtcl -S! r2 t2 | grep manual`" != "manual!ok?" ] ; then
	echo "Incorrect manual condition"; exit 1
fi
if [ "`dio87c -- rtcl -S! r2 t4 | grep file`" != "file!/tmp/fic" ] ; then
	echo "Incorrect file condition"; exit 1
fi

# delete referential copy
dio87c -- rk r2 > /dev/null 2>&1
mem3=`dio87c -- memory`
if [ $mem3 -ne $mem2 ] ; then 
	echo "memory delta different after ref copy delete"
	exit 1
fi
# delete original referential
dio87c -- rk r1 > /dev/null 2>&1
mem3=`dio87c -- memory`
if [ $mem3 -ne $mem1 ] ; then 
	echo "memory delta different after orig ref delete"
	exit 1
fi

stopDio87d || exit 1
