#!/bin/sh
#
# file tst04.sh 
# (Test day planification)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Test day planification"

startDio87d || exit 1

memd1=`dio87c -- memory`

# add target and queues
dio87c -- ta tg1 || exit 1
for i in 1 2 3 4 ; do
	dio87c -- qa tg1 q$i || exit 1
done

echo "Add ref"
# add new ref
dio87c -- ra r1 || exit 1

# add tasks 
echo "Add tasks"
for i in 1 2 3 4 ; do
	dio87c -- rta r1 t$i task$i || exit 1
	dio87c -- rtst r1 t$i tg1 q$i || exit 1
	dio87c -- rtoa r1 t$i add always || exit 1
done
dio87c -- rtsoec r1 t1 false || exit 1
dio87c -- rtsoec r1 t4 true || exit 1
dio87c -- rtswad r1 t2 false || exit 1
dio87c -- rtswad r1 t3 true || exit 1

# add task dependencies
dio87c -- rtda r1 t1 t2 always || exit 1
dio87c -- rtda r1 t2 t3 onsuccess  || exit 1
dio87c -- rtda r1 t2 t4 onerror || exit 1

# add conditions
dio87c -- rtca r1 t1 hmin "08:00" 0 || exit 1
dio87c -- rtca r1 t1 hmax "09:00" 0 || exit 1
dio87c -- rtca r1 t1 hlate "10:00" 0 || exit 1
dio87c -- rtca r1 t1 file "/tmp/fic" || exit 1
dio87c -- rtca r1 t1 manual "ok" || exit 1

# add jobs 
echo "Add jobs"
for i in 1 2 3 4 ; do
	dio87c -- rja r1 t1 j$i job$i || exit 1
	dio87c -- rjsp r1 t1 j$i "/job$i.sh" || exit 1
	dio87c -- rjsa r1 t1 j$i "arg$i" || exit 1
	dio87c -- rjst r1 t1 j$i $(( $i + 10)) $(( $i + 20 )) || exit 1
done
dio87c -- rjsoec r1 t1 j1 false || exit 1
dio87c -- rjsoec r1 t1 j4 true || exit 1
dio87c -- rjswad r1 t1 j2 false || exit 1
dio87c -- rjswad r1 t1 j3 true || exit 1

# add job dependencies
dio87c -- rjda r1 t1 j1 j2 always || exit 1
dio87c -- rjda r1 t1 j2 j3 onsuccess || exit 1
dio87c -- rjda r1 t1 j2 j4 onerror || exit 1

echo "Plan ref"
dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1
dio87c -- ese true || exit 1
sleep 1

# verify that day is plan
echo "Verify day"
if [ "`dio87c -- xdc`" != "1" ] ; then echo "Day count isn't 1"; exit 1; fi
if [ "`dio87c -- xdl`" != "d1" ] ; then echo "Can't find the day"; exit 1; fi

if [ "`dio87c -- xdi -S! d1 | grep \"Start Time\"`" != "Start Time!$dayDate" ] ; then
	echo "Day date is not set." ; exit 1
fi
if [ "`dio87c -- xdi -S! d1 | grep Enabled`" != "Enabled!false" ] ; then
	echo "Day is not disabled." ; exit 1
fi

# verify task properties
echo "Verify tasks"
if [ "`dio87c -- xtc d1`" != "4" ] ; then echo "Task count isn't 4"; exit 1; fi
for i in 1 2 3 4 ; do
	if [ "`dio87c -- xtl -S! -d d1 | grep t$i`" != "t$i!task$i" ] ; then 
		echo "Can't find task t$i in task list"; exit 1; 
	fi
	if [ "`dio87c -- xti -S! d1 t$i | grep Queue`" != "Queue!q$i" ]; then
		echo "Queue is not set for task t$i"; exit 1; 
	fi
done
# verify waitAllDependencies property
if [ x`dio87c -- xti d1 t2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ] ; 
then
	echo "waitAllDependencies property is not false"; exit 1
fi
if [ x`dio87c -- xti d1 t3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ] ; 
then
	echo "waitAllDependencies property is not true"; exit 1
fi
# verify OnErrorContinue property
if [ x`dio87c -- xti d1 t1 | grep "On Error"| awk '{print $4;}'` != "xfalse" ] ; 
then
	echo "OnErrorContinue property is not false"; exit 1
fi
if [ x`dio87c -- xti d1 t4 | grep "On Error"| awk '{print $4;}'` != "xtrue" ] ; 
then
	echo "OnErrorContinue property is not true"; exit 1
fi

# verify task dependencies
echo "Verify task dependencies"
if [ "`dio87c -- xtdc d1`" != "3" ] ; then 
	echo "Task dependencies count isn't 3"; exit 1; 
fi
if [ "`dio87c -- xtdl -tS! d1 | grep always`" != "t1!t2!always" ] ; then 
	echo "Can't find task dependencies (type always)"; exit 1; 
fi
if [ "`dio87c -- xtdl -tS! d1 | grep onsuccess`" != "t2!t3!onsuccess" ] ; then 
	echo "Can't find task dependencies (type onsuccess)"; exit 1; 
fi
if [ "`dio87c -- xtdl -tS! d1 | grep onerror`" != "t2!t4!onerror" ] ; then 
	echo "Can't find task dependencies (type onerror)"; exit 1; 
fi

# verify task conditions 
echo "Verify task conditions"
if [ "`dio87c -- xtcc d1 t1`" != "5" ] ; then 
	echo "Task condition count isn't 5"; exit 1; 
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep hmin`" != "hmin!08:00!+0" ] ; then 
	echo "Can't find condition (type hmin)"; exit 1; 
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep hmax`" != "hmax!09:00!+0" ] ; then 
	echo "Can't find condition (type hmax)"; exit 1; 
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep hlate`" != "hlate!10:00!+0" ] ; then 
	echo "Can't find condition (type hlate)"; exit 1; 
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep file`" != "file!/tmp/fic" ] ; then 
	echo "Can't find condition (type file)"; exit 1; 
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep manual`" != "manual!ok" ] ; then 
	echo "Can't find condition (type manual)"; exit 1; 
fi

# verify job properties
echo "Verify jobs"
if [ "`dio87c -- xjc d1 t1`" != "4" ] ; then echo "Job count isn't 4"; exit 1; fi
for i in 1 2 3 4 ; do
	if [ "`dio87c -- xjl -S! -d d1 t1 | grep j$i`" != "j$i!job$i" ] ; then 
		echo "Can't find job j$i in task list"; exit 1; 
	fi
	if [ "`dio87c -- xji -S! d1 t1 j$i | grep Path`" != "Path!/job$i.sh" ]; then
		echo "Path is not set for job j$i"; exit 1; 
	fi
	if [ "`dio87c -- xji -S! d1 t1 j$i | grep Arg`" != "Argument!arg$i" ]; then
		echo "Argument is not set for job j$i"; exit 1; 
	fi
	if [ "`dio87c -- xji -S! d1 t1 j$i | grep \"Start Tempo\"`" != \
			"Start Tempo!$(( $i + 10 ))" ]; then
		echo "Start tempo is not set for job j$i"; exit 1; 
	fi
	if [ "`dio87c -- xji -S! d1 t1 j$i | grep \"End Tempo\"`" != \
			"End Tempo!$(( $i + 20 ))" ]; then
		echo "End tempo is not set for job j$i"; exit 1; 
	fi
done

# verify job dependencies
echo "Verify job dependencies"
if [ "`dio87c -- xjdc d1 t1`" != "3" ] ; then 
	echo "Job dependencies count isn't 3"; exit 1; 
fi
if [ "`dio87c -- xjdl -tS! d1 t1 | grep always`" != "j1!j2!always" ] ; then 
	echo "Can't find job dependencies (type always)"; exit 1; 
fi
if [ "`dio87c -- xjdl -tS! d1 t1 | grep onsuccess`" != "j2!j3!onsuccess" ] ; then 
	echo "Can't find job dependencies (type onsuccess)"; exit 1; 
fi
if [ "`dio87c -- xjdl -tS! d1 t1 | grep onerror`" != "j2!j4!onerror" ] ; then 
	echo "Can't find job dependencies (type onerror)"; exit 1; 
fi

# verify waitAllDependencies property
if [ x`dio87c -- xji d1 t1 j2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ]; then
	echo "waitAllDependencies property of job is not false"; exit 1
fi
if [ x`dio87c -- xji d1 t1 j3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ]; then
	echo "waitAllDependencies property of job is not true"; exit 1
fi

# verify OnErrorContinue property
if [ x`dio87c -- xji d1 t1 j1 | grep "On Error"| awk '{print $4;}'` != "xfalse" ]; then
	echo "OnErrorContinue property of job is not false"; exit 1
fi
if [ x`dio87c -- xji d1 t1 j4 | grep "On Error"| awk '{print $4;}'` != "xtrue" ]; then
	echo "OnErrorContinue property of job is not true"; exit 1
fi

# kill ref and day
dio87c -- rk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
dio87c -- srk r1 || exit 1

# delete target and queue
dio87c -- tk tg1 || exit 1
memd2=`dio87c -- memory`
if [ $memd2 -ne $memd1 ] ; then 
	echo "memory delta different for dio87d $memd1 -> $memd2"; exit 1
fi

stopDio87d || exit 1
