#!/bin/sh
#
# file tst05.sh 
# (test targets and queues)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Test targets and queues" 

startDio87d || exit 1

mem1=`dio87c -- memory` || exit 1

echo " check target commands"

# test the system target `localhost'
if [ "`dio87c tl | grep localhost`" != "localhost" ] ; then
	echo "Can't find the target 'localhost' !"; exit 1
fi
# try to kill the system target
dio87c tk localhost > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Should not delete the target 'localhost' !"; exit 1
fi
# check that the target 'localhost' is not deleted
if [ "`dio87c tl | grep localhost`" != "localhost" ] ; then
	echo "The target 'localhost' was deleted !"; exit 1
fi
# try to change the name of the system target
dio87c tsn localhost new_name > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Should not change the name of the target 'localhost' !"; exit 1
fi
# try to change the IP Port of the system target
dio87c tsp localhost 8888 > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Should not change the IP port of the target 'localhost' !"; exit 1
fi
# check that the target 'localhost' is not deleted
if [ "`dio87c tl | grep localhost`" != "localhost" ] ; then
	echo "The target 'localhost' was renamed!"; exit 1
fi
# try to change the hostname of the system target
dio87c tsh localhost hostname > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Should not change the hostname of the target 'localhost' !"; exit 1
fi
# try to change the description of the system target
dio87c tsn localhost newdescription > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Should not change the description of the target 'localhost' !"; exit 1
fi
# TODO try to change the IP port

dio87c -- ta "abAZ09-_"
if [ $? -ne 0 ]; then
	echo "The 'ta' function fails with legal characters !"; exit 1
fi
dio87c -- ta "ref@" > /dev/null
if [ $? -eq 0 ]; then
	echo "The 'ta' function accepts illegal characters !"; exit 1
fi
dio87c -- ta "" > /dev/null
if [ $? -eq 0 ]; then
	echo "The 'ta' function accepts an empty string !"; exit 1
fi
if [ "`dio87c -- tc`" -ne 2 ]; then
	echo "There is not 2 targets !"; exit 1
fi
dio87c -- tk "abAZ09-_" || exit 1

### tsn
dio87c -- ta t1 || exit 1
dio87c -- tsn t1 "abAZ09-_"
if [ $? -ne 0 ]; then
	echo "The 'tsn' function fails with legal characters !"; exit 1
fi
dio87c -- tsn "abAZ09-_" t1 || exit 1
dio87c -- tsn t1 "target@" > /dev/null
if [ $? -eq 0 ]; then
	echo "The 'tsn' function accepts illegal characters !"; exit 1
fi
dio87c -- tsn t1 "" > /dev/null
if [ $? -eq 0 ]; then
	echo "The 'tsn' function accepts an empty string !"; exit 1
fi

# rename the target
dio87c -- tsn t1 t2 
if [ $? -ne 0 ]; then
	echo "Can't change the name of the target !"; exit 1
fi
if [ "`dio87c tc`" != "2" ] ; then
	echo "The target count is not 2 !"; exit 1
fi
if [ "`dio87c tl | grep t2`" != "t2" ] ; then
	echo "The target was not renamed !"; exit 1
fi

dio87c -- ta t1 || exit 1
dio87c -- tsn t1 t2 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Should not change the name of the target !"; exit 1
fi
if [ "`dio87c tc`" != "3" ] ; then
	echo "The target count is not 3 !"; exit 1
fi
if [ "`dio87c tl | grep t2`" != "t2" ] ; then
	echo "Can't find the target t2 !"; exit 1
fi
if [ "`dio87c tl | grep t1`" != "t1" ] ; then
	echo "Can't find the target t1 !"; exit 1
fi

dio87c -- tk t1 || exit 1
dio87c -- tk t2 || exit 1

################################
echo " check queue commands"

# test the system queue `batch' of the target `localhost'
if [ "`dio87c ql localhost | grep batch`" != "batch" ] ; then
	echo "Can't find the queue 'batch' of the target 'localhost' !"; exit 1
fi
# try to kill the queue `batch' of the target `localhost'
dio87c qk localhost batch > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Should not delete the queue 'batch' of the target 'localhost' !"; exit 1
fi
# check that the queue `batch' of target 'localhost' is not deleted
if [ "`dio87c ql localhost | grep batch`" != "batch" ] ; then
	echo "The target 'localhost' was deleted !"; exit 1
fi
# try to change the name of the queue `batch' of the system target
dio87c qsn localhost batch new_name > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Should not change the name of the queue 'batch' of the target 'localhost' !"; exit 1
fi
# check that the queue `batch' of the target 'localhost' is not deleted
if [ "`dio87c ql localhost | grep batch`" != "batch" ] ; then
	echo "The queue 'batch' of the target 'localhost' was renamed!"; exit 1
fi
# try to change the description of queue `batch' of the target `localhost'
dio87c qsn localhost batch newdescription > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Should not change the description of the queue 'batch' of the target 'localhost' !"; exit 1
fi

dio87c -- ta target1  || exit 1
dio87c -- qa target1 "abAZ09-_"
if [ $? -ne 0 ]; then
	echo "The 'qa' function fails with legal characters !"; exit 1
fi
dio87c -- qa target1 "ref@" > /dev/null
if [ $? -eq 0 ]; then
	echo "The 'qa' function accepts illegal characters !"; exit 1
fi
dio87c -- qa target1 "" > /dev/null
if [ $? -eq 0 ]; then
	echo "The 'qa' function accepts an empty string !"; exit 1
fi
if [ "`dio87c -- qc target1`" -ne 2 ]; then
	echo "There is not 2 queues !"; exit 1
fi
dio87c -- qk target1 "abAZ09-_" || exit 1
dio87c -- tk target1  || exit 1

### qsn
dio87c -- ta t1 || exit 1
dio87c -- qa t1 q1 || exit 1
dio87c -- qsn t1 q1 "abAZ09-_"
if [ $? -ne 0 ]; then
	echo "The 'qsn' function fails with legal characters !"; exit 1
fi
dio87c -- qsn t1 "abAZ09-_" q1 || exit 1
dio87c -- qsn t1 q1 "target@" > /dev/null
if [ $? -eq 0 ]; then
	echo "The 'qsn' function accepts illegal characters !"; exit 1
fi
dio87c -- qsn t1 q1 "" > /dev/null
if [ $? -eq 0 ]; then
	echo "The 'qsn' function accepts an empty string !"; exit 1
fi

# rename the queue 
dio87c -- qsn t1 q1 q2
if [ $? -ne 0 ]; then
	echo "Can't change the name of the queue !"; exit 1
fi
if [ "`dio87c qc t1`" != "2" ] ; then
	echo "The queue count is not 2 !"; exit 1
fi
if [ "`dio87c ql t1 | grep q2`" != "q2" ] ; then
	echo "The queue was not renamed !"; exit 1
fi

dio87c -- qa t1 q1 || exit 1
dio87c -- qsn t1 q1 q2 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Should not change the name of the queue !"; exit 1
fi
if [ "`dio87c qc t1`" != "3" ] ; then
	echo "The queue count is not 3 !"; exit 1
fi
if [ "`dio87c ql t1 | grep q2`" != "q2" ] ; then
	echo "Can't find the queue q2 !"; exit 1
fi
if [ "`dio87c ql t1 | grep q1`" != "q1" ] ; then
	echo "Can't find the queue q1 !"; exit 1
fi

dio87c -- tk t1 || exit 1

# verify memory
mem2=`dio87c -- memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "memory error $mem1 -> $mem2"; exit 1
fi

stopDio87d || exit 1
