#!/bin/sh

#
# file tst07.sh 
# (test open/close property and max running jobs for target and queue */
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test open/close property and max running jobs "

startDio87d || exit 1


######################################## target open/close 
echo "Test target open/close property" 
memx1=`dio87c -- memory`
# close target 
dio87c -- tso localhost false
if [ "`dio87c -- ti localhost | grep Opened | awk '{print $2}'`" != "false" ]; then
	echo "Target is not closed."; exit 1
fi
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
# verify that a job is not running (target is closed)
waitretry 5 "dio87c -- xji d1 t1 j1 |grep State |awk '{print \$2}'" "ready_to_submit"
if [ $? -ne 0 ] ; then
	echo "Job isn't ready_to_submit."; exit 1
fi
if [ "`dio87c -- xti d1 t1 | grep State | awk '{print $2}'`" != "running" ]; then
	echo "Task isn't running."; exit 1
fi
if [ "`dio87c -- qxjc localhost batch`" != "1" ]; then
	echo "Number of running job in queue is not 1 !"; exit 1
fi
if [ "`dio87c -- qxjl localhost batch -S!`" != "d1!t1!j1" ]; then
	echo "Job isn't in running job list of the queue !"; exit 1
fi
# open the target 
dio87c -- tso localhost true 
if [ "`dio87c -- ti localhost | grep Opened | awk '{print $2}'`" != "true" ]; then
	echo "Target is not opened."; exit 1
fi
sleep 1

# verify that job is terminated
waitretry 3 "dio87c -- xji d1 t1 j1 | grep State | awk '{print \$2}'" "terminated" 
if [ $? -ne 0 ]; then
	echo "Job isn't terminated."; exit 1
fi
if [ "`dio87c -- xti d1 t1 | grep State | awk '{print $2}'`" != "terminated" ]; then
	echo "Task isn't terminated."; exit 1
fi
if [ "`dio87c -- xji d1 t1 j1 | grep \"Exit code\"| awk '{print $3}'`" != "0" ]; then
	echo "Exit code is not good."; exit 1
fi
if [ "`dio87c -- qxjc localhost batch`" != "0" ]; then
	echo "Number of running job in queue is not zero !"; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; 
	exit 1
fi

######################################## queue open/close 
echo "Test queue open/close property" 
memx1=`dio87c -- memory`
# close queue
dio87c -- qso localhost batch false
if [ "`dio87c -- qi localhost batch | grep Opened | awk '{print $2}'`" != "false" ]; then
	echo "Queue is not closed."; exit 1
fi
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# verify that job is not running
waitretry 3 "dio87c -- xji d1 t1 j1 |grep State |awk '{print \$2}'" "ready_to_submit"
if [ $? -ne 0 ]; then
	echo "Job isn't ready_to_submit."; exit 1
fi
if [ "`dio87c -- xti d1 t1 | grep State | awk '{print $2}'`" != "running" ]; then
	echo "Task isn't running."; exit 1
fi
if [ "`dio87c -- qxjc localhost batch`" != "1" ]; then
	echo "Number of running job in queue is not 1 !"; exit 1
fi
if [ "`dio87c -- qxjl localhost batch -S!`" != "d1!t1!j1" ]; then
	echo "Job isn't in running job list of the queue !"; exit 1
fi
# open the queue
dio87c -- qso localhost batch true 
if [ "`dio87c -- qi localhost batch | grep Opened | awk '{print $2}'`" != "true" ]; then
	echo "Queue is not opened."; exit 1
fi
sleep 1
# verify that job is terminated
waitretry 3 "dio87c -- xji d1 t1 j1 | grep State | awk '{print \$2}'" "terminated" 
if [ $? -ne 0 ] ; then
	echo "Job isn't terminated."; exit 1
fi
if [ "`dio87c -- xti d1 t1 | grep State | awk '{print $2}'`" != "terminated" ]; then
	echo "Task isn't terminated."; exit 1
fi
if [ "`dio87c -- xji d1 t1 j1 | grep \"Exit code\"| awk '{print $3}'`" != "0" ]; then
	echo "Exit code is not good."; exit 1
fi
if [ "`dio87c -- qxjc localhost batch`" != "0" ]; then
	echo "Number of running job in queue is not zero !"; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; 
	#exit 1
fi


######################################## queue max running job (max=1)
echo "Test maximum running jobs (max=1)" 
memx1=`dio87c -- memory`
# set maximum running jobs
dio87c -- qsm localhost batch 1 
if [ "`dio87c -- qi localhost batch | grep Max | awk '{print $2}'`" != "1" ]; then
	echo "Maximum running job isn't 1."; exit 1
fi
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job6.sh"
dio87c -- rja r1 t1 j2 || exit 1
dio87c -- rjsp r1 t1 j2 "$JOBPATH/job6.sh"
dio87c -- rja r1 t1 j3 || exit 1
dio87c -- rjsp r1 t1 j3 "$JOBPATH/job6.sh"
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
echo " Verify that first job is running"
# verify that only one job is running
waitretry 9 "dio87c -- xjl d1 t1 -s | grep running | wc -l | awk '{print \$1}'" "1"
if [ $? -ne 0 ]; then 
	echo "Number of running jobs isn't 1."; exit 1; 
fi
# verify that two jobs are waiting
waitretry 2 "dio87c -- xjl d1 t1 -s | grep ready | wc -l | awk '{print \$1}'" "2"
if [ $? -ne 0 ]; then 
	echo "Number of waiting jobs isn't 2."; exit 1; 
fi

echo " Verify that second job is running"
# verify that a job is terminated 
waitretry 5 "dio87c -- xjl d1 t1 -s | grep terminated | wc -l | awk '{print \$1}'" "1"
if [ $? -ne 0 ];
	then echo "Number of terminated jobs isn't 1."; exit 1; fi
# verify that only one job is running
waitretry 5 "dio87c -- xjl d1 t1 -s | grep running | wc -l | awk '{print \$1}'"  "1" 
if [ $? -ne 0 ] ; then 
	echo "Number of running jobs isn't 1."; exit 1; 
fi
# verify that a job is waiting
waitretry 2 "dio87c -- xjl d1 t1 -s | grep ready | wc -l | awk '{print \$1}'" "1"
if [ $? -ne 0 ];
	then echo "Number of waiting jobs isn't 1."; exit 1; fi

sleep 2

echo " Verify that third job is running"
# verify that two jobs are terminated 
waitretry 5 "dio87c -- xjl d1 t1 -s | grep terminated | wc -l | awk '{print \$1}'" "2"
if [ $? -ne 0 ];
	then echo "Number of terminated jobs isn't 2."; exit 1; fi
# verify that only one job is running
waitretry 5 "dio87c -- xjl d1 t1 -s | grep running | wc -l | awk '{print \$1}'"  "1"
if [ $? -ne 0 ] ;
	then echo "Number of running jobs isn't 1."; exit 1; fi

# wait end of task
waitretry 5 "dio87c -- xti d1 t1 | grep State | awk '{print \$2}'" "terminated"
if [ $? -ne 0 ];
	then echo "Task isn't terminated."; exit 1; fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; 
	#exit 1
fi

######################################## queue max running job (max=2)
echo "Test maximum running jobs (max=2)" 
memx1=`dio87c -- memory`
# set maximum running jobs
dio87c -- qsm localhost batch 2 
if [ "`dio87c -- qi localhost batch | grep Max | awk '{print $2}'`" != "2" ]; then
	echo "Maximum running job isn't 2."; exit 1
fi
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job6.sh"
dio87c -- rja r1 t1 j2 || exit 1
dio87c -- rjsp r1 t1 j2 "$JOBPATH/job6.sh"
dio87c -- rja r1 t1 j3 || exit 1
dio87c -- rjsp r1 t1 j3 "$JOBPATH/job6.sh"
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
echo " Verify that two jobs are running"
# verify that two jobs are running
waitretry 9 "dio87c -- xjl d1 t1 -s | grep running | wc -l | awk '{print \$1}'" "2"
if [ $? -ne 0 ]; then 
	echo "Number of running jobs isn't 2."; exit 1; 
fi
# verify that a job is waiting
if [ "`dio87c -- xjl d1 t1 -s | grep ready | wc -l | awk '{print $1}'`" != "1" ]; then
	dio87c -- xjl d1 t1 -s
	echo "Number of waiting jobs isn't 1."; exit 1; 
fi

echo " Verify that last job is running"
# verify that two jobs are terminated 
waitretry 4 "dio87c -- xjl d1 t1 -s | grep terminated | wc -l | awk '{print \$1}'" "2" 
if [ $? -ne 0 ] ; then
	echo "Number of terminated jobs isn't 2."; exit 1; 
fi
# verify that only one job is running
waitretry 2 "dio87c -- xjl d1 t1 -s | grep running | wc -l | awk '{print \$1}'" "1" 
if [ $? -ne 0 ] ; then
	echo "Number of running jobs isn't 1."; exit 1; 
fi

# wait end of task
waitretry 5 "dio87c -- xti d1 t1 | grep State | awk '{print \$2}'" "terminated" 
if [ $? -ne 0 ] ; then
	echo "Task isn't terminated."; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; 
	exit 1
fi

stopDio87d || exit 1
