#!/bin/sh

#
# file tst08.sh 
# (test task conditions)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test task conditions"

startDio87d || exit 1


######################################## manual condition 
echo "Test manual condition" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtca r1 t1 manual "OK"
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
waitretry 5 "dio87c -- xtcl d1 t1 -sS!" "manual!OK!!waiting"
if [ $? -ne 0 ] ; then
	echo "Manual condition is not waiting"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS!`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
dio87c -- xtcv d1 t1 manual "OK" || exit 1
if [ "`dio87c -- xtcl d1 t1 -sS!`" != "manual!OK!!finished" ] ; then
	echo "Manual condition is not finished"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS!`" != "t1!terminated" ]; then
	echo "Task is not terminated"; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

######################################## file condition 
echo "Test file condition" 
memx1=`dio87c -- memory`
file="$DATAPATH/fictop$$"
if [ -f $file ]; then
	echo "Ftop $file already exists"; exit 1
fi
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 none || exit 1
dio87c -- rtca r1 t1 file $file 
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
waitretry 3 "dio87c -- xtcl d1 t1 -sS!" "file!$file!!waiting" 
if [ $? -ne 0 ] ; then
	echo "File condition is not waiting"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS!`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
touch $file 
if [ $? -ne 0 ]; then
	echo "Can't create file $file"; exit 1
fi
sleep 1
if [ "`dio87c -- xtcl d1 t1 -sS!`" != "file!$file!!finished" ] ; then
	echo "File condition is not finished"; rm $file; exit 1
fi
if [ "`dio87c -- xtl d1 -sS!`" != "t1!terminated" ]; then
	echo "Task is not terminated"; rm $file; exit 1
fi
rm $file
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

######################################## hmin condition 
echo "Test hmin condition" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtca r1 t1 hmin "23:59" 0
dio87c -- rtca r1 t1 manual OK
dio87c -- rta r1 t2 || exit 1
dio87c -- rtoa r1 t2 add always || exit 1
dio87c -- rtca r1 t2 hmin "00:00" 0
dio87c -- rtca r1 t2 manual OK
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
# t1 is waiting
waitretry 3 "dio87c -- xtcl d1 t1 -sS! | grep hmin" "hmin!23:59!+0!waiting"
if [ $? -ne 0 ] ; then
	echo "HMin condition is not waiting"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS! | grep t1`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
# t2 is finished
if [ "`dio87c -- xtcl d1 t2 -sS! | grep hmin`" != "hmin!00:00!+0!finished" ] ; then
	echo "HMin condition is not finished"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS! | grep t2`" != "t2!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
# validate manual condition
dio87c -- xtcv d1 t2 manual "OK" || exit 1
sleep 1
if [ "`dio87c -- xtl d1 -sS! | grep t1`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS! | grep t2`" != "t2!terminated" ]; then
	echo "Task is not terminated"; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

######################################## hmax condition 
echo "Test hmax condition" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtca r1 t1 hmax "23:59" 0
dio87c -- rtca r1 t1 manual OK
dio87c -- rta r1 t2 || exit 1
dio87c -- rtoa r1 t2 add always || exit 1
dio87c -- rtca r1 t2 hmax "00:00" 0
dio87c -- rtca r1 t2 manual OK
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
# t1 is waiting
waitretry 3 "dio87c -- xtcl d1 t1 -sS! | grep hmax" "hmax!23:59!+0!waiting"
if [ $? -ne 0 ] ; then
	echo "HMax condition is not waiting"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS! | grep t1`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
# t2 is finished
if [ "`dio87c -- xtcl d1 t2 -sS! | grep hmax`" != "hmax!00:00!+0!finished" ] ; then
	echo "HMax condition is not finished"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS! | grep t2`" != "t2!terminated" ]; then
	echo "Task is not terminated"; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi


######################################## hlate condition 
echo "Test hlate condition" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtca r1 t1 hlate "23:59" 0
dio87c -- rtca r1 t1 manual OK
dio87c -- rta r1 t2 || exit 1
dio87c -- rtoa r1 t2 add always || exit 1
dio87c -- rtca r1 t2 hlate "00:00" 0
dio87c -- rtca r1 t2 manual OK
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
# t1 is waiting
waitretry 3 "dio87c -- xtcl d1 t1 -sS! | grep hlate" "hlate!23:59!+0!waiting"
if [ $? -ne 0 ] ; then
	echo "HLate condition is not waiting"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS! | grep t1`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
# t2 is canceled 
if [ "`dio87c -- xtcl d1 t2 -sS! | grep hlate`" != "hlate!00:00!+0!finished" ] ; then
	echo "HLate condition is not finished"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS! | grep t2`" != "t2!canceled" ]; then
	echo "Task is not canceled"; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

######################################## force a file condition 
echo "Force a file condition" 
memx1=`dio87c -- memory`
file="$DATAPATH/fictop$$"
if [ -f $file ]; then
	echo "Ftop $file already exists"; exit 1
fi
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 none || exit 1
dio87c -- rtca r1 t1 file $file 
dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
waitretry 3 "dio87c -- xtcl d1 t1 -sS!" "file!$file!!waiting"
if [ $? -ne 0 ] ; then
	echo "File condition is not waiting"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS!`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
dio87c -- xtcv d1 t1 file "$file" || exit 1
if [ "`dio87c -- xtcl d1 t1 -sS!`" != "file!$file!!finished" ] ; then
	echo "File condition is not finished"; exit 1
fi
if [ "`dio87c -- xtl d1 -sS!`" != "t1!terminated" ]; then
	echo "Task is not terminated"; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

stopDio87d || exit 1
