#!/bin/sh
#
# file tst12.sh 
# (test job tempo)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#

if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test job tempo"

startDio87d || exit 1

######################################## job start tempo
echo "Test Job start tempo" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1

dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job6.sh"
dio87c -- rjst r1 t1 j1 2 0

dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# verify job is waiting 
echo " verify that job is waiting"
waitretry 6 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!wait_start_tempo"
if [ $? -ne 0 ]; then
	echo "The job isn't waiting for start tempo."; exit 1
fi
waitretry 2 "dio87c -- xtl -sS! d1 | grep t1" "t1!running"
if [ $? -ne 0 ]; then
	echo "The task isn't running (while start tempo)."; exit 1
fi
sleep 1
# verify job is running 
echo " verify that job is running"
waitretry 4 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!running" 
if [ $? -ne 0 ]; then
	echo "The job isn't running."; exit 1
fi
waitretry 2 "dio87c -- xtl -sS! d1 | grep t1" "t1!running"
if [ $? -ne 0 ]; then
	echo "The task isn't running (while job is running)."; exit 1
fi
sleep 2
# verify job is terminated 
echo " verify that job is terminated"
waitretry 4 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!terminated" 
if [ $? -ne 0 ]; then
	echo "The job isn't terminated."; exit 1
fi
if [ $? -ne 0 ]; then
waitretry 2 "dio87c -- xtl -sS! d1 | grep t1" "t1!terminated"
	echo "The task isn't terminated."; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi


######################################## job end tempo
echo "Test Job end tempo (with normal termination)" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1

dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job6.sh"
dio87c -- rjst r1 t1 j1 0 3

dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# verify job is running 
echo " verify that job is running"
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!running" 
if [ $? -ne 0 ]; then
	echo "The job isn't running."; exit 1
fi
waitretry 4 "dio87c -- xtl -sS! d1 | grep t1" "t1!running" 
if [ $? -ne 0 ]; then
	echo "The task isn't running (while job is running)."; exit 1
fi
sleep 2
# verify job is waiting 
echo " verify that job is waiting"
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!wait_end_tempo" 
if [ $? -ne 0 ]; then
	echo "The job isn't waiting for end tempo."; exit 1
fi
waitretry 4 "dio87c -- xtl -sS! d1 | grep t1" "t1!running" 
if [ $? -ne 0 ]; then
	echo "The task isn't running (while start tempo)."; exit 1
fi
sleep 2
# verify job is terminated 
echo " verify that job is terminated"
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!terminated" 
if [ $? -ne 0 ]; then
	echo "The job isn't terminated."; exit 1
fi
waitretry 4 "dio87c -- xtl -sS! d1 | grep t1" "t1!terminated"
if [ $? -ne 0 ]; then
	echo "The task isn't terminated."; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

######################################## job end tempo with abort
echo "Test Job end tempo (with abort)" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1

dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job9.sh"
dio87c -- rjst r1 t1 j1 0 3

dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# verify job is running 
echo " verify that job is running"
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!running" 
if [ $? -ne 0 ]; then
	echo "The job isn't running."; exit 1
fi
waitretry 2 "dio87c -- xtl -sS! d1 | grep t1" "t1!running"
if [ $? -ne 0 ]; then
	echo "The task isn't running (while job is running)."; exit 1
fi
# verify job is waiting 
echo " verify that job is waiting"
waitretry 6 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!wait_end_tempo" 
if [ $? -ne 0 ]; then
	echo "The job isn't waiting for end tempo."; exit 1
fi
waitretry 2 "dio87c -- xtl -sS! d1 | grep t1" "t1!running"
if [ $? -ne 0 ]; then
	echo "The task isn't running (while start tempo)."; exit 1
fi
# verify job is aborted 
echo " verify that job is aborted"
waitretry 6 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!aborted" 
if [ $? -ne 0 ]; then
	echo "The job isn't aborted."; exit 1
fi
waitretry 2 "dio87c -- xtl -sS! d1 | grep t1" "t1!aborted" 
if [ $? -ne 0 ]; then
	echo "The task isn't aborted."; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

stopDio87d || exit 1
