#!/bin/sh
#
# file tst13.sh 
# (test xtdu and xjdu commands)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#

if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test xtdu and xjdu commands"

startDio87d || exit 1

######################################## validate a job dependence 
echo "Test manual job dependence validation" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1

dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dio87c -- rja r1 t1 j2 || exit 1
dio87c -- rjsp r1 t1 j2 "$JOBPATH/job1.sh"
dio87c -- rjda r1 t1 j1 j2 always

dio87c -- ese true || exit 1
dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1
sleep 1
dio87c -- xjse d1 t1 j1 false || exit 1
dio87c -- xdse d1 true || exit 1

# verify job is waiting 
echo " verify that jobs are waiting"
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!wait_enabled" 
if [ $? -ne 0 ]; then
	echo "The first job isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xjdl -sS! d1 t1" "j1!j2!waiting" 
if [ $? -ne 0 ]; then
	echo "The job dependence isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j2" "j2!wait_for_dep" 
if [ $? -ne 0 ]; then
	echo "The second job isn't waiting."; exit 1
fi
echo " force dependence validation"
dio87c -- xjdu d1 t1 j1 j2 always validated || exit 1
sleep 1
echo " verify that second job is terminated"
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!wait_enabled" 
if [ $? -ne 0 ]; then
	echo "The first job isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xjdl -sS! d1 t1" "j1!j2!validated" 
if [ $? -ne 0 ]; then
	echo "The job dependence isn't validated."; exit 1
fi
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j2" "j2!terminated" 
if [ $? -ne 0 ]; then
	echo "The second job isn't terminated."; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

######################################## cancel a job dependence 
echo "Test manual job dependence cancellation" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1

dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dio87c -- rja r1 t1 j2 || exit 1
dio87c -- rjsp r1 t1 j2 "$JOBPATH/job1.sh"
dio87c -- rjda r1 t1 j1 j2 always

dio87c -- ese true || exit 1
dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1
sleep 1
dio87c -- xjse d1 t1 j1 false || exit 1
dio87c -- xdse d1 true || exit 1

# verify job is waiting 
echo " verify that jobs are waiting"
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!wait_enabled" 
if [ $? -ne 0 ]; then
	echo "The first job isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xjdl -sS! d1 t1" "j1!j2!waiting"
if [ $? -ne 0 ]; then
	echo "The job dependence isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j2" "j2!wait_for_dep" 
if [ $? -ne 0 ]; then
	echo "The second job isn't waiting."; exit 1
fi
echo " force dependence cancellation"
dio87c -- xjdu d1 t1 j1 j2 always canceled || exit 1
sleep 1

echo " verify that second job is canceled"
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j1" "j1!wait_enabled" 
if [ $? -ne 0 ]; then
	echo "The first job isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xjdl -sS! d1 t1" "j1!j2!canceled"
if [ $? -ne 0 ]; then
	echo "The job dependence isn't canceled."; exit 1
fi
waitretry 9 "dio87c -- xjl -sS! d1 t1 | grep j2" "j2!canceled"
if [ $? -ne 0 ]; then
	echo "The second job isn't canceled."; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

######################################## validate a task dependence 
echo "Test manual task dependence validation" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1

dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job1.sh"

dio87c -- rta r1 t2 || exit 1
dio87c -- rtoa r1 t2 add always || exit 1
dio87c -- rtst r1 t2 localhost || exit 1
dio87c -- rja r1 t2 j1 || exit 1
dio87c -- rjsp r1 t2 j1 "$JOBPATH/job1.sh"

dio87c -- rtda r1 t1 t2 always

dio87c -- ese true || exit 1
dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1
sleep 1
dio87c -- xtse d1 t1 false || exit 1
dio87c -- xdse d1 true || exit 1

# verify task is waiting 
echo " verify that tasks are waiting"
waitretry 9 "dio87c -- xtl -sS! d1 | grep t1" "t1!wait_enabled" 
if [ $? -ne 0 ]; then
	echo "The first task isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xtdl -sS! d1" "t1!t2!waiting" 
if [ $? -ne 0 ]; then
	echo "The task dependence isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xtl -sS! d1 | grep t2" "t2!wait_for_dep"
if [ $? -ne 0 ]; then
	echo "The second task isn't waiting."; exit 1
fi

echo " force dependence validation"
dio87c -- xtdu d1 t1 t2 always validated || exit 1
sleep 1

echo " verify that second task is terminated"
waitretry 9 "dio87c -- xtl -sS! d1 | grep t1" "t1!wait_enabled" 
if [ $? -ne 0 ]; then
	echo "The first task isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xtdl -sS! d1" "t1!t2!validated" 
if [ $? -ne 0 ]; then
	echo "The task dependence isn't validated."; exit 1
fi
waitretry 9 "dio87c -- xtl -sS! d1 | grep t2" "t2!terminated" 
if [ $? -ne 0 ]; then
	echo "The second task isn't terminated."; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

######################################## cancel a task dependence 
echo "Test manual task dependence cancellation" 
memx1=`dio87c -- memory`
dio87c -- ra r1 || exit 1

dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/job1.sh"

dio87c -- rta r1 t2 || exit 1
dio87c -- rtoa r1 t2 add always || exit 1
dio87c -- rtst r1 t2 localhost || exit 1
dio87c -- rja r1 t2 j1 || exit 1
dio87c -- rjsp r1 t2 j1 "$JOBPATH/job1.sh"

dio87c -- rtda r1 t1 t2 always

dio87c -- ese true || exit 1
dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1
sleep 1
dio87c -- xtse d1 t1 false || exit 1
dio87c -- xdse d1 true || exit 1

# verify task is waiting 
echo " verify that tasks are waiting"
waitretry 9 "dio87c -- xtl -sS! d1 | grep t1" "t1!wait_enabled" 
if [ $? -ne 0 ]; then
	echo "The first task isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xtdl -sS! d1" "t1!t2!waiting"
if [ $? -ne 0 ]; then
	echo "The task dependence isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xtl -sS! d1 | grep t2" "t2!wait_for_dep"
if [ $? -ne 0 ]; then
	echo "The second task isn't waiting."; exit 1
fi

echo " force dependence cancellation"
dio87c -- xtdu d1 t1 t2 always canceled || exit 1
sleep 1

echo " verify that second task is cancelled"
waitretry 9 "dio87c -- xtl -sS! d1 | grep t1" "t1!wait_enabled" 
if [ $? -ne 0 ]; then
	echo "The first task isn't waiting."; exit 1
fi
waitretry 9 "dio87c -- xtdl -sS! d1" "t1!t2!canceled"
if [ $? -ne 0 ]; then
	echo "The task dependence isn't canceled."; exit 1
fi
waitretry 9 "dio87c -- xtl -sS! d1 | grep t2" "t2!canceled" 
if [ $? -ne 0 ]; then
	echo "The second task isn't canceled."; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

stopDio87d || exit 1


