#!/bin/sh
#
# file tst15.sh 
# (test 'deleted' state)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#

if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test 'deleted' state"

startDio87d || exit 1

######################################## 
echo "Test deleted state for job" 
memx1=`dio87c -- memory`

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1

dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/jobxxx.sh"

dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

sleep 3
waitretry 10 "dio87c -- xjl d1 t1 -sS!" "j1!deleted" 
if [ $? -ne 0 ]; then
	echo "Job is not deleted."; exit 1;
fi
waitretry 2 "dio87c -- xtl d1 -sS!" "t1!aborted"
if [ $? -ne 0 ]; then
	echo "Task is not aborted."; exit 1;
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

######################################## 
echo "Test dependencies with deleted state" 
memx1=`dio87c -- memory`

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1

dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/jobxxx.sh"

dio87c -- rja r1 t1 j2 || exit 1
dio87c -- rjsp r1 t1 j2 "$JOBPATH/job6.sh"
dio87c -- rjda r1 t1 j1 j2 always || exit 1

dio87c -- rja r1 t1 j3 || exit 1
dio87c -- rjsp r1 t1 j3 "$JOBPATH/job6.sh"
dio87c -- rjda r1 t1 j1 j3 onerror || exit 1

dio87c -- rja r1 t1 j4 || exit 1
dio87c -- rjsp r1 t1 j4 "$JOBPATH/job6.sh"
dio87c -- rjda r1 t1 j1 j4 onsuccess || exit 1

dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

waitretry 9 "dio87c -- xjl d1 t1 -sS! | grep j1" "j1!deleted"
if [ $? -ne 0 ]; then
	echo "Job1 is not deleted."; exit 1;
fi
waitretry 5 "dio87c -- xjl d1 t1 -sS! | grep j2" "j2!running"
if [ $? -ne 0 ]; then
	echo "Job2 is not running."; exit 1;
fi
waitretry 5 "dio87c -- xjl d1 t1 -sS! | grep j3" "j3!running" 
if [ $? -ne 0 ]; then
	echo "Job3 is not running."; exit 1;
fi
waitretry 5 "dio87c -- xjl d1 t1 -sS! | grep j4" "j4!canceled"
if [ $? -ne 0 ]; then
	echo "Job4 is not canceled."; exit 1;
fi
waitretry 5 "dio87c -- xtl d1 -sS! | grep t1" "t1!running"
if [ $? -ne 0 ]; then
	echo "Task1 is not running."; exit 1;
fi
waitretry 9 "dio87c -- xjl d1 t1 -sS! | grep j2" "j2!terminated" 
if [ $? -ne 0 ]; then
	echo "Job2 is not terminated."; exit 1;
fi
waitretry 9 "dio87c -- xjl d1 t1 -sS! | grep j3" "j3!terminated"
if [ $? -ne 0 ]; then
	echo "Job3 is not terminated."; exit 1;
fi
waitretry 5 "dio87c -- xtl d1 -sS! | grep t1" "t1!aborted"
if [ $? -ne 0 ]; then
	echo "Task1 is not aborted."; exit 1;
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

#############################################
memx1=`dio87c -- memory` || exit 1

echo "Test task with no queue"
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 none || exit 1

dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "/bin/sleep" || exit 1
dio87c -- rjsa r1 t1 j1 "1" || exit 1

dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

waitretry 10 "dio87c -- xji -S_ d1 t1 j1 | grep State" "State_deleted"
if [ $? -ne 0 ]; then
	echo "Job is not deleted !"; exit 1
fi
waitretry 5 "dio87c -- xti -S_ d1 t1 | grep State" "State_aborted"
if [ $? -ne 0 ]; then
	echo "Job is not deleted !"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending

# check memory
memx2=`dio87c -- memory` || exit 1
if [ $memx1 -ne $memx2 ]; then
	echo "dio87d : memory error $memx1 -> $memx2"; exit 1
fi


#############################################
memx1=`dio87c -- memory` || exit 1

echo "Test job with empty path"
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1

dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "" || exit 1

dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

waitretry 10 "dio87c -- xji -S_ d1 t1 j1 | grep State" "State_deleted"
if [ $? -ne 0 ]; then
	echo "Job is not deleted !"; exit 1
fi
waitretry 5 "dio87c -- xti -S_ d1 t1 | grep State" "State_aborted"
if [ $? -ne 0 ]; then
	echo "Job is not deleted !"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending

# check memory
memx2=`dio87c -- memory` || exit 1
if [ $memx1 -ne $memx2 ]; then
	echo "dio87d : memory error $memx1 -> $memx2"; exit 1
fi

######################################## 
echo "Test onErrorContinue with deleted state for job" 
memx1=`dio87c -- memory`

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 localhost || exit 1

dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "$JOBPATH/jobxxx.sh"
dio87c -- rjsoec r1 t1 j1 false || exit 1

dio87c -- rja r1 t1 j2 || exit 1
dio87c -- rjsp r1 t1 j2 "$JOBPATH/job6.sh"
dio87c -- rjda r1 t1 j1 j2 always || exit 1

dio87c -- ese true || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

waitretry 15 "dio87c -- xjl d1 t1 -sS! | grep j1" "j1!delete_stopped"
if [ $? -ne 0 ]; then
	echo "Job1 is not delete_stopped."; exit 1;
fi
waitretry 5 "dio87c -- xjl d1 t1 -sS! | grep j2" "j2!wait_for_dep"
if [ $? -ne 0 ]; then
	echo "Job2 is not waiting for dependence."; exit 1;
fi
waitretry 5 "dio87c -- xtl d1 -sS!" "t1!running"
if [ $? -ne 0 ]; then
	echo "Task is not running."; exit 1;
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending
memx2=`dio87c -- memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dio87d $memx1 -> $memx2"; exit 1
fi

stopDio87d || exit 1


