#!/bin/sh
#
# file tst16.sh 
# (test dio87d functions for rules and rule operations)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test dio87d functions for rules and rule operations"

startDio87d || exit 1

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

################################################## Rule 
echo "Test Rules"
dio87c -- ra r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a ref"; exit 1; fi
# add a rule
dio87c -- rra r1 rule1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a rule"; exit 1; fi
# verify ref count = 1
n=`dio87c -- rrc r1`
if [ $n != 1 ] ; then echo "Rule count isn't 1"; exit 1; fi
if [ "`dio87c -- rrl r1`" != "rule1" ]; then
	echo "Can't find rule1 in the list."; exit 1
fi
# try to add a second rule with same name
dio87c -- rra r1 rule1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add two rules with same name"; exit 1; fi
# delete rule 
dio87c -- rrk r1 rule1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the rule"; exit 1; fi
# verify rule count = 0
n=`dio87c -- rrc r1`
if [ $n != 0 ] ; then echo "Rule count isn't 0"; exit 1; fi
# try to delete a rule which doesn't exist
dio87c -- rrk r1 t1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete a rule which doesn't exist"; exit 1; fi
# delete the ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after rule $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a ref with rules 
mem1=`dio87c -- memory`
dio87c -- ra r1 || exit 1 
dio87c -- rra r1 rule1 || exit 1
dio87c -- rra r1 rule2 || exit 1
dio87c -- rra r1 rule3 || exit 1
dio87c -- rk r1 > /dev/null 2>&1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error deleting ref with rules $mem1 -> $mem2"
	exit 1
fi

################################################## rule operations 
echo "Test Rule Operations"
dio87c -- ra r1 || exit 1
dio87c -- rra r1 rule1 || exit 1 

echo " test 'always' operation"
dio87c -- rroa r1 rule1 add always
if [ $? != 0 ] ; then echo "Can't add a rule operation"; exit 1; fi
# verify ref count = 1
n=`dio87c -- rroc r1 rule1`
if [ $n != 1 ] ; then echo "Operation count isn't 1"; exit 1; fi
if [ "`dio87c -- rrol r1 rule1 -S!`" != "add!always!" ]; then
	echo "Can't find the operation in the list."; exit 1
fi
# delete operation 
dio87c -- rrok r1 rule1 1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the rule operation"; exit 1; fi
# verify rule count = 0
n=`dio87c -- rroc r1 rule1`
if [ $n != 0 ] ; then echo "Operation count isn't 0"; exit 1; fi
# try to delete a rule which doesn't exist
dio87c -- rrok r1 rule1 1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete an operation which doesn't exist"; exit 1; fi

# try to add an 'always' with an operator different from 'add'
dio87c -- rroa r1 rule2 remove always >/dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a 'always' operation with an operator different from 'add'"; exit 1; fi

# day_of_week
echo " test 'day_of_week' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rroa r1 rule1 $oper day_of_week 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper day_of_week' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "$oper!day_of_week!1" ] ; then
		echo "Can't find the '$oper day_of_week' operation."
		exit 1; 
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done
# wrong parameters
for value in 0 8 ; do
	dio87c -- rroa r1 rule1 add day_of_week $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'day_of_week' $value"; exit 1
	fi
	if [ "`dio87c -- rroc r1 rule1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 7 ; do
	dio87c -- rroa r1 rule1 add day_of_week $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'day_of_week' $value"; exit 1
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "add!day_of_week!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done


# day_of_month
echo " test 'day_of_month' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rroa r1 rule1 $oper day_of_month 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper day_of_month' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "$oper!day_of_month!1" ] ; then
		echo "Can't find the '$oper day_of_month' operation."
		exit 1; 
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done
# wrong parameters
for value in 0 32 ; do
	dio87c -- rroa r1 rule1 add day_of_month $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'day_of_month' $value"; exit 1
	fi
	if [ "`dio87c -- rroc r1 rule1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 31 ; do
	dio87c -- rroa r1 rule1 add day_of_month $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'day_of_month' $value"; exit 1
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "add!day_of_month!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done



# day_of_year
echo " test 'day_of_year' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rroa r1 rule1 $oper day_of_year 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper day_of_year' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "$oper!day_of_year!1" ] ; then
		echo "Can't find the '$oper day_of_year' operation."
		exit 1; 
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done
# wrong parameters
for value in 0 367 -367; do
	dio87c -- rroa -- r1 rule1 add day_of_year $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'day_of_year ' $value"; exit 1
	fi
	if [ "`dio87c -- rroc r1 rule1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 366 -1 -366 ; do
	dio87c -- rroa -- r1 rule1 add day_of_year $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'day_of_year ' $value"; exit 1
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "add!day_of_year!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done



# week_of_month 
echo " test 'week_of_month' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rroa r1 rule1 $oper week_of_month 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper week_of_month' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "$oper!week_of_month!1" ] ; then
		echo "Can't find the '$oper week_of_month' operation."
		exit 1; 
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done
# wrong parameters
for value in 0 6 -6; do
	dio87c -- rroa -- r1 rule1 add week_of_month $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'week_of_month' $value"; exit 1
	fi
	if [ "`dio87c -- rroc r1 rule1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 5 -1 -5 ; do
	dio87c -- rroa -- r1 rule1 add week_of_month $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'week_of_month' $value"; exit 1
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "add!week_of_month!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done

# week_of_year
echo " test 'week_of_year' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rroa r1 rule1 $oper week_of_year 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper week_of_year' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "$oper!week_of_year!1" ] ; then
		echo "Can't find the '$oper week_of_year' operation."
		exit 1; 
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done
# wrong parameters
for value in 0 54 -54; do
	dio87c -- rroa -- r1 rule1 add week_of_year $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'week_of_year' $value"; exit 1
	fi
	if [ "`dio87c -- rroc r1 rule1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 53 -1 -53; do
	dio87c -- rroa -- r1 rule1 add week_of_year $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'week_of_year ' $value"; exit 1
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "add!week_of_year!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done



# month_of_year
echo " test 'month_of_year' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rroa r1 rule1 $oper month_of_year 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper month_of_year' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "$oper!month_of_year!1" ] ; then
		echo "Can't find the '$oper month_of_year' operation."
		exit 1; 
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done
# wrong parameters
for value in 0 13; do
	dio87c -- rroa r1 rule1 add month_of_year $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'month_of_year' $value"; exit 1
	fi
	if [ "`dio87c -- rroc r1 rule1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 12; do
	dio87c -- rroa r1 rule1 add month_of_year $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'month_of_year ' $value"; exit 1
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "add!month_of_year!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done


# day 
echo " test 'day' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rroa r1 rule1 $oper day "11-20"
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper day' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "$oper!day!11-20" ] ; then
		echo "Can't find the '$oper day' operation."
		exit 1; 
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done
# wrong parameters
for value in "" "01x01" "01-00" "01-32" "00-01" "13-01" "1-1" "1-01" "01-1"; do
	dio87c -- rroa r1 rule1 add day "$value" > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'day' $value"; exit 1
	fi
	if [ "`dio87c -- rroc r1 rule1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in "01-01" "12-01" "01-31" "12-31"; do
	dio87c -- rroa r1 rule1 add day "$value"
	if [ $? != 0 ] ; then
		echo "Can't add a 'day' $value"; exit 1
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "add!day!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done



# date
echo " test 'date' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rroa r1 rule1 $oper date "2003-11-20"
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper date' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "$oper!date!2003-11-20" ] ; then
		echo "Can't find the '$oper date' operation."
		exit 1; 
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done
# wrong parameters
for value in "" "2003/01/01" "0000-01-01" "2003-00-01" "2003-13-01" "2003-01-00" "2003-01-32"; do
	dio87c -- rroa r1 rule1 add date $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'date' $value"; exit 1
	fi
	if [ "`dio87c -- rroc r1 rule1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in "2003-01-01" "2003-12-01" "2003-01-31" "2003-12-31"; do
	dio87c -- rroa r1 rule1 add date "$value"
	if [ $? != 0 ] ; then
		echo "Can't add a 'date' $value"; exit 1
	fi
	if [ "`dio87c -- rrol r1 rule1 -S!`" != "add!date!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	if [ "`dio87c -- rroc r1 rule1`" != "1" ] ; then
		echo "Operation count is not 1."; exit 1
	fi
	dio87c -- rrok r1 rule1 1 || exit 1
done

# delete the ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after rule $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a ref with rules 
mem1=`dio87c -- memory`
dio87c -- ra r1 || exit 1 
dio87c -- rra r1 rule1 || exit 1
dio87c -- rra r1 rule2 || exit 1
dio87c -- rra r1 rule3 || exit 1
dio87c -- rk r1 > /dev/null 2>&1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error deleting ref with rules $mem1 -> $mem2"
	exit 1
fi


################################################## circular reference 
echo "Test circular reference detection for complex rules"

echo " step 1"
dio87c -- ra r1 > /dev/null 2>&1 || exit 1
dio87c -- rra r1 rule1 || exit 1
# a rule can't be used by itself
dio87c -- rroa r1 rule1 add rule rule1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Circular reference is not detected."; exit 1
fi
dio87c -- rk r1 > /dev/null 2>&1

echo " step 2"
dio87c -- ra r1 > /dev/null 2>&1 || exit 1
# add three rules
dio87c -- rra r1 rule1 || exit 1
dio87c -- rra r1 rule2 || exit 1
dio87c -- rra r1 rule3 || exit 1
# second rule depends on first rule
dio87c -- rroa r1 rule2 add rule rule1 || exit 1
# third rule depends on second rule
dio87c -- rroa r1 rule3 add rule rule2 || exit 1
# verify that first rule can't depend on third rule
dio87c -- rroa r1 rule1 add rule rule3 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Circular reference is not detect."; exit 1
fi
# delete the ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after rule $mem1 -> $mem2"
	exit 1
fi

################################################## Rule deletion
echo "Test rule deletion"
dio87c -- ra r1 > /dev/null 2>&1 || exit 1
dio87c -- rra r1 rule1 || exit 1
dio87c -- rra r1 rule2 || exit 1
dio87c -- rra r1 rule3 || exit 1
# second rule depends on first rule
dio87c -- rroa r1 rule2 add rule rule1 || exit 1
# third rule depends on second rule
dio87c -- rroa r1 rule3 add rule rule2 || exit 1
# try to delete the first rule
dio87c -- rrk r1 rule1 > /dev/null 2>&1
if [ $? -eq 0 ] ; then 
	echo "Can delete a rule which is used by another."; exit 1; 
fi
# verify that rule is always in list
if [ "`dio87c -- rrl r1 | grep rule1`" != "rule1" ]; then
	echo "Rule isn't in the list."; exit 1
fi
# delete a rule which isn't used by another
dio87c -- rrk r1 rule3
if [ $? -ne 0 ] ; then
	echo "Can't delete rule which isn't used by another."; exit 1;
fi
# try again to delete the first rule
dio87c -- rrk r1 rule1 > /dev/null 2>&1
if [ $? -eq 0 ] ; then 
	echo "Can delete a rule which is used by another."; exit 1; 
fi
# delete the second rule
dio87c -- rrk r1 rule2 || exit 1
# verify that the first rule can be delete
dio87c -- rrk r1 rule1
if [ $? -ne 0 ] ; then 
	echo "Can't delete a rule which should be unused."; exit 1; 
fi

# delete the ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after rule $mem1 -> $mem2"
	exit 1
fi
################################################## Rule operation deletion
echo "Test rule operation deletion"
dio87c -- ra r1 > /dev/null 2>&1 || exit 1
dio87c -- rra r1 rule1 || exit 1
dio87c -- rra r1 rule2 || exit 1
dio87c -- rra r1 rule3 || exit 1
# second rule depends on first rule
dio87c -- rroa r1 rule2 add rule rule1 || exit 1
# third rule depends on second rule
dio87c -- rroa r1 rule3 add rule rule1 || exit 1
# try to delete the first rule
dio87c -- rrk r1 rule1 > /dev/null 2>&1
if [ $? -eq 0 ] ; then 
	echo "Can delete a rule which is used by another."; exit 1; 
fi
# delete operation of second rule
dio87c -- rrok r1 rule2 1 
if [ $? -ne 0 ] ; then
	echo "Can't delete rule operation"; exit 1;
fi
# try to delete the first rule
dio87c -- rrk r1 rule1 > /dev/null 2>&1 
if [ $? -eq 0 ] ; then 
	echo "Can delete a rule which is used by another."; exit 1; 
fi
# delete operation of third rule
dio87c -- rrok r1 rule3 1 
if [ $? -ne 0 ] ; then
	echo "Can't delete rule operation"; exit 1;
fi
# try to delete the first rule
dio87c -- rrk r1 rule1
if [ $? -ne 0 ] ; then 
	echo "Can't delete a rule which should be unused."; exit 1; 
fi

# delete the ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after rule $mem1 -> $mem2"
	exit 1
fi


stopDio87d || exit 1
