#!/bin/sh
#
# file tst18.sh 
# (test dio87d functions rtoa rtok rtol rtoc)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied wartanty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test dio87d functions rtoa rtok rtol rtoc"

startDio87d || exit 1

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

################################################## task operations 
echo "Test Task Operations"
dio87c -- ra r1 || exit 1
dio87c -- rta r1 task1 || exit 1 

echo " test 'always' operation"
dio87c -- rtoa r1 task1 add always
if [ $? != 0 ] ; then echo "Can't add a task operation"; exit 1; fi
# verify ref count = 1
n=`dio87c -- rtoc r1 task1`
if [ $n != 1 ] ; then echo "Operation count isn't 1"; exit 1; fi
if [ "`dio87c -- rtol r1 task1 -S!`" != "add!always!" ]; then
	echo "Can't find the operation in the list."; exit 1
fi
# delete operation 
dio87c -- rtok r1 task1 1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the task operation"; exit 1; fi
# verify task count = 0
n=`dio87c -- rtoc r1 task1`
if [ $n != 0 ] ; then echo "Operation count isn't 0"; exit 1; fi
# try to delete a task which doesn't exist
dio87c -- rtok r1 task1 1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete an operation which doesn't exist"; exit 1; fi

# try to add an 'always' with an operator different from 'add'
dio87c -- rtoa r1 task2 remove always >/dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add a 'always' operation with an operator different from 'add'"; exit 1; fi

# day_of_week
echo " test 'day_of_week' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rtoa r1 task1 $oper day_of_week 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper day_of_week' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "$oper!day_of_week!1" ] ; then
		echo "Can't find the '$oper day_of_week' operation."
		exit 1; 
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done
# wrong parameters
for value in 0 8 ; do
	dio87c -- rtoa r1 task1 add day_of_week $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'day_of_week' $value"; exit 1
	fi
	if [ "`dio87c -- rtoc r1 task1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 7 ; do
	dio87c -- rtoa r1 task1 add day_of_week $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'day_of_week' $value"; exit 1
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "add!day_of_week!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done


# day_of_month
echo " test 'day_of_month' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rtoa r1 task1 $oper day_of_month 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper day_of_month' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "$oper!day_of_month!1" ] ; then
		echo "Can't find the '$oper day_of_month' operation."
		exit 1; 
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done
# wrong parameters
for value in 0 32 ; do
	dio87c -- rtoa r1 task1 add day_of_month $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'day_of_month' $value"; exit 1
	fi
	if [ "`dio87c -- rtoc r1 task1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 31 ; do
	dio87c -- rtoa r1 task1 add day_of_month $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'day_of_month' $value"; exit 1
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "add!day_of_month!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done



# day_of_year
echo " test 'day_of_year' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rtoa r1 task1 $oper day_of_year 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper day_of_year' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "$oper!day_of_year!1" ] ; then
		echo "Can't find the '$oper day_of_year' operation."
		exit 1; 
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done
# wrong parameters
for value in 0 367 -367; do
	dio87c -- rtoa -- r1 task1 add day_of_year $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'day_of_year ' $value"; exit 1
	fi
	if [ "`dio87c -- rtoc r1 task1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 366 -1 -366 ; do
	dio87c -- rtoa -- r1 task1 add day_of_year $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'day_of_year ' $value"; exit 1
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "add!day_of_year!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done



# week_of_month 
echo " test 'week_of_month' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rtoa r1 task1 $oper week_of_month 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper week_of_month' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "$oper!week_of_month!1" ] ; then
		echo "Can't find the '$oper week_of_month' operation."
		exit 1; 
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done
# wrong parameters
for value in 0 6 -6; do
	dio87c -- rtoa -- r1 task1 add week_of_month $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'week_of_month' $value"; exit 1
	fi
	if [ "`dio87c -- rtoc r1 task1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 5 -1 -5 ; do
	dio87c -- rtoa -- r1 task1 add week_of_month $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'week_of_month' $value"; exit 1
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "add!week_of_month!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done

# week_of_year
echo " test 'week_of_year' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rtoa r1 task1 $oper week_of_year 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper week_of_year' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "$oper!week_of_year!1" ] ; then
		echo "Can't find the '$oper week_of_year' operation."
		exit 1; 
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done
# wrong parameters
for value in 0 54 -54; do
	dio87c -- rtoa -- r1 task1 add week_of_year $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'week_of_year' $value"; exit 1
	fi
	if [ "`dio87c -- rtoc r1 task1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 53 -1 -53; do
	dio87c -- rtoa -- r1 task1 add week_of_year $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'week_of_year ' $value"; exit 1
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "add!week_of_year!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done



# month_of_year
echo " test 'month_of_year' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rtoa r1 task1 $oper month_of_year 1
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper month_of_year' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "$oper!month_of_year!1" ] ; then
		echo "Can't find the '$oper month_of_year' operation."
		exit 1; 
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done
# wrong parameters
for value in 0 13; do
	dio87c -- rtoa r1 task1 add month_of_year $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'month_of_year' $value"; exit 1
	fi
	if [ "`dio87c -- rtoc r1 task1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in 1 12; do
	dio87c -- rtoa r1 task1 add month_of_year $value
	if [ $? != 0 ] ; then
		echo "Can't add a 'month_of_year ' $value"; exit 1
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "add!month_of_year!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done


# day 
echo " test 'day' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rtoa r1 task1 $oper day "11-20"
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper day' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "$oper!day!11-20" ] ; then
		echo "Can't find the '$oper day' operation."
		exit 1; 
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done
# wrong parameters
for value in "" "01x01" "01-00" "01-32" "00-01" "13-01" "1-1" "1-01" "01-1"; do
	dio87c -- rtoa r1 task1 add day "$value" > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'day' $value"; exit 1
	fi
	if [ "`dio87c -- rtoc r1 task1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in "01-01" "12-01" "01-31" "12-31"; do
	dio87c -- rtoa r1 task1 add day "$value"
	if [ $? != 0 ] ; then
		echo "Can't add a 'day' $value"; exit 1
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "add!day!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done



# date
echo " test 'date' operation"
for oper in add add_from add_until remove remove_from remove_until
do
	dio87c -- rtoa r1 task1 $oper date "2003-11-20"
	if [ $? != 0 ] ; then 
		echo "Can't add a '$oper date' operation"; 
		exit 1; 
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "$oper!date!2003-11-20" ] ; then
		echo "Can't find the '$oper date' operation."
		exit 1; 
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done
# wrong parameters
for value in "" "2003/01/01" "0000-01-01" "2003-00-01" "2003-13-01" "2003-01-00" "2003-01-32"; do
	dio87c -- rtoa r1 task1 add date $value > /dev/null 2>&1
	if [ $? = 0 ] ; then
		echo "Can add a 'date' $value"; exit 1
	fi
	if [ "`dio87c -- rtoc r1 task1`" != "0" ] ; then
		echo "Operation count is not zero."; exit 1
	fi
done
# good parameters
for value in "2003-01-01" "2003-12-01" "2003-01-31" "2003-12-31"; do
	dio87c -- rtoa r1 task1 add date "$value"
	if [ $? != 0 ] ; then
		echo "Can't add a 'date' $value"; exit 1
	fi
	if [ "`dio87c -- rtol r1 task1 -S!`" != "add!date!$value" ]; then
		echo "Can't find operation"; exit 1
	fi
	if [ "`dio87c -- rtoc r1 task1`" != "1" ] ; then
		echo "Operation count is not 1."; exit 1
	fi
	dio87c -- rtok r1 task1 1 || exit 1
done

# delete the ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after task $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a ref with tasks 
mem1=`dio87c -- memory`
dio87c -- ra r1 || exit 1 
dio87c -- rta r1 task1 || exit 1
dio87c -- rta r1 task2 || exit 1
dio87c -- rta r1 task3 || exit 1
dio87c -- rk r1 > /dev/null 2>&1
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error deleting ref with tasks $mem1 -> $mem2"
	exit 1
fi


################################################## Task deletion
echo "Test task deletion"
dio87c -- ra r1 > /dev/null 2>&1 || exit 1
dio87c -- rta r1 task1 || exit 1
dio87c -- rra r1 rule1 || exit 1

# rule depends of task 
dio87c -- rtoa r1 task1 add rule rule1 || exit 1

# try to delete the rule 
dio87c -- rrk r1 rule1 > /dev/null 2>&1
if [ $? -eq 0 ] ; then 
	echo "Can delete a task which is used by another."; exit 1; 
fi
# verify that Rule is always in list
if [ "`dio87c -- rrl r1 | grep rule1`" != "rule1" ]; then
	echo "Rule isn't in the list."; exit 1
fi
# delete the task
dio87c -- rtk r1 task1 || exit 1
# verify that the Rule can be delete
dio87c -- rrk r1 rule1
if [ $? -ne 0 ] ; then 
	echo "Can't delete a rule which should be unused."; exit 1; 
fi

# delete the ref
dio87c -- rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error before and after task $mem1 -> $mem2"
	exit 1
fi

stopDio87d || exit 1
