#!/bin/sh
#
# file tst25.sh 
# (test variable for dio87d) 
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied wartanty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test variable for dio87d"

startDio87d || exit 1

################################################## 
echo "Test variables commands " 

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

# add variable 
dio87c -- va var1
if [ $? -ne 0 ]; then echo "Can't add a variable."; exit 1; fi
if [ "`dio87c -- vc`" -ne 1 ] ; then
	echo "Variables count isn't 1."; exit 1
fi
if [ "`dio87c -- vl | grep var1 | wc -l`" -ne 1 ]; then
	echo "Can't find 'var1' variable."; exit 1
fi
# add another variable with the same name
dio87c -- va var1 > /dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a second variable with the same name !"; exit 1;fi
if [ "`dio87c -- vc`" -ne 1 ] ; then
	echo "Variable count isn't 1."; exit 1
fi
# add a second variable 
dio87c -- va var2
if [ $? -ne 0 ]; then echo "Can't add a second variable."; exit 1; fi
if [ "`dio87c -- vc`" -ne 2 ] ; then
	echo "Variables count isn't 2."; exit 1
fi
if [ "`dio87c -- vl | grep var2 | wc -l`" -ne 1 ]; then
	echo "Can't find 'var2' variable."; exit 1
fi

# delete variable
dio87c -- vk var1
if [ $? -ne 0 ]; then echo "Can't delete variable."; exit 1; fi
if [ "`dio87c -- vc`" -ne 1 ] ; then
	echo "Users count isn't 1."; exit 1
fi
dio87c -- vk var2 || exit 1

# add a variable with an initial value
dio87c -- va var1 value1
if [ $? -ne 0 ]; then echo "Can't add a variable with initial value."; exit 1; fi
if [ "`dio87c -- vc`" -ne 1 ] ; then
	echo "Variables count isn't 1."; exit 1
fi
if [ "`dio87c -- vl -vS! | grep var1`" != "var1!value1" ]; then
	echo "Can't find 'var1' variable with initial value."; exit 1
fi
if [ "`dio87c -- vi -S! var1 | grep Value`" != "Value!value1" ]; then
	echo "Can't find 'var1' initial value."; exit 1
fi

# change value of initial variable
dio87c -- vsv var1 value2
if [ $? -ne 0 ]; then echo "Can't change variable value."; exit 1; fi
if [ "`dio87c -- vi -S! var1 | grep Value`" != "Value!value2" ]; then
	echo "Can't find 'var1' value."; exit 1
fi
dio87c -- vk var1 || exit 1

# verify memory
mem2=`dio87c -- memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_equal rule operations " 

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dio87c -- ra ref || exit 1
dio87c -- va v1 || exit 1
dio87c -- rra ref r1 || exit 1

# try to add a 'if_equal' operation whith unknown variable
dio87c -- rroa ref r1 add if_equal vxxx 33 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can add an 'if_equal' operation with an unknown variable !"
	exit 1
fi
# add a 'if_equal' operation
dio87c -- rroa ref r1 add if_equal v1 33 
if [ "`dio87c -- rrol ref r1 -S! | grep v1`" != "add!if_equal!v1!33" ]; then
	echo "Can't find if_equal operation"; exit 1
fi
# verify that variable is linked
if [ "`dio87c -- vi v1 -S! | grep Rules`" != "Linked Rules!1" ]; then
	echo "Linked rules is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dio87c -- vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with rules !"; exit 1
fi
# verify that variable is not deleted
if [ "`dio87c -- vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dio87c -- rrok ref r1 1
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_equal' operation !"; exit 1
fi
# verify that variable is not linked
if [ "`dio87c -- vi v1 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dio87c -- vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with rules !"; exit 1
fi
# verify that variable is deleted
if [ "`dio87c -- vl | grep v1`" = "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dio87c -- rk ref || exit 1

# verify that variable is unlinked after referential deletion
dio87c -- va v2 || exit 1
dio87c -- ra ref || exit 1
dio87c -- rra ref r1 || exit 1
dio87c -- rroa ref r1 add if_equal v2 5 || exit 1
dio87c -- rra ref r2 || exit 1
dio87c -- rroa ref r2 add if_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dio87c -- vi v2 -S! | grep Rules`" != "Linked Rules!2" ]; then
	echo "Linked rules is not 2 for v2 variable"; exit 1
fi
dio87c -- rk ref || exit 1
# verify that variable is not linked
if [ "`dio87c -- vi v2 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v2 variable"; exit 1
fi
dio87c -- vk v2 || exit 1


# test 'if_equal' operation with 'add' and 'remove' operators
dio87c -- va v1 || exit 1
dio87c -- ra ref || exit 1

echo "  test 'add' operator"
dio87c -- rra ref r1 || exit 1
dio87c -- rroa ref r1 add if_equal v1 OK || exit 1
dio87c -- vsv v1 NOK || exit 1
if [ "`dio87c -- rrsr ref r1 \"2004-01-01\"`" != "N" ] ; then
	echo "Rule is not unselected !"; exit 1
fi
dio87c -- vsv v1 OK || exit 1
if [ "`dio87c -- rrsr ref r1 \"2004-01-01\"`" != "Y" ] ; then
	echo "Rule is not selected !"; exit 1
fi

echo "  test 'remove' operator"
dio87c -- rra ref r2 || exit 1
dio87c -- rroa ref r2 add always || exit 1
dio87c -- rroa ref r2 remove if_equal v1 OK || exit 1
dio87c -- vsv v1 NOK || exit 1
if [ "`dio87c -- rrsr ref r2 \"2004-01-01\"`" != "Y" ] ; then
	echo "Rule is not selected !"; exit 1
fi
dio87c -- vsv v1 OK || exit 1
if [ "`dio87c -- rrsr ref r2 \"2004-01-01\"`" != "N" ] ; then
	echo "Rule is not unselected !"; exit 1
fi
dio87c -- rk ref || exit 1
dio87c -- vk v1 || exit 1

# verify memory
mem2=`dio87c -- memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_not_equal rule operations " 

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dio87c -- ra ref || exit 1
dio87c -- va v1 || exit 1
dio87c -- rra ref r1 || exit 1

# try to add a 'if_not_equal' operation whith unknown variable
dio87c -- rroa ref r1 add if_not_equal vxxx 33 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can add an 'if_not_equal' operation with an unknown variable !"
	exit 1
fi
# add a 'if_not_equal' operation
dio87c -- rroa ref r1 add if_not_equal v1 33 
if [ "`dio87c -- rrol ref r1 -S! | grep v1`" != "add!if_not_equal!v1!33" ]; then
	echo "Can't find if_not_equal operation"; exit 1
fi
# verify that variable is linked
if [ "`dio87c -- vi v1 -S! | grep Rules`" != "Linked Rules!1" ]; then
	echo "Linked rules is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dio87c -- vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with rules !"; exit 1
fi
# verify that variable is not delete
if [ "`dio87c -- vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dio87c -- rrok ref r1 1
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_not_equal' operation !"; exit 1
fi
# verify that variable is not linked
if [ "`dio87c -- vi v1 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dio87c -- vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with rules !"; exit 1
fi
# verify that variable is deleted
if [ "`dio87c -- vl | grep v1`" = "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dio87c -- rk ref || exit 1

# verify that variable is unlinked after referential deletion
dio87c -- va v2 || exit 1
dio87c -- ra ref || exit 1
dio87c -- rra ref r1 || exit 1
dio87c -- rroa ref r1 add if_not_equal v2 5 || exit 1
dio87c -- rra ref r2 || exit 1
dio87c -- rroa ref r2 add if_not_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dio87c -- vi v2 -S! | grep Rules`" != "Linked Rules!2" ]; then
	echo "Linked rules is not 2 for v2 variable"; exit 1
fi
dio87c -- rk ref || exit 1
# verify that variable is not linked
if [ "`dio87c -- vi v2 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v2 variable"; exit 1
fi
dio87c -- vk v2 || exit 1

# test 'if_not_equal' operation with 'add' and 'remove' operators
dio87c -- va v1 || exit 1
dio87c -- ra ref || exit 1

echo "  test 'add' operator"
dio87c -- rra ref r1 || exit 1
dio87c -- rroa ref r1 add if_not_equal v1 OK || exit 1
dio87c -- vsv v1 NOK || exit 1
if [ "`dio87c -- rrsr ref r1 \"2004-01-01\"`" != "Y" ] ; then
	echo "Rule is not selected !"; exit 1
fi
dio87c -- vsv v1 OK || exit 1
if [ "`dio87c -- rrsr ref r1 \"2004-01-01\"`" != "N" ] ; then
	echo "Rule is not unselected !"; exit 1
fi

echo "  test 'remove' operator"
dio87c -- rra ref r2 || exit 1
dio87c -- rroa ref r2 add always || exit 1
dio87c -- rroa ref r2 remove if_not_equal v1 OK || exit 1
dio87c -- vsv v1 NOK || exit 1
if [ "`dio87c -- rrsr ref r2 \"2004-01-01\"`" != "N" ] ; then
	echo "Rule is not unselected !"; exit 1
fi
dio87c -- vsv v1 OK || exit 1
if [ "`dio87c -- rrsr ref r2 \"2004-01-01\"`" != "Y" ] ; then
	echo "Rule is not selected !"; exit 1
fi
dio87c -- rk ref || exit 1
dio87c -- vk v1 || exit 1


# verify memory
mem2=`dio87c -- memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_equal task operations " 

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dio87c -- ra ref || exit 1
dio87c -- va v1 || exit 1
dio87c -- rta ref t1 || exit 1

# try to add a 'if_equal' operation whith unknown variable
dio87c -- rtoa ref t1 add if_equal vxxx 33 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can add an 'if_equal' operation with an unknown variable !"
	exit 1
fi
# add a 'if_equal' operation
dio87c -- rtoa ref t1 add if_equal v1 33 
if [ "`dio87c -- rtol ref t1 -S! | grep v1`" != "add!if_equal!v1!33" ]; then
	echo "Can't find if_equal operation"; exit 1
fi
# verify that variable is linked
if [ "`dio87c -- vi v1 -S! | grep Rules`" != "Linked Rules!1" ]; then
	echo "Linked rules is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dio87c -- vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with tasks !"; exit 1
fi
# verify that variable is not delete
if [ "`dio87c -- vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dio87c -- rtok ref t1 1
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_equal' operation !"; exit 1
fi
# verify that variable is not linked
if [ "`dio87c -- vi v1 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dio87c -- vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with tasks !"; exit 1
fi
# verify that variable is deleted
if [ "`dio87c -- vl | grep v1`" = "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dio87c -- rk ref || exit 1

# verify that variable is unlinked after referential deletion
dio87c -- va v2 || exit 1
dio87c -- ra ref || exit 1
dio87c -- rta ref t1 || exit 1
dio87c -- rtoa ref t1 add if_equal v2 5 || exit 1
dio87c -- rta ref t2 || exit 1
dio87c -- rtoa ref t2 add if_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dio87c -- vi v2 -S! | grep Rules`" != "Linked Rules!2" ]; then
	echo "Linked rules is not 2 for v2 variable"; exit 1
fi
dio87c -- rk ref || exit 1
# verify that variable is not linked
if [ "`dio87c -- vi v2 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked Rules is not 0 for v2 variable"; exit 1
fi
dio87c -- vk v2 || exit 1


# test 'if_equal' operation with 'add' and 'remove' operators
dio87c -- va v1 || exit 1
dio87c -- ra ref || exit 1

echo "  test 'add' operator"
dio87c -- rta ref t1 || exit 1
dio87c -- rtoa ref t1 add if_equal v1 OK || exit 1
dio87c -- vsv v1 NOK || exit 1
if [ "`dio87c -- rtsr ref t1 \"2004-01-01\"`" != "N" ] ; then
	echo "Tasks is not unselected !"; exit 1
fi
dio87c -- vsv v1 OK || exit 1
if [ "`dio87c -- rtsr ref t1 \"2004-01-01\"`" != "Y" ] ; then
	echo "Task is not selected !"; exit 1
fi

echo "  test 'remove' operator"
dio87c -- rta ref t2 || exit 1
dio87c -- rtoa ref t2 add always || exit 1
dio87c -- rtoa ref t2 remove if_equal v1 OK || exit 1
dio87c -- vsv v1 NOK || exit 1
if [ "`dio87c -- rtsr ref t2 \"2004-01-01\"`" != "Y" ] ; then
	echo "Task is not selected !"; exit 1
fi
dio87c -- vsv v1 OK || exit 1
if [ "`dio87c -- rtsr ref t2 \"2004-01-01\"`" != "N" ] ; then
	echo "Task is not unselected !"; exit 1
fi
dio87c -- rk ref || exit 1
dio87c -- vk v1 || exit 1

# verify memory
mem2=`dio87c -- memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_not_equal task operations " 

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dio87c -- ra ref || exit 1
dio87c -- va v1 || exit 1
dio87c -- rta ref t1 || exit 1

# try to add a 'if_not_equal' operation whith unknown variable
dio87c -- rtoa ref t1 add if_not_equal vxxx 33 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can add an 'if_not_equal' operation with an unknown variable !"
	exit 1
fi
# add a 'if_not_equal' operation
dio87c -- rtoa ref t1 add if_not_equal v1 33 
if [ "`dio87c -- rtol ref t1 -S! | grep v1`" != "add!if_not_equal!v1!33" ]; then
	echo "Can't find if_not_equal operation"; exit 1
fi
# verify that variable is linked
if [ "`dio87c -- vi v1 -S! | grep Rules`" != "Linked Rules!1" ]; then
	echo "Linked rules is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dio87c -- vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with tasks !"; exit 1
fi
# verify that variable is not delete
if [ "`dio87c -- vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dio87c -- rtok ref t1 1
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_not_equal' operation !"; exit 1
fi
# verify that variable is not linked
if [ "`dio87c -- vi v1 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dio87c -- vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with tasks !"; exit 1
fi
# verify that variable is deleted
if [ "`dio87c -- vl | grep v1`" = "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dio87c -- rk ref || exit 1

# verify that variable is unlinked after referential deletion
dio87c -- va v2 || exit 1
dio87c -- ra ref || exit 1
dio87c -- rta ref t1 || exit 1
dio87c -- rtoa ref t1 add if_not_equal v2 5 || exit 1
dio87c -- rta ref t2 || exit 1
dio87c -- rtoa ref t2 add if_not_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dio87c -- vi v2 -S! | grep Rules`" != "Linked Rules!2" ]; then
	echo "Linked rules is not 2 for v2 variable"; exit 1
fi
dio87c -- rk ref || exit 1
# verify that variable is not linked
if [ "`dio87c -- vi v2 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked Rules is not 0 for v2 variable"; exit 1
fi
dio87c -- vk v2 || exit 1


# test 'if_not_equal' operation with 'add' and 'remove' operators
dio87c -- va v1 || exit 1
dio87c -- ra ref || exit 1

echo "  test 'add' operator"
dio87c -- rta ref t1 || exit 1
dio87c -- rtoa ref t1 add if_not_equal v1 OK || exit 1
dio87c -- vsv v1 NOK || exit 1
if [ "`dio87c -- rtsr ref t1 \"2004-01-01\"`" != "Y" ] ; then
	echo "Tasks is not selected !"; exit 1
fi
dio87c -- vsv v1 OK || exit 1
if [ "`dio87c -- rtsr ref t1 \"2004-01-01\"`" != "N" ] ; then
	echo "Task is not unselected !"; exit 1
fi

echo "  test 'remove' operator"
dio87c -- rta ref t2 || exit 1
dio87c -- rtoa ref t2 add always || exit 1
dio87c -- rtoa ref t2 remove if_not_equal v1 OK || exit 1
dio87c -- vsv v1 NOK || exit 1
if [ "`dio87c -- rtsr ref t2 \"2004-01-01\"`" != "N" ] ; then
	echo "Task is not unselected !"; exit 1
fi
dio87c -- vsv v1 OK || exit 1
if [ "`dio87c -- rtsr ref t2 \"2004-01-01\"`" != "Y" ] ; then
	echo "Task is not selected !"; exit 1
fi
dio87c -- rk ref || exit 1
dio87c -- vk v1 || exit 1

# verify memory
mem2=`dio87c -- memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_equal condition " 

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dio87c -- va v1 || exit 1
dio87c -- ra ref || exit 1
dio87c -- rta ref tsk || exit 1

# try to add a condition with an unknown variable
dio87c -- rtca ref tsk if_equal vxx OK  > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Can add an 'if_equal' condition with an unknown variable !"
	exit 1
fi
# add an 'if_equal' condition 
dio87c -- rtca ref tsk if_equal v1 OK 
if [ $? -ne 0 ] ; then
	echo "Can't add an 'if_equal' condition !"; exit 1
fi
if [ "`dio87c -- rtcl ref tsk -S! | grep v1`" != "if_equal!v1!OK" ]; then
	echo "Can't find if_equal condition"; exit 1
fi
# verify that variable is linked
if [ "`dio87c -- vi v1 -S! | grep \"Ref Conditions\"`" != "Linked Ref Conditions!1" ]; then
	echo "Linked conditions is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dio87c -- vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with tasks !"; exit 1
fi
# verify that variable is not delete
if [ "`dio87c -- vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dio87c -- rtck ref tsk if_equal v1 OK 
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_equal' condition!"; exit 1
fi
# verify that variable is not linked
if [ "`dio87c -- vi v1 -S! | grep \"Ref Conditions\"`" != "Linked Ref Conditions!0" ]; then
	echo "Linked conditions is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dio87c -- vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with tasks !"; exit 1
fi
# verify that variable is deleted
if [ "`dio87c -- vl | grep v1`" = "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dio87c -- rk ref || exit 1

# verify that variable is unlinked after referential deletion
dio87c -- va v2 || exit 1
dio87c -- ra ref || exit 1
dio87c -- rta ref t1 || exit 1
dio87c -- rtca ref t1 if_equal v2 5 || exit 1
dio87c -- rta ref t2 || exit 1
dio87c -- rtca ref t2 if_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dio87c -- vi v2 -S! | grep \"Ref Conditions\"`" != "Linked Ref Conditions!2" ]; then
	echo "Linked conditions is not 2 for v2 variable"; exit 1
fi
dio87c -- rk ref || exit 1
# verify that variable is not linked
if [ "`dio87c -- vi v2 -S! | grep \"Ref Conditions\"`" != "Linked Ref Conditions!0" ]; then
	echo "Linked Conditions is not 0 for v2 variable"; exit 1
fi
dio87c -- vk v2 || exit 1


# verify memory
mem2=`dio87c -- memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_not_equal condition " 

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dio87c -- va v1 || exit 1
dio87c -- ra ref || exit 1
dio87c -- rta ref tsk || exit 1

# try to add a condition with an unknown variable
dio87c -- rtca ref tsk if_not_equal vxx OK  > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Can add an 'if_not_equal' condition with an unknown variable !"
	exit 1
fi
# add an 'if_not_equal' condition 
dio87c -- rtca ref tsk if_not_equal v1 OK 
if [ $? -ne 0 ] ; then
	echo "Can't add an 'if_not_equal' condition !"; exit 1
fi
if [ "`dio87c -- rtcl ref tsk -S! | grep v1`" != "if_not_equal!v1!OK" ]; then
	echo "Can't find if_not_equal condition"; exit 1
fi
# verify that variable is linked
if [ "`dio87c -- vi v1 -S! | grep \"Ref Conditions\"`" != "Linked Ref Conditions!1" ]; then
	echo "Linked conditions is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dio87c -- vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with tasks !"; exit 1
fi
# verify that variable is not delete
if [ "`dio87c -- vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dio87c -- rtck ref tsk if_not_equal v1 OK 
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_not_equal' condition!"; exit 1
fi
# verify that variable is not linked
if [ "`dio87c -- vi v1 -S! | grep \"Ref Conditions\"`" != "Linked Ref Conditions!0" ]; then
	echo "Linked conditions is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dio87c -- vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with tasks !"; exit 1
fi
# verify that variable is deleted
if [ "`dio87c -- vl | grep v1`" = "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dio87c -- rk ref || exit 1

# verify that variable is unlinked after referential deletion
dio87c -- va v2 || exit 1
dio87c -- ra ref || exit 1
dio87c -- rta ref t1 || exit 1
dio87c -- rtca ref t1 if_not_equal v2 5 || exit 1
dio87c -- rta ref t2 || exit 1
dio87c -- rtca ref t2 if_not_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dio87c -- vi v2 -S! | grep \"Ref Conditions\"`" != "Linked Ref Conditions!2" ]; then
	echo "Linked conditions is not 2 for v2 variable"; exit 1
fi
dio87c -- rk ref || exit 1
# verify that variable is not linked
if [ "`dio87c -- vi v2 -S! | grep \"Ref Conditions\"`" != "Linked Ref Conditions!0" ]; then
	echo "Linked Conditions is not 0 for v2 variable"; exit 1
fi
dio87c -- vk v2 || exit 1

################################################## 
echo "Test referential duplicate" 

# dio87d : test 
mem1=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dio87c -- va v1 || exit 1
dio87c -- va v2 || exit 1
dio87c -- ra refori || exit 1
# rule operations
dio87c -- rra refori rule1 || exit 1
dio87c -- rroa refori rule1 add if_equal v1 OK || exit 1
dio87c -- rroa refori rule1 add if_not_equal v2 OK || exit 1
dio87c -- rra refori rule2 || exit 1
dio87c -- rroa refori rule2 remove if_equal v1 OK || exit 1
dio87c -- rroa refori rule2 remove if_not_equal v2 OK || exit 1
# task operations
dio87c -- rta refori task1 || exit 1
dio87c -- rtoa refori task1 add if_equal v1 OK || exit 1
dio87c -- rtoa refori task1 add if_not_equal v2 OK || exit 1
dio87c -- rta refori task2 || exit 1
dio87c -- rtoa refori task2 remove if_equal v1 OK || exit 1
dio87c -- rtoa refori task2 remove if_not_equal v2 OK || exit 1
# task conditions
dio87c -- rta refori task3 || exit 1
dio87c -- rtca refori task3 if_equal v1 OK || exit 1
dio87c -- rtca refori task3 if_not_equal v2 OK || exit 1

dio87c -- rd refori refdup 
if [ $? -ne 0 ]; then
	echo "Can't duplicate referential !"; exit 1
fi
if [ "`dio87c -- rl | grep refdup`" != "refdup" ]; then
	echo "Can't find Duplicate referential !"; exit 1
fi
# verify rules copy
echo "  verify rule copy"
for r in rule1 rule2 ; do
	if [ "`dio87c -- rrl refdup | grep $r`" != "$r" ]; then
		echo "Can't find '$r' rule in duplicate referential !"; exit 1
	fi
	if [ "`dio87c -- rrol refdup $r | grep v1`" != "`dio87c -- rrol refori $r | grep v1`" ]; then
		echo "Error in rule copy !" ; exit 1
	fi
	if [ "`dio87c -- rrol refdup $r | grep v2`" != "`dio87c -- rrol refori $r | grep v2`" ]; then
		echo "Error in rule copy !" ; exit 1
	fi
done
# verify task operation copy
echo "  verify task operations copy"
for r in task1 task2 task3; do
	if [ "`dio87c -- rtl refdup | grep $r`" != "$r" ]; then
		echo "Can't find '$r' task in duplicate referential !"; exit 1
	fi
	if [ "`dio87c -- rtol refdup $r | grep v1`" != "`dio87c -- rtol refori $r | grep v1`" ]; then
		echo "Error in task operation copy !" ; exit 1
	fi
	if [ "`dio87c -- rtol refdup $r | grep v2`" != "`dio87c -- rtol refori $r | grep v2`" ]; then
		echo "Error in task operation copy !" ; exit 1
	fi
done

# verify task condition copy
echo "  verify task conditions copy"
if [ "`dio87c -- rtcl refdup task3 | grep v1`" != "`dio87c -- rtcl refori task3 | grep v1`" ]; then
	echo "Error in task condition copy !" ; exit 1
fi
if [ "`dio87c -- rtcl refdup task3 | grep v2`" != "`dio87c -- rtcl refori task3 | grep v2`" ]; then
	echo "Error in task condition copy !" ; exit 1
fi

# verify variable informations
for v in v1 v2; do
	if [ "`dio87c -- vi -S! $v | grep Rules`" != "Linked Rules!8" ]; then
		echo "Linked rules is not 8 !"; exit 1
	fi
	if [ "`dio87c -- vi -S! $v | grep \"Ref Conditions\"`" != "Linked Ref Conditions!2" ]; then
		echo "Linked conditions is not 2 !"; exit 1
	fi
done

dio87c -- rk refori || exit 1
dio87c -- rk refdup || exit 1
# verify variable informations
for v in v1 v2; do
	if [ "`dio87c -- vi -S! $v | grep Rules`" != "Linked Rules!0" ]; then
		echo "Linked rules is not 0 !"; exit 1
	fi
	if [ "`dio87c -- vi -S! $v | grep \"Ref Conditions\"`" != "Linked Ref Conditions!0" ]; then
		echo "Linked conditions is not 0 !"; exit 1
	fi
done
dio87c -- vk v1 || exit 1
dio87c -- vk v2 || exit 1

# verify memory
mem2=`dio87c -- memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

stopDio87d || exit 1
