#!/bin/sh
#
# file tst30.sh 
# (test SIGHUP signal)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#




if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test SIGHUP signal"

tmppidfile="$DATAPATH/dg87.pid"
tmppidfile2="$DATAPATH/dg87.pid2"
tmpconffile="$DATAPATH/dg87.conf"

#trap "rm $tmpconffile $tmppidfile $tmppidfile2 2>/dev/null" SIGINT

#################################### Verify dynamic change of IP Port

echo "Test dynamic change of IP port of dio87d"

# create original configuration file
echo "  create configuration file"
cat > $tmpconffile <<EOF
IPPORT=9009
PID_FILE=$tmppidfile
DATA_DIR=$DATAPATH
SAVE_TEMPO=300
LOG_LEVEL=ERROR
EOF

# start daemon
echo "  start dio87d on IP port 9009"
rm $DATAPATH/dio87d.data >/dev/null 2>&1
$PathDio87d -C $tmpconffile -I >/dev/null 2>&1
$PathDio87d -dC $tmpconffile
sleep 2

# verify PID file exists
if [ ! -f $tmppidfile ] ; then
	echo "PID file '$tmppidfile' doesn't exist !"; exit 1
fi

# verify communication 
echo "  verify communication on IP port 9009"
$PathDio87c -P 9009 -- version > /dev/null 2>&1
if [ $? -ne 0 ] ; then
	echo "Can't communicate with dio87d on IP port 9009"; exit 1
fi
if [ "`$PathDio87c -P 9009 -- ei -S! | grep '^IP port'`" != "IP port!9009" ] ; then
	echo "IP Port isn't 9009"; exit 1
fi


# change IP Port
echo "  change IP Port (9109)"
cat > $tmpconffile <<EOF
IPPORT=9109
PID_FILE=$tmppidfile
DATA_DIR=$DATAPATH
SAVE_TEMPO=300
LOG_LEVEL=ERROR
EOF

# send SIGHUP signal
echo "  send SIGHUP signal"
kill -s HUP `cat $tmppidfile` || exit 1
sleep 3

# verify new IP port
echo "  verify communication on new IP port"
$PathDio87c -P 9109 -- version > /dev/null 2>&1
if [ $? -ne 0 ] ; then
	echo "Can't communicate with dio87d on IP port 9109"; exit 1
fi
if [ "`$PathDio87c -P 9109 -- ei -S! | grep '^IP port'`" != "IP port!9109" ] ; then
	echo "IP Port isn't 9109"; exit 1
fi


echo "  stop dio87d"
$PathDio87c -P 9109 -- shutdown > /dev/null || exit 1

rm $tmpconffile $tmppidfile 2>/dev/null


#################################### Verify dynamic change of PID file 


echo "Test dynamic change of PID file of dio87d"

# remove previous PID files
rm $tmppidfile $tmppidfile2 2>/dev/null 

# create original configuration file
echo "  create configuration file"
cat > $tmpconffile <<EOF
IPPORT=9009
PID_FILE=$tmppidfile
DATA_DIR=$DATAPATH
SAVE_TEMPO=300
LOG_LEVEL=ERROR
EOF
# start daemon
echo "  start dio87d on IP port 9009"
rm $DATAPATH/dio87d.data >/dev/null 2>&1
$PathDio87d -C $tmpconffile -I >/dev/null 2>&1
$PathDio87d -dC $tmpconffile
sleep 2

echo "  verify communication"
$PathDio87c -P 9009 -- version > /dev/null || exit 1
if [ "`$PathDio87c -P 9009 -- ei -S! | grep '^Pid file'`" != "Pid file!$tmppidfile" ] ; then
echo "PID file is not '$tmppidfile'"; exit 1
fi

# verify PID file exists
if [ ! -f $tmppidfile ] ; then
	echo "PID file '$tmppidfile' doesn't exist !"; exit 1
fi

# change PID file 
echo "  change PID file"
cat > $tmpconffile <<EOF
IPPORT=9009
DATA_DIR=$DATAPATH
SAVE_TEMPO=300
PID_FILE=$tmppidfile2
LOG_LEVEL=ERROR
EOF

# send SIGHUP signal
echo "  send SIGHUP signal"
kill -s HUP `cat $tmppidfile` || exit 1
sleep 2

echo "  verify communication"
$PathDio87c -P 9009 -- version > /dev/null 2>&1 || exit 1
if [ "`$PathDio87c -P 9009 -- ei -S! | grep '^Pid file'`" != "Pid file!$tmppidfile2" ] ; then
echo "PID file is not '$tmppidfile2'"; exit 1
fi


# verify new PID file exists
echo "  verify PID files"
if [ ! -f $tmppidfile2 ] ; then
	echo "New PID file '$tmppidfile2' doesn't exist !"; exit 1
fi
if [ -f $tmppidfile ] ; then
	echo "Old PID file '$tmppidfile' is not deleted !"; exit 1
fi

echo "  stop dio87d"
$PathDio87c -P 9009 -- shutdown > /dev/null || exit 1

rm $tmpconffile $tmppidfile $tmppidfile2 2>/dev/null

exit 0
