#!/bin/sh
#
# file tst36.sh 
# (test the rtom and rrom commands)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon


echo "test the rtom and rrom commands"

startDio87d || exit 1
memd1=`dio87c -- memory` || exit 1

dio87c ra ref1 || exit 1

########## rrom command
echo " test the rrom command"
dio87c rra ref1 rul1 || exit 1
dio87c rroa ref1 rul1 add day_of_week 1 || exit 1
dio87c rroa ref1 rul1 add day_of_week 2 || exit 1

if [ "`dio87c -- rrol -S_ ref1 rul1 | head -n1`" != "add_day_of_week_1" ]; then
	echo "step 1) the first operation is not good !"; exit 1
fi
if [ "`dio87c -- rrol -S_ ref1 rul1 | tail -n1`" != "add_day_of_week_2" ]; then
	echo "step 1) the last operation is not good !"; exit 1
fi

# rrom : move down
dio87c rrom ref1 rul1 1 down || exit 1
if [ "`dio87c -- rrol -S_ ref1 rul1 | head -n1`" != "add_day_of_week_2" ]; then
	echo "step 2) the first operation is not good !"; exit 1
fi
if [ "`dio87c -- rrol -S_ ref1 rul1 | tail -n1`" != "add_day_of_week_1" ]; then
	echo "step 2) the last operation is not good !"; exit 1
fi

# rrom : move up
dio87c rrom ref1 rul1 2 up || exit 1
if [ "`dio87c -- rrol -S_ ref1 rul1 | head -n1`" != "add_day_of_week_1" ]; then
	echo "step 3) the first operation is not good !"; exit 1
fi
if [ "`dio87c -- rrol -S_ ref1 rul1 | tail -n1`" != "add_day_of_week_2" ]; then
	echo "step 3) the last operation is not good !"; exit 1
fi

# rrom : check the delta parameter control
dio87c rrom ref1 rul1 2 incorrect >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An incorrect value for the delta parameter is not detected !"; exit 1
fi

# rrom : check the index control
dio87c rrom ref1 rul1 0 up >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An index which is out of bound, is not detected !"; exit 1
fi
dio87c rrom ref1 rul1 3 up >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An index which is out of bound, is not detected !"; exit 1
fi

# rrom : check the day and task names control
dio87c rrom refxxx rul1 1 up >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An incorrect day name is not detected !"; exit 1 
fi
dio87c rrom ref1 rulxxx 1 up >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An incorrect task name is not detected !"; exit 1 
fi

dio87c rrk ref1 rul1 || exit 1

########## rtom command
echo " test the rtom command"
dio87c rta ref1 tsk1 || exit 1
dio87c rtoa ref1 tsk1 add day_of_week 1 || exit 1
dio87c rtoa ref1 tsk1 add day_of_week 2 || exit 1

if [ "`dio87c -- rtol -S_ ref1 tsk1 | head -n1`" != "add_day_of_week_1" ]; then
	echo "step 1) the first operation is not good !"; exit 1
fi
if [ "`dio87c -- rtol -S_ ref1 tsk1 | tail -n1`" != "add_day_of_week_2" ]; then
	echo "step 1) the last operation is not good !"; exit 1
fi

# rtom : move down
dio87c rtom ref1 tsk1 1 down || exit 1
if [ "`dio87c -- rtol -S_ ref1 tsk1 | head -n1`" != "add_day_of_week_2" ]; then
	echo "step 2) the first operation is not good !"; exit 1
fi
if [ "`dio87c -- rtol -S_ ref1 tsk1 | tail -n1`" != "add_day_of_week_1" ]; then
	echo "step 2) the last operation is not good !"; exit 1
fi

# rtom : move up
dio87c rtom ref1 tsk1 2 up || exit 1
if [ "`dio87c -- rtol -S_ ref1 tsk1 | head -n1`" != "add_day_of_week_1" ]; then
	echo "step 3) the first operation is not good !"; exit 1
fi
if [ "`dio87c -- rtol -S_ ref1 tsk1 | tail -n1`" != "add_day_of_week_2" ]; then
	echo "step 3) the last operation is not good !"; exit 1
fi

# rtom : check the delta parameter control
dio87c rtom ref1 tsk1 2 incorrect >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An incorrect value for the delta parameter is not detected !"; exit 1
fi

# rtom : check the index control
dio87c rtom ref1 tsk1 0 up >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An index which is out of bound, is not detected !"; exit 1
fi
dio87c rtom ref1 tsk1 3 up >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An index which is out of bound, is not detected !"; exit 1
fi

# rrom : check the day and rule names control
dio87c rtom refxxx tsk1 1 up >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An incorrect day name is not detected !"; exit 1 
fi
dio87c rtom ref1 tskxxx 1 up >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "An incorrect rule name is not detected !"; exit 1 
fi

dio87c rtk ref1 tsk1 || exit 1

dio87c rk ref1 || exit 1

# check memory
memd2=`dio87c -- memory` || exit 1
if [ $memd1 -ne $memd2 ]; then
	echo "dio87d : memory error $memd1 -> $memd2"; exit 1
fi

# stop the two daemons
stopDio87d || exit 1

exit 0

