#!/bin/sh
#
# file tst41.sh 
# (test task enabled property in dio87d)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Test task enabled property in dio87d"

startDio87d || exit 1

dio87c -- ra ref1 || exit 1
dio87c -- rta ref1 task1 || exit 1
dio87c -- rtoa ref1 task1 add always || exit 1
dio87c -- rta ref1 task2 || exit 1
dio87c -- rtoa ref1 task2 add always || exit 1


# task2 enabled
echo " test 'rp' command with task enabled"
if [ "`dio87c -- rti -S_ ref1 task1 | grep \"Enabled\"`" != "Enabled_true" ] ; then
	echo "Task1 is not enabled"; exit 1
fi
if [ "`dio87c -- rti -S_ ref1 task2 | grep \"Enabled\"`" != "Enabled_true" ] ; then
	echo "Task2 is not enabled"; exit 1
fi

# test `rs' command
if [ "`dio87c -- rs ref1 \"2004-09-21\" | wc -l | awk '{print $1}'`" -ne "2" ] ; then
	echo "Can't find two tasks with 'rs' command !"; exit 1
fi
if [ "`dio87c -- rs ref1 \"2004-09-21\" | grep task1`" != "task1" ] ; then
	echo "Can't find task1 with 'rs' command !"; exit 1
fi
if [ "`dio87c -- rs ref1 \"2004-09-21\" | grep task2`" != "task2" ] ; then
	echo "Can't find task2 with 'rs' command !"; exit 1
fi

# test `rd' command
dio87c -- rp ref1 "$dayDate" d1 >/dev/null || exit 1
sleep 1
if [ "`dio87c -- xtc d1`" != "2" ] ; then
	echo "Can't find two tasks in d1 !"; exit 1
fi
if [ "`dio87c -- xtl d1 | grep task1`" != "task1" ] ; then
	echo "Can't find task1 in d1 !"; exit 1
fi
if [ "`dio87c -- xtl d1 | grep task2`" != "task2" ] ; then
	echo "Can't find task2 in d1 !"; exit 1
fi

# task2 disabled
echo " test 'rp' command with task disabled"
dio87c -- rtse ref1 task2 false || exit 1

if [ "`dio87c -- rti -S_ ref1 task1 | grep \"Enabled\"`" != "Enabled_true" ] ; then
	echo "Task1 is not enabled"; exit 1
fi
if [ "`dio87c -- rti -S_ ref1 task2 | grep \"Enabled\"`" != "Enabled_false" ] ; then
	echo "Task2 is not disabled"; exit 1
fi

# test `rs' command
if [ "`dio87c -- rs ref1 \"2004-09-21\" | wc -l | awk '{print $1}'`" -ne "1" ] ; then
	echo "Can't find one tasks with 'rs' command !"; exit 1
fi
if [ "`dio87c -- rs ref1 \"2004-09-21\" | grep task1`" != "task1" ] ; then
	echo "Can't find task1 with 'rs' command !"; exit 1
fi
if [ "`dio87c -- rs ref1 \"2004-09-21\" | grep task2`" = "task2" ] ; then
	echo "Can find task1 with 'rs' command !"; exit 1
fi

# test `rd' command
dio87c -- rp ref1 "$dayDate" d2 >/dev/null || exit 1
sleep 1
if [ "`dio87c -- xtc d2`" != "1" ] ; then
	echo "Can't find one task in d2 !"; exit 1
fi
if [ "`dio87c -- xtl d2 | grep task1`" != "task1" ] ; then
	echo "Can't find task1 in d2 !"; exit 1
fi
if [ "`dio87c -- xtl d2 | grep task2`" = "task2" ] ; then
	echo "Can find task2 in d2 !"; exit 1
fi

# task2 re-enabled
echo " test 'rp' command with task re-enabled"
dio87c -- rtse ref1 task2 true || exit 1
if [ "`dio87c -- rti -S_ ref1 task2 | grep \"Enabled\"`" != "Enabled_true" ] ; then
	echo "Task2 is not enabled"; exit 1
fi

# test `rs' command
if [ "`dio87c -- rs ref1 \"2004-09-21\" | wc -l | awk '{print $1}'`" -ne "2" ] ; then
	echo "Can't find two tasks with 'rs' command !"; exit 1
fi
if [ "`dio87c -- rs ref1 \"2004-09-21\" | grep task1`" != "task1" ] ; then
	echo "Can't find task1 with 'rs' command !"; exit 1
fi
if [ "`dio87c -- rs ref1 \"2004-09-21\" | grep task2`" != "task2" ] ; then
	echo "Can't find task2 with 'rs' command !"; exit 1
fi

# test `rd' command
dio87c -- rp ref1 "$dayDate" d3 >/dev/null || exit 1
sleep 1
if [ "`dio87c -- xtc d3`" != "2" ] ; then
	echo "Can't find two tasks in d3 !"; exit 1
fi
if [ "`dio87c -- xtl d3 | grep task1`" != "task1" ] ; then
	echo "Can't find task1 in d3 !"; exit 1
fi
if [ "`dio87c -- xtl d3 | grep task2`" != "task2" ] ; then
	echo "Can't find task2 in d3 !"; exit 1
fi

echo " test 'rtsy' and 'rtsr' commands"

dio87c -- rtse ref1 task2 true || exit 1
dio87c -- rtsy ref1 task2 2004 > /dev/null
if [ $? -ne 0 ]; then
	echo "rtsy command returns an error."; exit 1
fi
if [ "`dio87c -- rtsr ref1 task2 \"2004-09-11\"`" != "Y" ] ; then
	echo "rtsr command returns an error."; exit 1
fi

dio87c -- rtse ref1 task2 false || exit 1
dio87c -- rtsy ref1 task2 2004 > /dev/null
if [ $? -eq 0 ]; then
	echo "rtsy command doesn't return an error."; exit 1
fi

dio87c -- rtsr ref1 task2 "2004-09-11" > /dev/null
if [ $? -eq 0 ]; then
	echo "rtsr command doesn't return an error."; exit 1
fi

stopDio87d || exit 1

