#!/bin/sh
#
# file tst42.sh 
# (test cyclical tasks)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Test cyclical tasks"

startDio87d || exit 1

memd1=`dio87c -- memory` || exit 1

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rta r1 t2 || exit 1
dio87c -- rtoa r1 t2 add always || exit 1

echo " check 'rtsc' command"

# check default values
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Iterations\"`" != "Iterations_1" ] ; then
	echo "Default value for 'Iterations' property is not 1"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Interval\"`" != "Interval_0" ] ; then
	echo "Default value for 'Interval' property is not 0"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"From\"`" != "From_begin" ] ; then
	echo "Default value for 'From' property is not 'begin'"; exit 1
fi

# change value
dio87c -- rtsc r1 t1 22 33 end || exit 1
dio87c -- rtsc r1 t2 unlimited || exit 1

# check -1
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Iterations\"`" != "Iterations_22" ] ; then
	echo "Value for 'Iterations' property was not changed"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Interval\"`" != "Interval_33" ] ; then
	echo "Value for 'Interval' property was not changed"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"From\"`" != "From_end" ] ; then
	echo "Value for 'From' property was not changed"; exit 1
fi

# check t2
if [ "`dio87c -- rti -S_ r1 t2 | grep \"Iterations\"`" != "Iterations_unlimited" ] ; then
	echo "Value for 'Iterations' property was not changed"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t2 | grep \"Interval\"`" != "Interval_0" ] ; then
	echo "Value for 'Interval' is not 0"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t2 | grep \"From\"`" != "From_begin" ] ; then
	echo "Value for 'From' property is not 'begin'"; exit 1
fi

# create a new day
echo " check 'rp' command"
dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1
sleep 1

if [ "`dio87c -- xdl | grep d1`" != "d1" ] ; then
	echo "Can't find 'd1' in dio87d"; exit 1
fi

# check t1
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Iterations\"`" != "Iterations_22" ] ; then
	echo "Value for 'Iterations' property was not changed"; exit 1
fi
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Interval\"`" != "Interval_33" ] ; then
	echo "Value for 'Interval' property was not changed"; exit 1
fi
if [ "`dio87c -- xti -S_ d1 t1 | grep \"From\"`" != "From_end" ] ; then
	echo "Value for 'From' property was not changed"; exit 1
fi

# check t2
if [ "`dio87c -- xti -S_ d1 t2 | grep \"Iterations\"`" != "Iterations_unlimited" ] ; then
	echo "Value for 'Iterations' property is not 'unlimited'"; exit 1
fi
if [ "`dio87c -- xti -S_ d1 t2 | grep \"Interval\"`" != "Interval_0" ] ; then
	echo "Value for 'Interval' is not 0"; exit 1
fi
if [ "`dio87c -- xti -S_ d1 t2 | grep \"From\"`" != "From_begin" ] ; then
	echo "Value for 'From' property is not 'begin'"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

# check memory
memd2=`dio87c -- memory` || exit 1
if [ $memd1 -ne $memd2 ]; then
	echo "dio87d : memory error $memd1 -> $memd2"; exit 1
fi

#############################
echo " check cyclical task with no temporization"

memx1=`dio87c -- memory` || exit 1

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "/bin/sleep" || exit 1
dio87c -- rjsa r1 t1 j1 "4" || exit 1

# 2 cycles 
dio87c -- rtsc r1 t1 3 || exit 1

dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# first execution
echo "   - first execution"
waitretry 9 "dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration\"" "Actual Iteration_1"
if [ $? -ne 0 ]; then
	echo "Task is not in first iteration"; exit 1
fi
if [ "`dio87c -- etm -rS_ | grep \"R_d1_t1_running\" | wc -l`" -ne 1 ] ; then
	echo "Task is not running"; exit 1
fi
sleep 3

# second execution
echo "   - second execution"
waitretry 9 "dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration\"" "Actual Iteration_2"
if [ $? -ne 0 ]; then
	echo "Task is not in second iteration"; exit 1
fi
if [ "`dio87c -- etm -rS_ | grep \"R_d1_t1_running\" | wc -l`" -ne 1 ] ; then
	echo "Task is not running"; exit 1
fi
sleep 3

# third execution
echo "   - third execution"
waitretry 9 "dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration\"" "Actual Iteration_3"
if [ $? -ne 0 ]; then
	echo "Task is not in third iteration"; exit 1
fi
if [ "`dio87c -- etm -rS_ | grep \"R_d1_t1_running\" | wc -l`" -ne 1 ] ; then
	echo "Task is not running"; exit 1
fi
sleep 3

waitretry 9 "dio87c -- etm -tS_" "T_d1_t1_terminated"
if [ $? -ne 0 ]; then
	echo "Task is not terminated"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending

# check memory
memx2=`dio87c -- memory` || exit 1
if [ $memx1 -ne $memx2 ]; then
	echo "dio87d : memory error $memx1 -> $memx2"; exit 1
fi


#############################
echo " check cyclical task with unlimited iterations"

memx1=`dio87c -- memory` || exit 1

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "/bin/sleep" || exit 1
dio87c -- rjsa r1 t1 j1 "4" || exit 1

# unlimited cycle 
dio87c -- rtsc r1 t1 unlimited || exit 1

dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# first execution
echo "   - first execution"
waitretry 9 "dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration\"" "Actual Iteration_1"
if [ $? -ne 0 ]; then
	echo "Task is not in first iteration"; exit 1
fi
if [ "`dio87c -- etm -rS_ | grep \"R_d1_t1_running\" | wc -l`" -ne 1 ] ; then
	echo "Task is not running"; exit 1
fi
sleep 3

# second execution
echo "   - second execution"
waitretry 9 "dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration\"" "Actual Iteration_2"
if [ $? -ne 0 ]; then
	echo "Task is not in second iteration"; exit 1
fi
if [ "`dio87c -- etm -rS_ | grep \"R_d1_t1_running\" | wc -l`" -ne 1 ] ; then
	echo "Task is not running"; exit 1
fi
sleep 3

# third execution
echo "   - third execution"
waitretry 9 "dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration\"" "Actual Iteration_3"
if [ $? -ne 0 ]; then
	echo "Task is not in third iteration"; exit 1
fi
if [ "`dio87c -- etm -rS_ | grep \"R_d1_t1_running\" | wc -l`" -ne 1 ] ; then
	echo "Task is not running"; exit 1
fi

# disable task
dio87c -- xtse d1 t1 false || exit 1
waitretry 9 "dio87c -- ejm -wS_" "W_d1_t1_j1_uninitialized"
if [ $? -ne 0 ]; then
	echo "Job is not uninitialized"; exit 1
fi
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending

# check memory
memx2=`dio87c -- memory` || exit 1
if [ $memx1 -ne $memx2 ]; then
	echo "dio87d : memory error $memx1 -> $memx2"; exit 1
fi

#############################
echo " check cyclical task with temporization"

memx1=`dio87c -- memory` || exit 1

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "/bin/sleep" || exit 1
dio87c -- rjsa r1 t1 j1 "4" || exit 1

# 2 cycles with 4 seconds of temporizations
dio87c -- rtsc r1 t1 2 4 end || exit 1

dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# first execution
waitretry 9 "dio87c -- etm -rS_" "R_d1_t1_running"
if [ $? -ne 0 ]; then
	echo "Task is not runing"; exit 1
fi
echo "   - first execution"
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration_1\" | wc -l`" -ne 1 ]; then
	echo "Actual iteration is not 1"; exit 1
fi
sleep 2

waitretry 9 "dio87c -- etm -wS_" "W_d1_t1_wait_tempo"
if [ $? -ne 0 ]; then
	echo "Task is not waiting"; exit 1
fi
echo "   - first temporization"
sleep 2

waitretry 9 "dio87c -- etm -rS_" "R_d1_t1_running"
if [ $? -ne 0 ]; then
	echo "Task is not runing"; exit 1
fi
echo "   - second execution"
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration_2\" | wc -l`" -ne 1 ]; then
	echo "Actual iteration is not 2"; exit 1
fi
sleep 2

waitretry 9 "dio87c -- etm -wS_" "W_d1_t1_wait_tempo"
if [ $? -ne 0 ]; then
	echo "Task is not waiting"; exit 1
fi
echo "   - second temporization"
sleep 2

waitretry 9 "dio87c -- etm -tS_" "T_d1_t1_terminated"
if [ $? -ne 0 ]; then
	echo "Task is not terminated"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending

# check memory
memx2=`dio87c -- memory` || exit 1
if [ $memx1 -ne $memx2 ]; then
	echo "dio87d : memory error $memx1 -> $memx2"; exit 1
fi

#############################
echo " check cyclical task with start condition"

memx1=`dio87c -- memory` || exit 1

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtca r1 t1 manual "OK" || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "/bin/sleep" || exit 1
dio87c -- rjsa r1 t1 j1 "4" || exit 1

# 2 cycles with 4 seconds of temporizations
dio87c -- rtsc r1 t1 2 4 end || exit 1

dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# wait for condition
waitretry 9 "dio87c -- etm -wS_" "W_d1_t1_wait_for_condition"
if [ $? -ne 0 ]; then
	echo "Task is not waiting for condition"; exit 1
fi
echo "   - task waits for condition"

#validate condition
dio87c -- xtcv d1 t1 manual "OK" 

# first execution
waitretry 9 "dio87c -- etm -rS_" "R_d1_t1_running"
if [ $? -ne 0 ]; then
	echo "Task is not runing"; exit 1
fi
echo "   - first execution"
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration_1\" | wc -l`" -ne 1 ]; then
	echo "Actual iteration is not 1"; exit 1
fi
sleep 2

waitretry 9 "dio87c -- etm -wS_" "W_d1_t1_wait_tempo"
if [ $? -ne 0 ]; then
	echo "Task is not waiting"; exit 1
fi
echo "   - first temporization"
sleep 2

# wait for condition
waitretry 9 "dio87c -- etm -wS_" "W_d1_t1_wait_for_condition"
if [ $? -ne 0 ]; then
	echo "Task is not waiting for condition"; exit 1
fi
echo "   - task waits for condition"

#validate condition
dio87c -- xtcv d1 t1 manual "OK"

# second execution
waitretry 9 "dio87c -- etm -rS_" "R_d1_t1_running"
if [ $? -ne 0 ]; then
	echo "Task is not runing"; exit 1
fi
echo "   - second execution"
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration_2\" | wc -l`" -ne 1 ]; then
	echo "Actual iteration is not 2"; exit 1
fi
sleep 2

waitretry 9 "dio87c -- etm -wS_" "W_d1_t1_wait_tempo"
if [ $? -ne 0 ]; then
	echo "Task is not waiting"; exit 1
fi
echo "   - second temporization"
sleep 2

waitretry 9 "dio87c -- etm -tS_" "T_d1_t1_terminated"
if [ $? -ne 0 ]; then
	echo "Task is not terminated"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

waitMessageSending

# check memory
memx2=`dio87c -- memory` || exit 1
if [ $memx1 -ne $memx2 ]; then
	echo "dio87d : memory error $memx1 -> $memx2"; exit 1
fi



#############################
echo " check cyclical task with start condition (2)"

memx1=`dio87c -- memory` || exit 1

dio87c -- va startTask NO || exit 1
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtca r1 t1 if_equal startTask "OK" || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsp r1 t1 j1 "/bin/sleep" || exit 1
dio87c -- rjsa r1 t1 j1 "4" || exit 1

# 2 cycles with 4 seconds of temporizations
dio87c -- rtsc r1 t1 2 4 end || exit 1

dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# wait for condition
waitretry 9 "dio87c -- etm -wS_" "W_d1_t1_wait_for_condition"
if [ $? -ne 0 ]; then
	echo "Task is not waiting for condition"; exit 1
fi
echo "   - task waits for condition"

#change value of variable to validate condition
dio87c -- vsv startTask OK || exit 1

# first execution
waitretry 9 "dio87c -- etm -rS_" "R_d1_t1_running"
if [ $? -ne 0 ]; then
	echo "Task is not runing"; exit 1
fi
echo "   - first execution"
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration_1\" | wc -l`" -ne 1 ]; then
	echo "Actual iteration is not 1"; exit 1
fi
sleep 2

waitretry 9 "dio87c -- etm -wS_" "W_d1_t1_wait_tempo"
if [ $? -ne 0 ]; then
	echo "Task is not waiting"; exit 1
fi
echo "   - first temporization"
sleep 2

# second execution
waitretry 9 "dio87c -- etm -rS_" "R_d1_t1_running"
if [ $? -ne 0 ]; then
	echo "Task is not runing"; exit 1
fi
echo "   - second execution"
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Actual Iteration_2\" | wc -l`" -ne 1 ]; then
	echo "Actual iteration is not 2"; exit 1
fi
sleep 2

waitretry 9 "dio87c -- etm -wS_" "W_d1_t1_wait_tempo"
if [ $? -ne 0 ]; then
	echo "Task is not waiting"; exit 1
fi
echo "   - second temporization"
sleep 2

waitretry 9 "dio87c -- etm -tS_" "T_d1_t1_terminated"
if [ $? -ne 0 ]; then
	echo "Task is not terminated"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
dio87c -- vk startTask || exit 1

waitMessageSending

# check memory
memx2=`dio87c -- memory` || exit 1
if [ $memx1 -ne $memx2 ]; then
	echo "dio87d : memory error $memx1 -> $memx2"; exit 1
fi

stopDio87d || exit 1

