#!/bin/sh
#
# file tst43.sh 
# (test lock-variable)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Test lock-variables"

startDio87d || exit 1

################################# check rtsl command
echo " check 'rtsl' command"
memd1=`dio87c -- memory` || exit 1

dio87c -- va v1 || exit 1
dio87c -- va v2 || exit 1

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rta r1 t2 || exit 1

# check default value
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Lock Variable_none\" | wc -l`" -ne 1 ]; then
	echo "Default value for 'Lock variable' is not 'none'"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Lock Action_none\" | wc -l`" -ne 1 ]; then
	echo "Default value for 'Lock action' is not 'none'"; exit 1
fi
if [ "`dio87c -- vi -S_ v1 | grep \"Linked Ref Tasks_0\" | wc -l`" -ne 1 ]; then
	echo "'Linked Tasks' property of a variable is not 0"; exit 1
fi

# change : lock v1
dio87c -- rtsl r1 t1 v1 lock 
if [ $? -ne 0 ]; then
	echo "Can't change lock-variable"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Lock Variable_v1\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock variable' is not 'v1'"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Lock Action_lock\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock action' is not 'lock'"; exit 1
fi
if [ "`dio87c -- vi -S_ v1 | grep \"Linked Ref Tasks_1\" | wc -l`" -ne 1 ]; then
	echo "'Linked Tasks' property of a variable is not 1"; exit 1
fi
# try to delete v1
dio87c -- vk v1 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable used by a task"; exit 1
fi
if [ "`dio87c -- vl | grep v1 | wc -l`" -ne 1 ]; then
	echo "Can't find the variable 'v1' in the list"; exit 1
fi

# change : unlock v2
dio87c -- rtsl r1 t1 v2 unlock 
if [ $? -ne 0 ]; then
	echo "Can't change lock-variable"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Lock Variable_v2\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock variable' is not 'v2'"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Lock Action_unlock\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock action' is not 'unlock'"; exit 1
fi
if [ "`dio87c -- vi -S_ v1 | grep \"Linked Ref Tasks_0\" | wc -l`" -ne 1 ]; then
	echo "'Linked Tasks' property of v1 is not 0"; exit 1
fi
if [ "`dio87c -- vi -S_ v2 | grep \"Linked Ref Tasks_1\" | wc -l`" -ne 1 ]; then
	echo "'Linked Tasks' property of v2 is not 1"; exit 1
fi

# change : none 
dio87c -- rtsl r1 t1 none 
if [ $? -ne 0 ]; then
	echo "Can't change lock-variable"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Lock Variable_none\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock variable' is not 'none'"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t1 | grep \"Lock Action_none\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock action' is not 'none'"; exit 1
fi
if [ "`dio87c -- vi -S_ v1 | grep \"Linked Ref Tasks_0\" | wc -l`" -ne 1 ]; then
	echo "'Linked Tasks' property of v1 is not 0"; exit 1
fi
if [ "`dio87c -- vi -S_ v2 | grep \"Linked Ref Tasks_0\" | wc -l`" -ne 1 ]; then
	echo "'Linked Tasks' property of v2 is not 0"; exit 1
fi

# check task copy
dio87c -- rtsl r1 t1 v1 lock || exit 1
dio87c -- rtd r1 t1 t3 || exit 1
if [ "`dio87c -- rti -S_ r1 t3 | grep \"Lock Variable_v1\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock variable' of duplicated task is not 'v1'"; exit 1
fi
if [ "`dio87c -- rti -S_ r1 t3 | grep \"Lock Action_lock\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock action' of duplicated task is not 'lock'"; exit 1
fi
if [ "`dio87c -- vi -S_ v1 | grep \"Linked Ref Tasks_2\" | wc -l`" -ne 1 ]; then
	echo "'Linked Tasks' property of a variable is not 2"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- vk v1 || exit 1
dio87c -- vk v2 || exit 1

waitMessageSending

# check memory
memd2=`dio87c -- memory` || exit 1
if [ $memd1 -ne $memd2 ]; then
	echo "dio87d : memory error $memd1 -> $memd2"; exit 1
fi


################################# check rp command
echo " check 'rp' command"
memd1=`dio87c -- memory` || exit 1
memx1=`dio87c -- memory` || exit 1

dio87c -- va v1 || exit 1
dio87c -- va v2 || exit 1
dio87c -- ra r1 || exit 1

dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtsl r1 t1 v1 lock || exit 1

dio87c -- rta r1 t2 || exit 1
dio87c -- rtoa r1 t2 add always || exit 1
dio87c -- rtsl r1 t2 v2 unlock || exit 1

dio87c -- rta r1 t3 || exit 1
dio87c -- rtoa r1 t3 add always || exit 1
dio87c -- rtsl r1 t3 none || exit 1

dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1
sleep 1
if [ "`dio87c -- xdl`" != "d1" ] ; then
	echo "Can't find day 'd1'"; exit 1
fi

# check t1
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Lock Variable_v1\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock variable' is not 'v1'"; exit 1
fi
if [ "`dio87c -- xti -S_ d1 t1 | grep \"Lock Action_lock\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock action' is not 'lock'"; exit 1
fi
if [ "`dio87c -- vi -S_ v1 | grep \"Linked Ref Tasks_1\" | wc -l`" -ne 1 ]; then
	echo "'Linked Tasks' property of v1 is not 1"; exit 1
fi

# check t2
if [ "`dio87c -- xti -S_ d1 t2 | grep \"Lock Variable_v2\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock variable' is not 'v2'"; exit 1
fi
if [ "`dio87c -- xti -S_ d1 t2 | grep \"Lock Action_unlock\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock action' is not 'unlock'"; exit 1
fi
if [ "`dio87c -- vi -S_ v2 | grep \"Linked Ref Tasks_1\" | wc -l`" -ne 1 ]; then
	echo "'Linked Tasks' property of v2 is not 1"; exit 1
fi

# check t3
if [ "`dio87c -- xti -S_ d1 t3 | grep \"Lock Variable_none\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock variable' is not 'none'"; exit 1
fi
if [ "`dio87c -- xti -S_ d1 t3 | grep \"Lock Action_none\" | wc -l`" -ne 1 ]; then
	echo "Value for 'Lock action' is not 'none'"; exit 1
fi

# try to delete v1
dio87c -- vk v1 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable used by a task"; exit 1
fi
if [ "`dio87c -- vl | grep v1 | wc -l`" -ne 1 ]; then
	echo "Can't find the variable 'v1' in the list"; exit 1
fi


dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- vk v1 || exit 1
dio87c -- vk v2 || exit 1
waitMessageSending
waitMessageSending

# check memory
memd2=`dio87c -- memory` || exit 1
if [ $memd1 -ne $memd2 ]; then
	echo "dio87d : memory error $memd1 -> $memd2"; exit 1
fi
memx2=`dio87c -- memory` || exit 1
if [ $memx1 -ne $memx2 ]; then
	echo "dio87d : memory error $memx1 -> $memx2"; exit 1
fi

################################# check lock
echo " check lock"

dio87c -- va v1 || exit 1
dio87c -- ra r1 || exit 1

dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtsl r1 t1 v1 lock || exit 1

dio87c -- rta r1 t2 || exit 1
dio87c -- rtoa r1 t2 add always || exit 1
dio87c -- rtca r1 t2 manual OK || exit 1

dio87c -- rta r1 t3 || exit 1
dio87c -- rtoa r1 t3 add always || exit 1
dio87c -- rtsl r1 t3 v1 unlock || exit 1

# dependencies : t1 -> t2 -> t3
dio87c -- rtda r1 t1 t2 || exit 1
dio87c -- rtda r1 t2 t3 || exit 1

dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1
sleep 1

# lock variable
dio87c -- vsv v1 locked || exit 1
dio87c -- xdse d1 true || exit 1
sleep 1

# check task is waiting
if [ "`dio87c -- xti -S_ d1 t1 | grep \"State_wait_lock\" | wc -l`" -ne 1 ]; then
	echo "Task is not waiting for lock-variable"; exit 1
fi

# unlock variable
dio87c -- vsv v1 "" || exit 1

sleep 1
# t1 is terminated
if [ "`dio87c -- xti -S_ d1 t1 | grep \"State_terminated\" | wc -l`" -ne 1 ]; then
	echo "Task 't1' is not terminated"; exit 1
fi
# t2 is waiting for conditions
if [ "`dio87c -- xti -S_ d1 t2 | grep \"State_wait_for_condition\" | wc -l`" -ne 1 ]; then
	echo "Task is not terminated"; exit 1
fi
# variable is locked
if [ "`dio87c -- vl -vS_`" != "v1_locked" ]; then
	echo "Variable is not locked"; exit 1
fi

# validate condition of t2
dio87c -- xtcv d1 t2 manual OK || exit 1
sleep 1

# t2 is terminated
if [ "`dio87c -- xti -S_ d1 t2 | grep \"State_terminated\" | wc -l`" -ne 1 ]; then
	echo "Task 't2' is not terminated"; exit 1
fi
# t3 is terminated
if [ "`dio87c -- xti -S_ d1 t3 | grep \"State_terminated\" | wc -l`" -ne 1 ]; then
	echo "Task 't3' is not terminated"; exit 1
fi

# variable is not locked
if [ "`dio87c -- vl -vS_`" != "v1_" ]; then
	echo "Variable is not unlocked"; exit 1
fi

dio87c -- xdk d1 || exit 1


################################# check lock
echo " check lock (2)"

# create 3 days
dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1
sleep 1
dio87c -- rp -d r1 "$dayDate" d2 >/dev/null || exit 1
sleep 1
dio87c -- rp -d r1 "$dayDate" d3 >/dev/null || exit 1
sleep 1

dio87c -- xdse d1 true || exit 1
dio87c -- xdse d2 true || exit 1
dio87c -- xdse d3 true || exit 1
sleep 1

# t1 of d1 is terminated
if [ "`dio87c -- xti -S_ d1 t1 | grep \"State_terminated\" | wc -l`" -ne 1 ]; then
	echo "Task 't1' of 'd1' is not terminated"; exit 1
fi
# t1 of d2 is waiting for lock 
if [ "`dio87c -- xti -S_ d2 t1 | grep \"State_wait_lock\" | wc -l`" -ne 1 ]; then
	echo "Task 't1' of 'd2' is not waiting for lock"; exit 1
fi
# t1 of d3 is waiting for lock 
if [ "`dio87c -- xti -S_ d3 t1 | grep \"State_wait_lock\" | wc -l`" -ne 1 ]; then
	echo "Task 't1' of 'd2' is not waiting for lock"; exit 1
fi

# validate condition of t2 of d1
echo "   first day is running"
dio87c -- xtcv d1 t2 manual OK || exit 1
sleep 1

# t3 of d1 is terminated
if [ "`dio87c -- xti -S_ d1 t3 | grep \"State_terminated\" | wc -l`" -ne 1 ]; then
	echo "Task 't3' is not terminated"; exit 1
fi
# t1 of d2 is terminated
if [ "`dio87c -- xti -S_ d2 t1 | grep \"State_terminated\" | wc -l`" -ne 1 ]; then
	echo "Task 't1' of 'd2' is not terminated"; exit 1
fi
# t1 of d3 is waiting for lock 
if [ "`dio87c -- xti -S_ d3 t1 | grep \"State_wait_lock\" | wc -l`" -ne 1 ]; then
	echo "Task 't1' of 'd2' is not waiting for lock"; exit 1
fi

# validate condition of t2 of d2
echo "   second day is running"
dio87c -- xtcv d2 t2 manual OK || exit 1
sleep 1

# t3 of d2 is terminated
if [ "`dio87c -- xti -S_ d2 t3 | grep \"State_terminated\" | wc -l`" -ne 1 ]; then
	echo "Task 't3' of 'd2' is not terminated"; exit 1
fi
# t1 of d3 is terminated
if [ "`dio87c -- xti -S_ d3 t1 | grep \"State_terminated\" | wc -l`" -ne 1 ]; then
	echo "Task 't1' of 'd3' is not terminated"; exit 1
fi

# validate condition of t2 of d3
echo "   third day is running"
dio87c -- xtcv d3 t2 manual OK || exit 1
sleep 1
# t3 of d3 is terminated
if [ "`dio87c -- xti -S_ d3 t3 | grep \"State_terminated\" | wc -l`" -ne 1 ]; then
	echo "Task 't3' of 'd3' is not terminated"; exit 1
fi

# variable is not locked
if [ "`dio87c -- vl -vS_`" != "v1_" ]; then
	echo "Variable is not unlocked"; exit 1
fi


stopDio87d || exit 1
