#!/bin/sh
#
# file tst45.sh 
# (test commands of dio87d to test file presence)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test commands of dio87d to test file presence"

startDio87d || exit 1

file="$DATAPATH/dg87fic$$"
if [ -f $file ]; then
	echo "File '$file' already exists !"; exit 1
fi

mem1=`dio87c -- memory` || exit 1

echo " check file test commands"
waitretry 2 "dio87c -- fc" "0"
if [ $? -ne 0 ]; then
	echo "The files count is not zero "; exit 1
fi

# add a first file test
dio87c -- fa d1 t1 "$file" localhost 9993
if [ $? -ne 0 ]; then
	echo "Can't add a file test !"; exit 1
fi
if [ "`dio87c -- fc`" -ne 1 ]; then
	echo "The files count is not one"; exit 1
fi
waitretry 2 "dio87c -- fl -S_ | grep d1" "d1_t1_$file"
if [ $? -ne 0 ]; then
	echo "Can't find the file test in dio87d"; exit 1
fi

# try to add the same file test
dio87c -- fa d1 t1 "$file" localhost 9993 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can add the same file test !"; exit 1
fi
if [ "`dio87c -- fc`" -ne 1 ]; then
	echo "The files count is not one"; exit 1
fi

# add a second file test (with another day name)
dio87c -- fa d2 t1 "$file" localhost 9993
if [ $? -ne 0 ]; then
	echo "Can't add a second file test !"; exit 1
fi
if [ "`dio87c -- fc`" -ne 2 ]; then
	echo "The files count is not two"; exit 1
fi
waitretry 2 "dio87c -- fl -S_ | grep d1" "d1_t1_$file"
if [ $? -ne 0 ]; then
	echo "Can't find the file test in dio87d"; exit 1
fi
waitretry 2 "dio87c -- fl -S_ | grep d2" "d2_t1_$file"
if [ $? -ne 0 ]; then
	echo "Can't find the file test in dio87d"; exit 1
fi

# add a third file test (with another task name)
dio87c -- fa d1 t2 "$file" localhost 9993
if [ $? -ne 0 ]; then
	echo "Can't add a third file test !"; exit 1
fi
if [ "`dio87c -- fc`" -ne 3 ]; then
	echo "The files count is not three"; exit 1
fi
waitretry 2 "dio87c -- fl -S_ | grep d1_t1" "d1_t1_$file"
if [ $? -ne 0 ]; then
	echo "Can't find the file test in dio87d"; exit 1
fi
waitretry 2 "dio87c -- fl -S_ | grep d2_t1" "d2_t1_$file"
if [ $? -ne 0 ]; then
	echo "Can't find the file test in dio87d"; exit 1
fi
waitretry 2 "dio87c -- fl -S_ | grep d1_t2" "d1_t2_$file"
if [ $? -ne 0 ]; then
	echo "Can't find the file test in dio87d"; exit 1
fi

# delete a file test
dio87c -- fk d2 t1 "$file"
if [ $? -ne 0 ]; then
	echo "Can't delete a file test !"; exit 1
fi
if [ "`dio87c -- fc`" -ne 2 ]; then
	echo "The files count is not two"; exit 1
fi
waitretry 2 "dio87c -- fl -S_ | grep d1_t1" "d1_t1_$file"
if [ $? -ne 0 ]; then
	echo "Can't find a file test in dio87d"; exit 1
fi
waitretry 2 "dio87c -- fl -S_ | grep d1_t2" "d1_t2_$file"
if [ $? -ne 0 ]; then
	echo "Can't find a file test in dio87d"; exit 1
fi

# try to delete the same file test a second time
dio87c -- fk d2 t1 "$file" >/dev/null 2>&1
if [ $? -ne 1 ]; then
	echo "Can delete the same file test a second time !"; exit 1
fi

# delete the remaining file tests
dio87c -- fk d1 t1 "$file" || exit 1
dio87c -- fk d1 t2 "$file" || exit 1
if [ "`dio87c -- fc`" -ne 0 ]; then
	echo "The files count is not zero"; exit 1
fi

################ check file presence detection
echo " check file presence detection"

dio87c -- fa d1 t1 "$file" localhost 9993 || exit 1
waitretry 2 "dio87c -- fl -S_ | grep d1" "d1_t1_$file"
if [ $? -ne 0 ]; then
	echo "Can't find the file test in dio87d"; exit 1
fi
touch $file || exit 1
# wait file presence detection
waitretry 20 "dio87c -- fc" "0"
if [ $? -ne 0 ]; then
	echo "File count is not zero"; exit 1
fi
rm $file >/dev/null 2>&1

# verify memory
mem2=`dio87c -- memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

stopDio87d || exit 1

