#!/bin/sh
#
# file tst46.sh 
# (test file conditions tested by dio87d)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test file conditions tested by dio87d"

startDio87d || exit 1

file="$DATAPATH/dg87fic$$"
if [ -f $file ]; then
	echo "File '$file' already exists !"; exit 1
fi



echo "File tested with a defined task queue"
memr1=`dio87c -- memory` || exit 1
dio87c -- memory -v > /tmp/mem1

dio87c -- ra r1 || exit 1
dio87c -- rta r1 task1 || exit 1 
dio87c -- rtoa r1 task1 add always || exit 1
dio87c -- rtca r1 task1 manual "OK" || exit 1
dio87c -- rtca r1 task1 file "$file" || exit 1

dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

waitretry 5 "dio87c -- xtl -sS_ d1" "task1_wait_for_condition"
if [ $? -ne 0 ]; then
	echo "Task1 is not waiting for condition"; exit 1
fi

waitretry 2 "dio87c -- xtcl -sS_ d1 task1 | grep manual" "manual_OK__waiting"
if [ $? -ne 0 ]; then
	echo "The manual condition is not waiting"; exit 1
fi
waitretry 2 "dio87c -- xtcl -sS_ d1 task1 | grep file" "file_${file}__waiting"
if [ $? -ne 0 ]; then
	echo "The file condition is not waiting"; exit 1
fi
waitretry 2 "dio87c -- fc" "0"
if [ $? -ne 0 ]; then
	echo "The file count is not zero in dio87d"; exit 1
fi

echo " validate manual condition"
dio87c -- xtcv d1 task1 manual "OK"
waitretry 5 "dio87c -- xtl -sS_ d1" "task1_wait_for_condition"
if [ $? -ne 0 ]; then
	echo "Task1 is not waiting for condition"; exit 1
fi

waitretry 2 "dio87c -- xtcl -sS_ d1 task1 | grep manual" "manual_OK__finished"
if [ $? -ne 0 ]; then
	echo "The manual condition is not finished"; exit 1
fi
waitretry 2 "dio87c -- xtcl -sS_ d1 task1 | grep file" "file_${file}__waiting"
if [ $? -ne 0 ]; then
	echo "The file condition is not waiting"; exit 1
fi
waitretry 2 "dio87c -- fc" "1"
if [ $? -ne 0 ]; then
	echo "The files count is not one in dio87d"; exit 1
fi
waitretry 2 "dio87c -- fl -S_" "d1_task1_$file"
if [ $? -ne 0 ]; then
	echo "Can't find the file test in dio87d"; exit 1
fi

echo " create file"
touch $file || exit 1
waitretry 20 "dio87c -- fc" "0"
if [ $? -ne 0 ]; then
	echo "The files count is not 0 in dio87d"; exit 1
fi

waitretry 5 "dio87c -- xtcl -sS_ d1 task1 | grep file" "file_${file}__finished"
if [ $? -ne 0 ]; then
	echo "The file condition is not finished"; exit 1
fi

waitretry 5 "dio87c -- xtl -sS_ d1" "task1_terminated"
if [ $? -ne 0 ]; then
	echo "Task1 is not terminated"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

rm $file >/dev/null 2>&1

# verify memory
memr2=`dio87c -- memory` || exit 1
dio87c -- memory -v > /tmp/mem2
if [ $memr1 != $memr2 ] ; then
	echo "dio87d : memory error $memr1 -> $memr2"
	exit 1
fi

echo "File tested with an undefined task queue"
memr1=`dio87c -- memory` || exit 1
dio87c -- memory -v > /tmp/mem1
dio87c -- ra r1 || exit 1
dio87c -- rta r1 task1 || exit 1 
dio87c -- rtoa r1 task1 add always || exit 1
dio87c -- rtst r1 task1 none || exit 1
dio87c -- rtca r1 task1 manual "OK" || exit 1
dio87c -- rtca r1 task1 file "$file" || exit 1

dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

waitretry 5 "dio87c -- xtl -sS_ d1" "task1_wait_for_condition"
if [ $? -ne 0 ]; then
	echo "Task1 is not waiting for condition"; exit 1
fi

waitretry 2 "dio87c -- xtcl -sS_ d1 task1 | grep manual" "manual_OK__waiting"
if [ $? -ne 0 ]; then
	echo "The manual condition is not waiting"; exit 1
fi
waitretry 2 "dio87c -- xtcl -sS_ d1 task1 | grep file" "file_${file}__waiting"
if [ $? -ne 0 ]; then
	echo "The file condition is not waiting"; exit 1
fi
waitretry 2 "dio87c -- fc" "0"
if [ $? -ne 0 ]; then
	echo "The files count is not zero in dio87d"; exit 1
fi

echo " validate manual condition"
dio87c -- xtcv d1 task1 manual "OK"
waitretry 5 "dio87c -- xtl -sS_ d1" "task1_wait_for_condition"
if [ $? -ne 0 ]; then
	echo "Task1 is not waiting for condition"; exit 1
fi

waitretry 2 "dio87c -- xtcl -sS_ d1 task1 | grep manual" "manual_OK__finished"
if [ $? -ne 0 ]; then
	echo "The manual condition is not finished"; exit 1
fi
waitretry 2 "dio87c -- xtcl -sS_ d1 task1 | grep file" "file_${file}__waiting"
if [ $? -ne 0 ]; then
	echo "The file condition is not waiting"; exit 1
fi
waitretry 2 "dio87c -- fc" "0"
if [ $? -ne 0 ]; then
	echo "The files count is not zero in dio87d"; exit 1
fi

echo " create file"
touch $file || exit 1
waitretry 5 "dio87c -- xtcl -sS_ d1 task1 | grep file" "file_${file}__finished"
if [ $? -ne 0 ]; then
	echo "The file condition is not finished"; exit 1
fi

waitretry 5 "dio87c -- xtl -sS_ d1" "task1_terminated"
if [ $? -ne 0 ]; then
	echo "Task1 is not terminated"; exit 1
fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1

rm $file >/dev/null 2>&1

waitMessageSending dio87c

# verify memory
memr2=`dio87c -- memory` || exit 1
dio87c -- memory -v > /tmp/mem2
if [ $memr1 != $memr2 ] ; then
	echo "dio87d : memory error $memr1 -> $memr2"
	exit 1
fi

stopDio87d || exit 1

