#!/bin/sh


CURDIR="`pwd`"
SPECFILE="$CURDIR/tmp/SPECS/diogene87.spec"
SPECFILEIN="$CURDIR/diogene87.spec.in"
RPMDIR="$CURDIR/tmp"

if [ ! -f $SPECFILEIN ]; then
	echo "Can't find '$SPECFILEIN' !"; exit 1
fi

if [ "$#" -ne 1 ] ; then
	echo "usage : ./makerpm [build|clean]"; exit 1
fi

package=$1
if [ $package != "clean" -a $package != "build" ] ; then
	echo "usage : ./buildpackage [clean|build]"; exit 1
fi


if [ $package == clean ] ; then
	echo "clean"
	if [ -d $RPMDIR ] ; then
		echo "remove $RPMDIR"
		rm -rf $RPMDIR
	fi
fi

if [ $package == build ] ; then

	# create tmp directories
	for rep in SPECS BUILD RPMS SRPMS INSTALL
	do
		if [ ! -f $RPMDIR/$rep ] ; then
			mkdir -p $RPMDIR/$rep || exit 1
		fi
	done
	# create SOURCES directories 
	ln -s $CURDIR/.. $RPMDIR/SOURCES

	# extract version from configure.in
	VERSION=`sed -ne "s/AC_INIT(.*,\[\(.*\)\],.*)/\1/p" ../configure.in`

	# check rpmbuild programm
	which rpmbuild > /dev/null 2>&1
	if [ $? -ne 0 ] ; then
		echo "Can't find 'rpmbuild' !"; exit 1
	fi
	# check spec.in file
	if [ ! -f "$SPECFILEIN" ] ; then
		echo "Can't find $SPECFILEIN !" ; exit 1
	fi
	# clean previous Makefile
	if [ -f ../Makefile ] ; then
		(cd .. && make distclean)
	fi

	# create spec file from spec.in file
	sed -e "s!DG87_RPMDIR!$RPMDIR!g;" \
	    -e "s!DG87_VERSION!$VERSION!g;" \
		$SPECFILEIN > $SPECFILE	

	if [ ! -f "$SPECFILE" ] ; then
		echo "Can't find $SPECFILE !" ; exit 1
	fi

	echo "Build rpm packages ..."
	rpmbuild -bb $SPECFILE 
fi
