/* file d_cmdoperation.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "context.h"
#include "d_cmdoperation.h"
#include "d_rule.h"
#include "util.h"
#include "engine.h"

/******************************************************************************/
dg87_ref_rule_operation *
cmd_operation_ra (dg87_connection * connection, dg87_ref_model * ref)
{
  dg87_ref_rule *complex_rule;
  dg87_ref_rule_operation *operation;
  dg87_command *command;
  dg87_ref_rule_operator oper;
  dg87_ref_rule_type type;
  dg87_context *context;
  dg87_variable *variable;
  int integer;
  time_t date;
  int day, month;
  char *value;

  assert (connection != NULL && ref != NULL);
  if (connection == NULL || ref == NULL)
    return NULL;
  command = connection->command;
  context = connection->server->data;

  /* interpret operator */
  oper = dg87_ref_rule_ascii_to_operator (command->argv[2]);
  if (oper == -1)
    {
      dg87_connection_printf (connection,
                              gettext ("Unknown operator '%s'\n"), 
															command->argv[2]);
      return NULL;
    }
  /* interpret type */
  type = dg87_ref_rule_ascii_to_type (command->argv[3]);
  if (type == -1)
    {
      dg87_connection_printf (connection,
                              gettext ("Unknown type '%s'\n"), 
															command->argv[3]);
      return NULL;
    }
  /* control last argument */
  if (type == dg87_ref_rule_type_always)
    {
      if (command->argc != 4)
        {
          dg87_connection_printf (connection,
													gettext ("the data argument is not requiered\n"));
          return NULL;
        }
    }
  else if (type == dg87_ref_rule_type_ifequal ||
           type == dg87_ref_rule_type_ifnequal)
    {
      if (command->argc != 6)
        {
          dg87_connection_printf (connection,
													gettext ("Two data arguments are requiered\n"));
          return NULL;
        }
    }
  else
    {
      if (command->argc != 5)
        {
          dg87_connection_printf (connection, 
													gettext ("The data argument is requiered\n"));
          return NULL;
        }
    }

  /* create operation */
  operation = NULL;
  switch (type)
    {
    case dg87_ref_rule_type_dow:
			value = command->argv[4];
			if (value[0] >= '1' && value[0] <= '9')
			{
				/* the index of the day is specified */
      	integer = atoi (value) - 1;
			}
			else
			{
				/* the name of the day is specified */
				for (integer = 0; integer < 7; integer++)
								if (strcmp(day_name_array[integer], value) == 0)
												break;
				if (integer == 7){
          dg87_connection_printf (connection,
													gettext ("Incorrect day name '%s'\n"), value);
					dg87_err_set_error_with_arg(
													DG87_ERR_OPER_INCORRECT_DAY_NAME, value);

          return NULL;
				}
			}
			/* integer is the day index (index>=0 && index<=7) */
      operation = dg87_ref_operation_dayofweek_new (oper, integer);
      break;
    case dg87_ref_rule_type_dom:
      integer = atoi (command->argv[4]);
      operation = dg87_ref_operation_dayofmonth_new (oper, integer);
      break;
    case dg87_ref_rule_type_doy:
      integer = atoi (command->argv[4]);
      operation = dg87_ref_operation_dayofyear_new (oper, integer);
      break;
    case dg87_ref_rule_type_wom:
      integer = atoi (command->argv[4]);
      operation = dg87_ref_operation_weekofmonth_new (oper, integer);
      break;
    case dg87_ref_rule_type_woy:
      integer = atoi (command->argv[4]);
      operation = dg87_ref_operation_weekofyear_new (oper, integer);
      break;
    case dg87_ref_rule_type_moy:
			value = command->argv[4];
			if (value[0] >= '1' && value[0] <= '9')
			{
				/* the index of the month is specified */
      	integer = atoi (value) - 1;
			}
			else
			{
				/* the name of the month is specified */
				for (integer = 0; integer < 12; integer++)
								if (strcmp(month_name[integer], value) == 0)
												break;
				if (integer == 12){
          dg87_connection_printf (connection,
													gettext ("Incorrect month name '%s'\n"), value);
          return NULL;
				}
			}
      operation = dg87_ref_operation_monthofyear_new (oper, integer);
      break;
    case dg87_ref_rule_type_day:
      if (dg87_ref_rule_ascii_to_day (command->argv[4], &month, &day) ==
          FALSE)
        {
          dg87_connection_printf (connection,
                                  gettext ("Incorrect value for day '%s'\n"),
                                  command->argv[4]);
          return NULL;
        }
      operation = dg87_ref_operation_day_new (oper, month, day);
      break;
    case dg87_ref_rule_type_date:
      date = dg87_ref_rule_ascii_to_date (command->argv[4]);
      if (date == -1)
        {
          dg87_connection_printf (connection,
                                  gettext ("Incorrect value for date '%s'\n"),
                                  command->argv[4]);
          return NULL;
        }
      operation = dg87_ref_operation_date_new (oper, date);
      break;
    case dg87_ref_rule_type_always:
      if (oper != dg87_ref_rule_operator_add)
        {
          dg87_connection_printf (connection,
                                  gettext ("Operator must be 'add' with an "\
                                  "'always' rule.\n"));
          return NULL;
        }
      operation = dg87_ref_operation_always_new ();
      break;
    case dg87_ref_rule_type_complex:
      if (oper != dg87_ref_rule_operator_add &&
          oper != dg87_ref_rule_operator_remove &&
          oper != dg87_ref_rule_operator_shift_after &&
          oper != dg87_ref_rule_operator_shift_before)
        {
          dg87_connection_printf (connection,
													gettext ("Operator must be 'add' or 'remove' "\
													"or 'shift_after' or 'shift_before' "\
													"with a 'complex' rule.\n"));
          return NULL;
        }
      complex_rule = dg87_ref_model_rule_get_by_name (ref, command->argv[4]);
      if (complex_rule == NULL)
        {
          dg87_connection_printf (connection,
													gettext ("Can't find the rule '%s' in the "\
																	"referential '%s' ! %s\n"),
													command->argv[4], ref->name,
													dg87_err_get_error_str ());
          return NULL;
        }
      operation = dg87_ref_operation_complex_new (oper, complex_rule);
      break;
    case dg87_ref_rule_type_ifequal:
      if (oper != dg87_ref_rule_operator_add &&
          oper != dg87_ref_rule_operator_remove)
        {
          dg87_connection_printf (connection,
													gettext ("Operator must be 'add' or 'remove' "\
													"with a 'if_equal' rule.\n"));
          return NULL;
        }
      variable = dg87_engine_variable_get_by_name (context->engine,
                                                   command->argv[4]);
      if (variable == NULL)
        {
          dg87_connection_printf (connection,
													gettext ("Can't find the variable '%s' ! %s \n"),
													command->argv[4],
													dg87_err_get_error_str ());
          return NULL;
        }
      value = command->argv[5];
      operation = dg87_ref_operation_ifequal_new (oper, variable, value);
      break;
    case dg87_ref_rule_type_ifnequal:
      if (oper != dg87_ref_rule_operator_add &&
          oper != dg87_ref_rule_operator_remove)
        {
          dg87_connection_printf (connection,
													gettext ("Operator must be 'add' or 'remove' "\
													"with a 'if_not_equal' rule.\n"));
          return NULL;
        }
      variable = dg87_engine_variable_get_by_name (context->engine,
                                                   command->argv[4]);
      if (variable == NULL)
        {
          dg87_connection_printf (connection,
													gettext ("Can't find the variable '%s' ! %s \n"),
													command->argv[4],
													dg87_err_get_error_str ());
          return NULL;
        }
      value = command->argv[5];
      operation = dg87_ref_operation_ifnequal_new (oper, variable, value);
      break;

    }

  return operation;
}

/******************************************************************************/
int
cmd_operation_rsr (dg87_connection * connection, dg87_list * operation_list)
{
  char *sep, *beg, *end; /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  dg87_command *command;
  struct tm dc;
  time_t day, first_day, last_day; /* of the Range */
  BOOL result;
  BOOL gshow_day_name = FALSE;
  BOOL show_date = FALSE;
  char *string;

  assert (connection != NULL && operation_list != NULL);
  if (connection == NULL || operation_list == NULL)
    return 1;
  command = connection->command;

  /* interpret date1 argument */
  first_day = dg87_ref_rule_ascii_to_date (command->argv[2]);
  if (first_day == -1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s' for date1 argument.\n"),
											command->argv[2]);
      return 1;
    }

  /* interpret date2 argument (which is optional) */
  if (command->argc != 4)
    {
      last_day = first_day;
    }
  else
    {
      last_day = dg87_ref_rule_ascii_to_date (command->argv[3]);
      if (last_day == -1)
        {
          dg87_connection_printf (connection,
									gettext ("Incorrect value '%s' for date2 argument.\n"),
									command->argv[3]);
          return 1;
        }
    }

  /* reverse day if necessary */
  if (last_day < first_day)
    {
      day = first_day;
      first_day = last_day;
      last_day = day;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'n':              /* name of week day */
          gshow_day_name = TRUE;
          break;
        case 'd':              /* name of date */
          show_date = TRUE;
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* calculate */
  for (day = first_day; day <= last_day; day += 24 * 3600)
    {
      dc = *localtime (&day);
      result = FALSE;

      if (dg87_ref_operationlist_calculate (operation_list,
                                            day, &result, -1) == FALSE)
        {
          dg87_connection_printf (connection,
													gettext ("Error while rule calculation ! %s\n"),
													dg87_err_get_error_str ());
          /* free memory */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }

      dg87_connection_printf (connection, "%s", beg);
      if (gshow_day_name == TRUE)
        dg87_connection_printf (connection, "%s%s", 
												day_name_array[dc.tm_wday], sep);
      if (show_date == TRUE)
        {
          string = dg87_ref_rule_date_to_ascii (day);
          dg87_connection_printf (connection, "%s%s", string, sep);
          dg87_free (string, DG87_CHAR);
        }

      dg87_connection_printf (connection, "%s%s\n",
                              (result == TRUE) ? "Y" : "N", end);
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
int
cmd_operation_rsy (dg87_connection * connection, dg87_list * operation_list)
{
  dg87_command *command;
  int year;
  struct tm dc;
  time_t day, first_day, last_day;      /* of the year */
  int d[12][31];
  int imonth, iday;
  char c;
  BOOL result;
	char short_name[4];

  assert (connection != NULL && operation_list != NULL);
  if (connection == NULL || operation_list == NULL)
    return 1;
  command = connection->command;

  /* interpret year argument */
  year = atoi (command->argv[2]);
  if (year < 1900)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s' for year argument.\n"),
											command->argv[2]);
      return 1;
    }
  /* calculate first day of the year */
  dc.tm_sec = 0;
  dc.tm_min = 0;
  dc.tm_hour = 0;
  dc.tm_mday = 1;
  dc.tm_mon = 0;
  dc.tm_wday = 0;
  dc.tm_yday = 0;
  dc.tm_isdst = -1;
  dc.tm_year = year - 1900;
  first_day = mktime (&dc);

  /* calculate last day of the year */
  dc.tm_sec = 1;
  dc.tm_min = 0;
  dc.tm_hour = 0;
  dc.tm_mday = 1;
  dc.tm_mon = 0;
  dc.tm_wday = 0;
  dc.tm_yday = 0;
  dc.tm_isdst = -1;
  dc.tm_year = year + 1 - 1900;
  last_day = mktime (&dc) - 24 * 3600;

  /* initialize array */
  for (imonth = 0; imonth < 12; imonth++)
    for (iday = 0; iday < 31; iday++)
      d[imonth][iday] = 7;

  /* calculate */
  for (day = first_day; day <= last_day; day += 24 * 3600)
    {
      dc = *localtime (&day);
      result = FALSE;
      d[dc.tm_mon][dc.tm_mday - 1] = dc.tm_wday;
      if (dg87_ref_operationlist_calculate (operation_list,
                                            day, &result, -1) == FALSE)
        {
          dg87_connection_printf (connection,
													gettext ("Error while rule calculation ! %s\n"),
													dg87_err_get_error_str ());
          return 1;
        }
      if (result == TRUE)
        d[dc.tm_mon][dc.tm_mday - 1] += 8;
    }

  /* print calendar */
  dg87_connection_printf (connection, "Year : %i\n  ", year);
  /* print header (month name) */
  for (imonth = 0; imonth < 12; imonth++)
	{
		strncpy(short_name, month_name[imonth], 3);
		short_name[3] = '\0';
    dg87_connection_printf (connection, "   %s", short_name);
	}
  /* each line is a day */
  for (iday = 0; iday < 31; iday++)
    {
      if (iday < 9)
        dg87_connection_printf (connection, "\n %i  ", iday + 1);
      else
        dg87_connection_printf (connection, "\n%i  ", iday + 1);
      /* each column is a month */
      for (imonth = 0; imonth < 12; imonth++)
        {
          if (d[imonth][iday] >= 8)
            {
              c = '*';
              d[imonth][iday] -= 8;
            }
          else
            {
              c = ' ';
            }
					if (d[imonth][iday] == 7)
						strcpy(short_name,"..");
					else
						strncpy(short_name, day_name_array[d[imonth][iday]], 2);
					short_name[2] = '\0';
          dg87_connection_printf (connection, " %s %c ", short_name, c);
        }
    }
  dg87_connection_printf (connection, "\n");
  return 0;
}

/******************************************************************************/
int
cmd_operation_ol (dg87_connection * connection, dg87_list * operation_list)
{
  char *sep, *beg, *end;/* separator, begin, end */
  /*static char optstring[] = "dtB:S:E:I:N:"; */
  int optch, iopt;
  char *string;
  char *optarg;
  int start = 0;  /* index of the first task to display */
  int number = 0; /* number of Rules to display */
  int count = 0;  /* number of Rule actually display */
  dg87_item *item = NULL;
  dg87_ref_rule_operation *operation;
  dg87_command *command;
  BOOL show_index = FALSE;
  BOOL show_human = FALSE;

  assert (connection != NULL && operation_list != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I':              /* index of the first operation to display */
          start = atoi (optarg);
          break;
        case 'N':              /* number of operations to display */
          number = atoi (optarg);
          break;
        case 'i':              /* print index */
          show_index = TRUE;
          break;
        case 'h':              /* human format */
          show_human = TRUE;
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first operation to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (operation_list, start);
  else
    item = operation_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      operation = item->object;

      dg87_connection_printf (connection, "%s", beg);
      /* index */
      if (show_index == TRUE)
        {
          dg87_connection_printf (connection, "%i%s", count, sep);
        }
      /* operator */
      dg87_connection_printf (connection, "%s%s",
                              dg87_ref_rule_operator_to_ascii (operation->
                                                               oper), sep);
      /* type of rule */
      dg87_connection_printf (connection, "%s%s",
                              dg87_ref_rule_type_to_ascii (operation->type),
                              sep);
      /* print data */
      switch (operation->type)
        {
        case dg87_ref_rule_type_dow:
					if (show_human == TRUE)
          	dg87_connection_printf (connection, "%s",
                               day_name_array[operation->rule.dow.day_of_week]);
					else
          	dg87_connection_printf (connection, "%i",
                                  operation->rule.dow.day_of_week + 1);
          break;
        case dg87_ref_rule_type_dom:
         	dg87_connection_printf (connection, "%i",
                                  operation->rule.dom.day_of_month);
          break;
        case dg87_ref_rule_type_doy:
          dg87_connection_printf (connection, "%i",
                                  operation->rule.doy.day_of_year);
          break;
        case dg87_ref_rule_type_wom:
          dg87_connection_printf (connection, "%i",
                                  operation->rule.wom.week_of_month);
          break;
        case dg87_ref_rule_type_woy:
          dg87_connection_printf (connection, "%i",
                                  operation->rule.woy.week_of_year);
          break;
        case dg87_ref_rule_type_moy:
					if (show_human == TRUE)
          	dg87_connection_printf (connection, "%s",
													month_name[operation->rule.moy.month_of_year]);
					else
          	dg87_connection_printf (connection, "%i",
													operation->rule.moy.month_of_year + 1);
          break;
        case dg87_ref_rule_type_day:
          string = dg87_ref_rule_day_to_ascii (operation->rule.day.month,
                                               operation->rule.day.day);
          if (string == NULL)
            break;
          dg87_connection_printf (connection, "%s", string);
          dg87_free (string, DG87_CHAR);
          break;
        case dg87_ref_rule_type_date:
          string = dg87_ref_rule_date_to_ascii (operation->rule.date.date);
          if (string == NULL)
            break;
          dg87_connection_printf (connection, "%s", string);
          dg87_free (string, DG87_CHAR);
          break;
        case dg87_ref_rule_type_always:
          /* nothing to print */
          break;
        case dg87_ref_rule_type_complex:
          dg87_connection_printf (connection, "%s",
                                  operation->rule.rule->name);
          break;
        case dg87_ref_rule_type_ifequal:
        case dg87_ref_rule_type_ifnequal:
          dg87_connection_printf (connection, "%s%s%s",
                                  operation->rule.ifequal.variable->name,
                                  sep, operation->rule.ifequal.value);
          break;
        }
      dg87_connection_printf (connection, "%s\n", end);
      item = item->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* End of file */
