/* file d_cmdrule.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "context.h"
#include "d_cmdrule.h"
#include "util.h"

/******************************************************************************/
/* rrsr
 * Ref Rule Simulate Range
 * Usage : rrsr RefName RuleName date1 [date2] 
 */
int
cmd_rrsr (dg87_connection * connection)
{
  dg87_command *command;
  dg87_ref_model *ref;
  dg87_ref_rule *rule;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 3 || command->argc > 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rrsr RefName RuleName date1 [date2]");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the rule */
  rule = dg87_util_find_ref_rule (connection, ref, command->argv[1]);
  if (rule == NULL)
    return 1;

  return cmd_operation_rsr (connection, rule->operation_list);
}

/******************************************************************************/
/* rrsy
 * Ref Rule Simulate Year
 * Usage : rrsy RefName RuleName Year
 */
int
cmd_rrsy (dg87_connection * connection)
{
  dg87_command *command;
  dg87_ref_model *ref;
  dg87_ref_rule *rule;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rrsy RefName RuleName Year");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the rule */
  rule = dg87_util_find_ref_rule (connection, ref, command->argv[1]);
  if (rule == NULL)
    return 1;

  return cmd_operation_rsy (connection, rule->operation_list);
}

/******************************************************************************/
/* rrk
 * Ref Rule Kill
 * Usage : rrk RefName RuleName
 */
int
cmd_rrk (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  dg87_ref_model *ref;
  dg87_ref_rule *rule;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rrk RefName RuleName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* find the rule */
  rule = dg87_util_find_ref_rule (connection, ref, command->argv[1]);
  if (rule == NULL)
    return 1;
  /* delete the rule */
  dg87_err_clear_error ();
  context = connection->server->data;
  if (dg87_ref_model_rule_delete_by_name (ref, command->argv[1]) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't delete the rule '%s' ! %s\n"),
                              command->argv[1], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rra
 * Ref Rule Add
 * Usage : rra RefName RuleName
 */
int
cmd_rra (dg87_connection * connection)
{
  dg87_context *context;
  dg87_ref_rule *rule;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 2 || command->argc > 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rra RefName RuleName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  /* add the rule */
  dg87_err_clear_error ();
  context = connection->server->data;
  rule = dg87_ref_model_rule_add (ref, command->argv[1]);
  if (rule == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't add a rule '%s' ! %s\n"),
                              command->argv[1], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* rrsn
 * Ref Rule Set Name 
 * Usage : rrsn RefName RuleName NewName 
 */
int
cmd_rrsn (dg87_connection * connection)
{
  dg87_ref_rule *rule;
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rrsn RefName RuleName NewName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the rule */
  rule = dg87_util_find_ref_rule (connection, ref, command->argv[1]);
  if (rule == NULL)
    return 1;
  /* change the name */
  dg87_err_clear_error ();
  if (dg87_ref_rule_set_name (rule, command->argv[2]) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't change the name of the rule '%s' ! %s\n"),
											rule->name, dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* cmd_rrc
 * Ref Rule Count
 * return the number of Rules in a referentiel.
 * Usage : rrc RefName
 */
int
cmd_rrc (dg87_connection * connection)
{
  dg87_command *command;
  dg87_ref_model *ref;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"), 
											"rrc RefName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;
  dg87_connection_printf (connection, "%i\n", ref->rule_vector->count);
  return 0;
}

/******************************************************************************/
/* cmd_rrl
 * Ref Rule List
 * Usage : rrl [options] RefName
 * options
 * -u : print usage counters
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first rule to display (first index is 0)
 * -Nnumber : number of rule to display (no limit if -N < 1)
 */
int
cmd_rrl (dg87_connection * connection)
{
  dg87_context *context;
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  char *optarg;
  int start = 0;                /* index of the first rule to display */
  int number = 0;               /* number of RULEs to display */
  int count = 0;                /* number of RULE actually display */
  dg87_element *element = NULL;
  dg87_ref_rule *rule;
  dg87_command *command;
  dg87_ref_model *ref;
  BOOL show_counters = FALSE;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  context = connection->server->data;

  /* control number of arguments */
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rrl [options] RefName");
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S': /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B': /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E': /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I': /* index of the first rule to display */
          start = atoi (optarg);
          break;
        case 'N': /* number of rule to display */
          number = atoi (optarg);
          break;
        case 'u': /* show use counters  */
          show_counters = TRUE;
          break;
        default:  /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first rule to display */
  element = NULL;
  if (start > 0)
    element = dg87_vector_get_element_by_index (ref->rule_vector, start);
  else
    element = ref->rule_vector->first;

  /* create the list */
  count = 0;
  while (element != NULL)
    {
      /* reach the limit of rule to display */
      count++;
      if (number > 0 && count > number)
        break;
      rule = element->object;
      /* print rule name */
      dg87_connection_printf (connection, "%s%s", beg, rule->name);
      /* print counters */
      if (show_counters == TRUE)
        {
          dg87_connection_printf (connection, "%s%i%s%i",
                                  sep, rule->linked_rules,
                                  sep, rule->linked_tasks);
        }

      dg87_connection_printf (connection, "%s\n", end);
      element = element->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}

/******************************************************************************/
/* rrok
 * Referential Rule Operation Kill
 * Usage : rrok RefName RuleName OperationIndex 
 */
int
cmd_rrok (dg87_connection * connection)
{
  int index;
  dg87_ref_model *ref;
  dg87_ref_rule *rule;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rrok RefName RuleName OperationIndex");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the rule */
  rule = dg87_util_find_ref_rule (connection, ref, command->argv[1]);
  if (rule == NULL)
    return 1;
  /* find the operation */
  dg87_err_clear_error ();
  index = atoi (command->argv[2]) - 1;
  if (dg87_ref_rule_operation_delete (rule, index) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't delete the operation for "\
															"the rule '%s' in the referential '%s' ! %s\n"),
                              rule->name, ref->name,
                              dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/******************************************************************************
 * cmd_rroa
 * Ref Rule Operation Add
 * Usage : rroa RefName RuleName operator type [data]
 */
int
cmd_rroa (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_rule *rule;
  dg87_ref_rule_operation *operation;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc < 4 || command->argc > 6)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rroa RefName RuleName operator type [data1] [data2]");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the rule */
  rule = dg87_util_find_ref_rule (connection, ref, command->argv[1]);
  if (rule == NULL)
    return 1;

  /* create operation */
  operation = cmd_operation_ra (connection, ref);
  if (operation == NULL)
    {
      dg87_connection_printf (connection,
											gettext ("Can't add an operation to the rule '%s' "\
											"of the referential '%s' ! %s\n"),
											rule->name, ref->name,
											dg87_err_get_error_str ());
      return 1;
    }

  /* add operation to the list */
  if (dg87_ref_rule_operation_add (rule, operation) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't add an operation to the rule '%s' "\
											"of the referential '%s' ! %s\n"),
											rule->name, ref->name,
											dg87_err_get_error_str ());
      dg87_ref_operation_delete (operation);
      return 1;
    }

  return 0;
}

/******************************************************************************/
/* cmd_rroc
 * Usage : rroc RefName RuleName
 * Rule Operation Count
 * return the number of operations in the rule.
 */
int
cmd_rroc (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_rule *rule;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rroc RefName RuleName");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the rule */
  rule = dg87_util_find_ref_rule (connection, ref, command->argv[1]);
  if (rule == NULL)
    return 1;

  /* return operations count */
  dg87_connection_printf (connection, "%i\n", rule->operation_list->count);
  return 0;
}

/******************************************************************************/
/* cmd_rrol
 * rrol [options] RefName RuleName
 * Ref Rule Operation List
 * Options
 * -i : print operation index
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first operation to display (first index is 0)
 * -Nnumber : number of operation to display (no limit if -N < 1)
 */
int
cmd_rrol (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_rule *rule;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 2)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rrol [options] RefName RuleName");
      return 1;
    }
  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the rule */
  rule = dg87_util_find_ref_rule (connection, ref, command->argv[1]);
  if (rule == NULL)
    return 1;

  return cmd_operation_ol (connection, rule->operation_list);
}

/******************************************************************************/
/* cmd_rrom
 * Usage : rrom RefName RuleName OperationIndex delta
 * Referential Rule Operation Move 
 * Move an operation in the list
 */
int
cmd_rrom (dg87_connection * connection)
{
  dg87_ref_model *ref;
  dg87_ref_rule *rule;
  dg87_command *command;
  int index, delta;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  if (command->argc != 4)
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect call.\nUsage : %s\n"),
											"rrom RefName RuleName OperationIndex Delta");
      return 1;
    }
  if (command->optc != 0)
    {
      dg87_connection_printf (connection,
											gettext ("Unknown option '%c' for command '%s'\n"),
											command->optv[0], command->function_name);
      return 1;
    }

  /* find the ref */
  ref = dg87_util_find_ref_model (connection, command->argv[0]);
  if (ref == NULL)
    return 1;

  /* find the rule */
  rule = dg87_util_find_ref_rule (connection, ref, command->argv[1]);
  if (rule == NULL)
    return 1;

  /* get index from parameter */
  index = atoi (command->argv[2]) - 1;

	/* interpret the delta parameter */
	if (strcmp(command->argv[3], "up") == 0)
					delta = -1;
	else if (strcmp(command->argv[3], "down") == 0)
					delta = 1;
	else
    {
      dg87_connection_printf (connection,
											gettext ("Incorrect value '%s' for the delta parameter "\
															"of the command %s, "\
															"please use 'up' or 'down'.\n"),
											command->argv[3], command->function_name);
      return 1;
    }

  if (dg87_ref_rule_operation_move (rule, index, delta) == FALSE)
    {
      dg87_connection_printf (connection,
											gettext ("Can't move an operation in the rule '%s' "\
											"of the referential '%s' ! %s\n"),
											rule->name, ref->name,
											dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* End of file */
