/* file d_condition.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <libdg87/global.h>
#include "d_condition.h"
#include "variable.h"

/****************************************************************************/
dg87_ref_condition *
dg87_ref_condition_duplicate (dg87_ref_condition * original_condition)
{
  dg87_ref_condition *new_condition;
  assert (original_condition != NULL);
  switch (original_condition->type)
    {
    case condition_hmin:
      new_condition =
        dg87_ref_condition_hmin_new (original_condition->data.condition_hmin->
                                     minutes,
                                     original_condition->data.condition_hmin->
                                     day_delta);
      break;
    case condition_hmax:
      new_condition =
        dg87_ref_condition_hmax_new (original_condition->data.condition_hmin->
                                     minutes,
                                     original_condition->data.condition_hmin->
                                     day_delta);
      break;
    case condition_hlate:
      new_condition =
        dg87_ref_condition_hlate_new (original_condition->data.
                                      condition_hmin->minutes,
                                      original_condition->data.
                                      condition_hmin->day_delta);
      break;
    case condition_file:
      new_condition =
        dg87_ref_condition_file_new (original_condition->data.condition_File->
                                     path);
      break;
    case condition_manual:
      new_condition =
        dg87_ref_condition_manual_new (original_condition->data.
                                       condition_manual->message);
      break;
    case condition_ifequal:
      new_condition =
        dg87_ref_condition_ifequal_new (original_condition->data.
                                        condition_ifequal->variable,
                                        original_condition->data.
                                        condition_ifequal->value);
      break;
    case condition_ifnequal:
      new_condition =
        dg87_ref_condition_ifnotequal_new (original_condition->data.
                                           condition_ifnotequal->variable,
                                           original_condition->data.
                                           condition_ifnotequal->value);
      break;

    default:
      new_condition = NULL;
    }
	DG87_SAVE_NEEDED;
  return new_condition;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_condition_hmin_new (int minutes, dg87ubyte day_delta)
{
  dg87_ref_condition *condition;
  if (minutes < 0 || minutes >= 24 * 60)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_ref_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_hmin =
       dg87_malloc (sizeof (struct dg87_ref_condition_h),
                    DG87_OTHER)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->type = condition_hmin;
  condition->data.condition_hmin->minutes = minutes;
  condition->data.condition_hmin->day_delta = day_delta;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_ref_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_condition_hmax_new (int minutes, dg87ubyte day_delta)
{
  dg87_ref_condition *condition;
  if (minutes < 0 || minutes >= 24 * 60)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_ref_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_hmax = dg87_malloc (sizeof (struct
                                                             dg87_ref_condition_h),
                                                     DG87_OTHER)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->type = condition_hmax;
  condition->data.condition_hmax->minutes = minutes;
  condition->data.condition_hmin->day_delta = day_delta;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_ref_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_condition_hlate_new (int minutes, dg87ubyte day_delta)
{
  dg87_ref_condition *condition;
  if (minutes < 0 || minutes >= 24 * 60)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_ref_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_hlate =
       dg87_malloc (sizeof (struct dg87_ref_condition_h),
                    DG87_OTHER)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->type = condition_hlate;
  condition->data.condition_hlate->minutes = minutes;
  condition->data.condition_hmin->day_delta = day_delta;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_ref_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_condition_file_new (char *file_path)
{
  dg87_ref_condition *condition;
  assert (file_path != NULL);
  if (file_path == NULL || file_path[0] == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_ref_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_File = dg87_malloc (sizeof (struct
                                                             dg87_ref_condition_file),
                                                     DG87_OTHER)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_File->path =
       dg87_strdup (file_path, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->type = condition_file;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_ref_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_condition_manual_new (char *message)
{
  dg87_ref_condition *condition;
  assert (message != NULL);
  if (message == NULL || message[0] == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_ref_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_manual = dg87_malloc (sizeof (struct
                                                               dg87_ref_condition_manual),
                                                       DG87_OTHER)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_manual->message =
       dg87_strdup (message, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->type = condition_manual;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_ref_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_condition_ifequal_new (dg87_variable * variable, char *value)
{
  dg87_ref_condition *condition;
  assert (variable != NULL && value != NULL);
  if (variable == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_ref_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_ifequal = dg87_malloc (sizeof (struct
                                                                dg87_ref_condition_eq),
                                                        DG87_OTHER)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_ifequal->value =
       dg87_strdup (value, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->type = condition_ifequal;
  /* increment variable usage count */
  variable->linked_ref_conditions++;
  condition->data.condition_ifequal->variable = variable;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_ref_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_ref_condition *
dg87_ref_condition_ifnotequal_new (dg87_variable * variable, char *value)
{
  dg87_ref_condition *condition;
  assert (variable != NULL && value != NULL);
  if (variable == NULL || value == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_ref_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_ifnotequal =
       dg87_malloc (sizeof (struct dg87_ref_condition_eq),
                    DG87_OTHER)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_ifnotequal->value =
       dg87_strdup (value, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->type = condition_ifnequal;
  /* increment variable usage count */
  variable->linked_ref_conditions++;
  condition->data.condition_ifnotequal->variable = variable;

	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_ref_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
BOOL
dg87_ref_condition_delete (dg87_ref_condition * condition)
{
  assert (condition != NULL);
  if (condition == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* delete memory for structure */
  switch (condition->type)
    {
    case condition_hmin:
      if (condition->data.condition_hmin != NULL)
        dg87_free (condition->data.condition_hmin, DG87_OTHER);
      break;
    case condition_hmax:
      if (condition->data.condition_hmax != NULL)
        dg87_free (condition->data.condition_hmax, DG87_OTHER);
      break;
    case condition_hlate:
      if (condition->data.condition_hlate != NULL)
        dg87_free (condition->data.condition_hlate, DG87_OTHER);
      break;
    case condition_file:
      if (condition->data.condition_File != NULL)
        {
          if (condition->data.condition_File->path != NULL)
            dg87_free (condition->data.condition_File->path, DG87_CHAR);
          dg87_free (condition->data.condition_File, DG87_OTHER);
        }
      break;
    case condition_manual:
      if (condition->data.condition_manual != NULL)
        {
          if (condition->data.condition_manual != NULL)
            dg87_free (condition->data.condition_manual->message, DG87_CHAR);
          dg87_free (condition->data.condition_manual, DG87_OTHER);
        }
      break;
    case condition_ifequal:
      if (condition->data.condition_ifequal != NULL)
        {
          if (condition->data.condition_ifequal->variable != NULL)
            /* variable decrement usage count */
            condition->data.condition_ifequal->variable->
              linked_ref_conditions--;
          if (condition->data.condition_ifequal->value != NULL)
            dg87_free (condition->data.condition_ifequal->value, DG87_CHAR);
          dg87_free (condition->data.condition_ifequal, DG87_OTHER);
        }
      break;
    case condition_ifnequal:
      if (condition->data.condition_ifnotequal != NULL)
        {
          /* variable decrement usage count */
          if (condition->data.condition_ifnotequal->variable != NULL)
            condition->data.condition_ifnotequal->variable->
              linked_ref_conditions--;
          if (condition->data.condition_ifnotequal != NULL)
            dg87_free (condition->data.
                       condition_ifnotequal->value, DG87_CHAR);
          dg87_free (condition->data.condition_ifnotequal, DG87_OTHER);
        }
      break;

    case condition_error:
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      break;
    }
  dg87_free (condition, DG87_CONDITION);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_condition_load (dg87_ref_condition * condition,
                         dg87_storage * storage)
{
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_ref_condition_save (dg87_ref_condition * condition,
                         dg87_storage * storage)
{
  return TRUE;
}

/****************************************************************************/
dg87_ref_condition_type
dg87_ref_condition_ascii_to_type (char *condition)
{
  assert (condition != NULL);
  if (condition == NULL)
    return condition_error;

  if (strcasecmp (condition, "hmin") == 0)
    return condition_hmin;

  if (strcasecmp (condition, "hmax") == 0)
    return condition_hmax;

  if (strcasecmp (condition, "hlate") == 0)
    return condition_hlate;

  if (strcasecmp (condition, "file") == 0)
    return condition_file;

  if (strncmp (condition, "manual", 6) == 0)
    return condition_manual;
  if (strncmp (condition, "if_equal", 8) == 0)
    return condition_ifequal;
  if (strncmp (condition, "if_not_equal", 13) == 0)
    return condition_ifnequal;

  return condition_error;
}

/****************************************************************************/
char *
dg87_ref_condition_type_to_ascii (dg87_ref_condition_type condition_type)
{
  switch (condition_type)
    {
    case condition_hmin:
      return "hmin";
    case condition_hmax:
      return "hmax";
    case condition_hlate:
      return "hlate";
    case condition_file:
      return "file";
    case condition_manual:
      return "manual";
    case condition_ifequal:
      return "if_equal";
    case condition_ifnequal:
      return "if_not_equal";
    case condition_error:
      return "error";
    }
  return "error";
}

/****************************************************************************/
int
dg87_ref_condition_hour_to_minute (char *hour)
{
  char *str_hour, *str_min;
  int result = -1;
  assert (hour != NULL);
  if (hour == NULL)
    return -1;
  str_hour = strdup (hour);
  if (str_hour == NULL)
    return -1;
  str_min = strchr (str_hour, ':');
  if (str_min == NULL)
    result = atoi (str_hour);
  else
    {
      *str_min = '\0';
      result = atoi (str_hour) * 60 + atoi (str_min + 1);
    }
  free (str_hour);
  return result;
}

/****************************************************************************/
char *
dg87_ref_condition_minute_to_hour (int minutes)
{
  int h, m;
  char *str_res;
  if (minutes < 0 || minutes >= 24 * 60)
    return NULL;
  str_res = dg87_malloc (sizeof (char) * 5, DG87_CHAR);
  h = minutes / 60;
  m = minutes % 60;
  sprintf (str_res, "%.2i:%.2i", h, m);
  return str_res;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_ref_condition_check (dg87_ref_condition * condition,
                          int (*print) (char *))
{
  int minutes;
  switch (condition->type)
    {
    case condition_hmin:
      minutes = condition->data.condition_hmin->minutes;
      if (minutes < 0 || minutes >= 24 * 60)
        {
          print ("Condition : incorrect value hmin !\n");
          return 1;
        }
      break;
    case condition_hmax:
      minutes = condition->data.condition_hmax->minutes;
      if (minutes < 0 || minutes >= 24 * 60)
        {
          print ("Condition : incorrect value hmax !\n");
          return 1;
        }
      break;
    case condition_hlate:
      minutes = condition->data.condition_hlate->minutes;
      if (minutes < 0 || minutes >= 24 * 60)
        {
          print ("Condition : incorrect value hlate !\n");
          return 1;
        }
      break;
    case condition_file:
      if (condition->data.condition_File->path == NULL)
        {
          print ("Condition : Path file is NULL !\n");
          return 1;
        }
      break;
    case condition_manual:
      if (condition->data.condition_manual->message == NULL)
        {
          print ("Condition : Message is NULL !\n");
          return 1;
        }
      break;
    case condition_ifequal:
    case condition_ifnequal:
      /* TODO variable */
      break;
    default:
      print ("Condition : incorrect type !\n");

    }
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
