/* file d_job.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_REF_JOB_H__
#define __DG87_REF_JOB_H__

#include <config.h>
#include <libdg87/list.h>
#include <libdg87/storage.h>
#include "global.h"
#include "d_task.h"

#define JOB_MAX_TEMPO 3600

struct _dg87_ref_job
{
  char name[DG87_JOB_NAME_LENGTH + 1];
  char description[DG87_JOB_DESC_LENGTH + 1];
  dg87_ref_task *task;
  char *path;                   /* path for the script */
  char *argument;               /* argument for the script */
  dg87uint start_tempo;         /* temporization before execution in seconds */
  dg87uint end_tempo;           /* temporization after execution in seconds */
  BOOL wait_all_dependencies;
  BOOL on_error_continue;
};

/*
 * dg87_ref_job_new 
 * create a new job and initialize the name property
 */
dg87_ref_job *dg87_ref_job_new (dg87_ref_task * task, char *name);

/*
 * dg87_ref_job_duplicate
 * create a new job by copying another
 */
dg87_ref_job *dg87_ref_job_duplicate (dg87_ref_task * new_task,
                                      dg87_ref_job * original_job);

/*
 * dg87_ref_job_delete
 */
BOOL dg87_ref_job_delete (dg87_ref_job * job);

/*
 * dg87_ref_job_load
 */
BOOL dg87_ref_job_load (dg87_ref_job * job, dg87_storage * storage);

/*
 * dg87_ref_job_save
 */
BOOL dg87_ref_job_save (dg87_ref_job * job, dg87_storage * storage);

/*
 * dg87_ref_job_set_name
 * Change the name of a job.
 * An error occurres if another job has the same name in the task.
 */
BOOL dg87_ref_job_set_name (dg87_ref_job * job, char *name);


/*
 * dg87_ref_job_set_description
 * set the description of the job.
 */
BOOL dg87_ref_job_set_description (dg87_ref_job * job, char *description);

/*
 * dg87_ref_job_set_tempo
 * set the start_tempo and EndTempo for the job.
 */
BOOL dg87_ref_job_set_tempo (dg87_ref_job * job, int start_tempo,
                             int end_tempo);

/*
 * dg87_ref_job_set_path
 * set the path of the job.
 * path could be NULL or zero length.
 */
BOOL dg87_ref_job_set_path (dg87_ref_job * job, char *path);

/*
 * dg87_ref_job_set_argument
 * set the argument of the job.
 * argument could be NULL or zero length.
 */
BOOL dg87_ref_job_set_argument (dg87_ref_job * job, char *argument);

/*
 * dg87_ref_job_set_wait_all_dependencies
 */
BOOL dg87_ref_job_set_wait_all_dependencies (dg87_ref_job * job, BOOL value);

/*
 * dg87_ref_job_set_on_error_continue
 */
BOOL dg87_ref_job_set_on_error_continue (dg87_ref_job * job, BOOL value);

/*
 * dg87_ref_job_find_upstream_jobs
 * This function adds to the list given in parameter all upstream jobs
 * of a job.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_ref_job_find_upstream_jobs (dg87_ref_job * job,
                                      dg87_list * job_list);

/*
 * dg87_ref_job_find_downstream_jobs
 * This function adds to the list given in parameter all downstream jobs
 * of a job.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_ref_job_find_downstream_jobs (dg87_ref_job * job,
                                        dg87_list * job_list);


#ifdef DEBUG
/*
 * dg87_ref_job_ckeck
 */
int dg87_ref_job_check (dg87_ref_job * job, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_REF_JOB_H__*/
/* End of file */
