/* file d_jobdep.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_REF_JOB_DEP_H__
#define __DG87_REF_JOB_DEP_H__
#include <libdg87/global.h>
#include <libdg87/storage.h>
#include "global.h"

/* type of jobs dependencies */
typedef enum
{
  dg87_ref_jobdep_unknown,
  dg87_ref_jobdep_always,
  dg87_ref_jobdep_on_success,
  dg87_ref_jobdep_on_error
}
dg87_ref_jobdep_type;

/* structure for jobdep objects */
struct _dg87_ref_jobdep
{
  dg87_ref_job *upstream_job;
  dg87_ref_job *downstream_job;
  dg87_ref_jobdep_type type;
};

/*
 * dg87_ref_jobdep_new
 * create a new dependence between two jobs and return a pointer on it.
 */
dg87_ref_jobdep *dg87_ref_jobdep_new (dg87_ref_job * upstream_job,
                                      dg87_ref_job * downstream_job,
                                      dg87_ref_jobdep_type type);

/*
 * dg87_ref_jobdep_delete
 * free memory allocate for the object.
 */
BOOL dg87_ref_jobdep_delete (dg87_ref_jobdep * job_dep);

/*
 * dg87_ref_jobdep_load
 */
BOOL dg87_ref_jobdep_load (dg87_ref_jobdep * job_dep, dg87_storage * storage);

/*
 * dg87_ref_jobdep_save
 */
BOOL dg87_ref_jobdep_save (dg87_ref_jobdep * job_dep, dg87_storage * storage);

/*
 * dg87_ref_jobdep_set_type
 * set the type of dependence for the jobdep.
 */
BOOL dg87_ref_jobdep_set_type (dg87_ref_jobdep * job_dep,
                               dg87_ref_jobdep_type type);

/*
 * dg87_ref_jobdep_ascii_to_type
 * convert a string into a dependence type.
 * return -1 if error.
 */
dg87_ref_jobdep_type dg87_ref_jobdep_ascii_to_type (char *str_type);

/*
 * dg87_ref_jobdep_type_to_ascii
 * convert a dependence type into a string
 */
char *dg87_ref_jobdep_type_to_ascii (dg87_ref_jobdep_type dep_type);

#ifdef DEBUG
/*
 * dg87_ref_jobdep_ckeck
 */
int dg87_ref_jobdep_check (dg87_ref_jobdep * jobdep, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_REF_JOB_DEP_H__*/
/* End of file */
