/* file d_taskdep.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_REF_TASK_DEP_H__
#define __DG87_REF_TASK_DEP_H__
#include "global.h"

typedef enum
{
  dg87_ref_taskdep_unknown,
  dg87_ref_taskdep_always,
  dg87_ref_taskdep_on_success,
  dg87_ref_taskdep_on_error
}
dg87_ref_taskdep_type;

struct _dg87_ref_taskdep
{
  dg87_ref_task *upstream_task;
  dg87_ref_task *downstream_task;
  dg87_ref_taskdep_type type;
};

/*
 * dg87_ref_taskdep_new
 */
dg87_ref_taskdep *dg87_ref_taskdep_new (dg87_ref_task * upstream_task,
                                        dg87_ref_task * downstream_task,
                                        dg87_ref_taskdep_type type);

/*
 * dg87_ref_taskdep_delete
 */
BOOL dg87_ref_taskdep_delete (dg87_ref_taskdep * taskdep);

/*
 * dg87_ref_taskdep_load
 */
BOOL dg87_ref_taskdep_load (dg87_ref_taskdep * taskdep,
                            dg87_storage * storage);

/*
 * dg87_ref_taskdep_save
 */
BOOL dg87_ref_taskdep_save (dg87_ref_taskdep * taskdep,
                            dg87_storage * storage);

/*
 * dg87_ref_taskdep_set_type
 * set the type of dependence for the taskdep.
 */
BOOL dg87_ref_taskdep_set_type (dg87_ref_taskdep * taskdep,
                                dg87_ref_taskdep_type type);

/*
 * dg87_ref_taskdep_ascii_to_type
 * convert a string into a dependence type.
 * return -1 if error.
 */
dg87_ref_taskdep_type dg87_ref_taskdep_ascii_to_type (char *str_type);

/*
 * dg87_ref_taskdep_type_to_ascii
 * convert a dependence type into a string
 */
char *dg87_ref_taskdep_type_to_ascii (dg87_ref_taskdep_type type);


#ifdef DEBUG
/*
 * dg87_ref_taskdep_ckeck
 */
int dg87_ref_taskdep_check (dg87_ref_taskdep * taskdep,
                            int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_REF_TASK_DEP_H__*/
/* End of file */
