/* file engine.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_ENGINE_H__
#define __DG87_ENGINE_H__

#include <time.h>

#include <libdg87/strbuf.h>
#include <libdg87/vector.h>
#include <libdg87/server.h>
#include "global.h"
#include "d_ref.h"
#include "context.h"
#include "target.h"

struct _dg87_engine
{
  dg87_context *context;
  BOOL enabled;
  char *new_day_name;
  time_t new_day_start_time;
  dg87_strbuf *log;
  dg87_vector *variable_vector;
  time_t next_auto_plan_time;
  time_t next_auto_purge_time;
  time_t check_time;
};

/*
 * dg87_engine_new
 */
dg87_engine *dg87_engine_new (dg87_context * context);

/*
 * dg87_engine_delete
 */
BOOL dg87_engine_delete (dg87_engine * engine);

/*
 * dg87_engine_run
 * This function is call by dg87_server.
 */
BOOL dg87_engine_run (dg87_engine * engine, dg87_server * server);

/*
 * dg87_engine_plan
 */
BOOL dg87_engine_plan (dg87_engine * engine, dg87_ref_model * ref,
                       char *new_day_name, time_t start_time,
                       BOOL enable_day);

/*
 * dg87_context_auto_plan
 */
BOOL dg87_engine_auto_plan (dg87_engine * engine, BOOL force);

/*
 * dg87_engine_auto_purge
 */
BOOL dg87_engine_auto_purge (dg87_engine * engine, BOOL force);

/*
 * this functions parses InBuffer to replace variables $<VarName> by
 * its value.
 * If a variable is not declared, the $<VarName> is copied.
 */
char *dg87_engine_replace_variables (dg87_engine * engine, char *in_buffer);

/*************************************************************************/
/*  functions for variable                                               */
/*************************************************************************/

/*
 * dg87_engine_variable_add
 * create a Variable object, add this object in the VariableVector 
 * of the engine and return a pointer on this object.
 * Return NULL if the variable object can't be create.
 */
dg87_variable *dg87_engine_variable_add
  (dg87_engine * engine, char *variable_name, char *value);

/*
 * dg87_engine_variable_count
 * return the number of variables in the engine.
 * return -1 if engine is NULL.
 */
int dg87_engine_variable_count (dg87_engine * engine);


/*
 * dg87_engine_variable_get_by_name
 * return a pointer on the variable which has the name
 * given in parameter. Return NULL if it can't be find_.
 */
dg87_variable *dg87_engine_variable_get_by_name (dg87_engine * engine,
                                                 char *name);

/*
 * dg87_engine_variable_get_by_index
 * return a pointer on the variable at the given index.
 */
dg87_variable *dg87_engine_variable_get_by_index (dg87_engine * engine,
                                                  int index);


/*
 * dg87_engine_variable_delete_by_name
 * delete the variable which has the name given in parameter.
 */
BOOL dg87_engine_variable_delete_by_name (dg87_engine * engine, char *name);

/*
 * dg87_engine_variable_delete_by_index
 * delete the variable at the given index.
 */
BOOL dg87_engine_variable_delete_by_index (dg87_engine * engine, int index);

/*
 * dg87_engine_save
 */
BOOL dg87_engine_save (dg87_engine * engine, dg87_storage * storage);

/*
 * dg87_engine_load
 */
BOOL dg87_engine_load (dg87_engine * engine, dg87_storage * storage);

/*
 * dg87_engine_set_enabled
 */
BOOL dg87_engine_set_enabled (dg87_engine * engine, BOOL value);

#ifdef DEBUG
/*
 * dg87_engine_ckeck
 */
int dg87_engine_check (dg87_engine * engine, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_ENGINE_H__*/
/* End of file */
