/* file l_day.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_DAY_l_H__
#define __DG87_DAY_l_H__

#include <stdio.h>

#include <config.h>
#include "global.h"
#include "l_log.h"

/* don't forget to update dg87_log_day_entry_type_string in day.c 
 * when adding new entries */
typedef enum _dg87_log_day_entry_type
{
  dg87_log_day_entry_type_created,
  dg87_log_day_entry_type_started,
  dg87_log_day_entry_type_terminated,
  dg87_log_day_entry_type_aborted,
  dg87_log_day_entry_type_deleted,
  dg87_log_day_entry_type_canceled,
  dg87_log_day_entry_type_ignored,
  dg87_log_day_entry_type_running,
  dg87_log_day_entry_type_wait_for_condition,
  dg87_log_day_entry_type_wait_enabled,
  dg87_log_day_entry_type_output,
  dg87_log_day_entry_type_command,
  dg87_log_day_entry_type_condition,
  dg87_log_day_entry_type_wait_tempo,
  dg87_log_day_entry_type_wait_lock,
  dg87_log_day_entry_type_wait_for_dep,
  dg87_log_day_entry_type_info
}
dg87_log_day_entry_type;


struct _dg87_log_day
{
  char name[DG87_DAY_NAME_LENGTH + 1];
  dg87_log_file *log_file;
};

/*
 * dg87_log_day_new
 * create a new day and return a pointer on this.
 */
dg87_log_day *dg87_log_day_new (char *name, char *filename, uid_t uid,
                                uid_t gid);

/*
 * dg87_log_day_load
 * create a new day by loading a data file
 */
dg87_log_day *dg87_log_day_load (char *file);

/*
 * dg87_log_day_delete
 */
BOOL dg87_log_day_delete (dg87_log_day * day, BOOL delete_file);

/******************************************************************************/
/*   days entries functions                                                   */
/******************************************************************************/

/* 
 * dg87_log_day_entry_add
 */
BOOL dg87_log_day_entry_add (dg87_log_day * day, dg87_log_day_entry_type type,
                             time_t date, char *data);

/*
 * dg87_log_day_entry_read_first
 */
BOOL dg87_log_day_entry_read_first (dg87_log_day * day,
                                    dg87_log_entry * entry);


/*
 * dg87_log_day_entry_read_next
 */
BOOL dg87_log_day_entry_read_next (dg87_log_day * day,
                                   dg87_log_entry * entry);

/*
 * dg87_log_day_entry_get_numbers
 */
BOOL dg87_log_day_entry_get_numbers (dg87_log_day * day, dg87uint * number);

/******************************************************************************/
/*   tasks functions                                                          */
/******************************************************************************/

/*
 * dg87_log_day_task_add
 */
BOOL dg87_log_day_task_add (dg87_log_day * day, char *task_name);

/*
 * dg87_log_day_task_read_first
 */
BOOL dg87_log_day_task_read_first (dg87_log_day * day,
                                   dg87_log_node * task_node);


/*
 * dg87_log_day_task_read_next
 */
BOOL dg87_log_day_task_read_next (dg87_log_day * day,
                                  dg87_log_node * task_node);


/*
 * dg87_log_day_task_get_count
 */
BOOL dg87_log_day_task_get_count (dg87_log_day * day, dg87uint * number);

/*
 * dg87_log_day_task_get_by_name
 */
BOOL dg87_log_day_task_get_by_name (dg87_log_day * day, char *name,
                                    dg87_log_node * task_node);

/******************************************************************************/
/*   tasks entries functions                                                  */
/******************************************************************************/

/*
 * dg87_log_day_task_entry_add
 */
BOOL dg87_log_day_task_entry_add (dg87_log_day * day,
                                  dg87_log_node * task_node,
                                  dg87_log_day_entry_type type, time_t date,
                                  char *data);

/*
 * dg87_log_day_task_entry_read_first
 */
BOOL dg87_log_day_task_entry_read_first (dg87_log_day * day,
                                         dg87_log_node * task_node,
                                         dg87_log_entry * entry);

/*
 * dg87_log_day_task_entry_read_next
 */
BOOL dg87_log_day_task_entry_read_next (dg87_log_day * day,
                                        dg87_log_node * task_node,
                                        dg87_log_entry * entry);

/*
 * dg87_log_day_task_entry_count
 */
BOOL dg87_log_day_task_entry_count (dg87_log_day * day,
                                    dg87_log_node * task_node,
                                    dg87uint * number);

/******************************************************************************/
/*   jobs functions                                                          */
/******************************************************************************/

/*
 * dg87_log_day_job_add
 */
BOOL dg87_log_day_job_add (dg87_log_day * day, dg87_log_node * task_node,
                           char *job_name);

/*
 * dg87_log_day_job_read_first
 */
BOOL dg87_log_day_job_read_first (dg87_log_day * day,
                                  dg87_log_node * task_node,
                                  dg87_log_node * job_node);

/*
 * dg87_log_day_job_read_next
 */
BOOL dg87_log_day_job_read_next (dg87_log_day * day,
                                 dg87_log_node * job_node);


/*
 * dg87_log_day_job_get_count
 */
BOOL dg87_log_day_job_get_count (dg87_log_day * day,
                                 dg87_log_node * task_node,
                                 dg87uint * number);

/*
 * dg87_log_day_job_get_by_name
 */
BOOL dg87_log_day_job_get_by_name (dg87_log_day * day,
                                   dg87_log_node * task_node, char *name,
                                   dg87_log_node * job_node);

/******************************************************************************/
/*   jobs entries functions                                                  */
/******************************************************************************/

/*
 * dg87_log_day_job_entry_add
 */
BOOL dg87_log_day_job_entry_add (dg87_log_day * day, dg87_log_node * job_node,
                                 dg87_log_day_entry_type type, time_t date,
                                 char *data);

/*
 * dg87_log_day_job_entry_read_first
 */
BOOL dg87_log_day_job_entry_read_first (dg87_log_day * day,
                                        dg87_log_node * job_node,
                                        dg87_log_entry * entry);

/*
 * dg87_log_day_job_entry_read_next
 */
BOOL dg87_log_day_job_entry_read_next (dg87_log_day * day,
                                       dg87_log_node * job_node,
                                       dg87_log_entry * entry);

/*
 * dg87_log_day_job_entry_count
 */
BOOL dg87_log_day_job_entry_count (dg87_log_day * day,
                                   dg87_log_node * job_node,
                                   dg87uint * number);


/******************************************************************************/
/*   other functions                                                          */
/******************************************************************************/

/*
 * dg87_log_day_ascii_to_entry_type
 * convert a string to a entry type
 */
dg87_log_day_entry_type dg87_log_day_ascii_to_entry_type (char *string);

/*
 * dg87_log_day_entry_type_to_ascii
 * convert an entry type to a string
 */
char *dg87_log_day_entry_type_to_ascii (dg87_log_day_entry_type entry_type);


#ifdef DEBUG
/*
 * dg87_log_day_ckeck
 */
int dg87_log_day_check (dg87_log_day * day, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_DAY_l_H__*/
/* End of file */
