/* file l_log.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_LOG_l_H__
#define __DG87_LOG_l_H__

#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <libdg87/global.h>

#define DG87_FILEPOS long

#define DG87L_LOGNODE_NAME_LENGTH 127
#define DG87L_ROOT_NODE_POSITION 10

/* 
 * LOG FILE FORMAT
 *
 * header file (7 bytes)
 *
 * 01-05 (05) : signature (dg87l)
 * 06-07 (02) : version number (int)
 * 08-09      : NULL character (reserved)
 * 10...      : root node
 *
 * node ( > 25 bytes)
 *
 * 01-04 (04) : file position of next node (brother)
 * 05-06 (02) : CHILDREN number of children nodes
 * 07-10 (04) :          file position of first child node
 * 11-14 (04) :          file position of last child node
 * 15-16 (02) : ENTRY number of entries
 * 17-20 (04) :       index of first entry
 * 21-24 (04) :       index of last entry
 * 25    (01) : name size ( < 255 characters)
 * 26 ...     : name
 */

/* entry
 *
 * 01-04 (04) : file position of next entry
 * 05-06 (02) : date
 * 07    (01) : type 
 * 08    (01) : data size ( <255 characters)
 * 09-... : datas
 */

/*
 * dg87_log_file object structure
 */
typedef struct _dg87_log_file
{
	FILE* file_stream;
  char *filename;
  DG87_FILEPOS root_node;
}
dg87_log_file;

/*
 * dg87_log_node object structure
 */
typedef struct _dg87_log_node
{
  DG87_FILEPOS position;        /* position of this node in the file */
  DG87_FILEPOS next_node;
  /* CHILDREN NODES */
  dg87uint children_number;
  DG87_FILEPOS first_child_node;
  DG87_FILEPOS last_child_node;
  /* ENTRIES */
  dg87uint entries_number;
  DG87_FILEPOS first_entry;
  DG87_FILEPOS last_entry;
  char name[DG87L_LOGNODE_NAME_LENGTH + 1];
}
dg87_log_node;

/*
 * dg87_log_entry object structure
 */
typedef struct _dg87_log_entry
{
  DG87_FILEPOS position;
  DG87_FILEPOS next_entry;
  dg87long date;
  dg87ubyte type;
  char data[255];
}
dg87_log_entry;

/*
 * dg87_log_file_create
 */
dg87_log_file *dg87_log_file_create (char *filename, uid_t uid, uid_t gid);

/*
 * dg87_log_file_open
 */
dg87_log_file *dg87_log_file_open (char *filename);

/*
 * dg87_log_file_close
 */
BOOL dg87_log_file_close (dg87_log_file * log_file, BOOL delete_file);

/*******************************************************************************
 * nodes functions
 ******************************************************************************/

/*
 * dg87_log_file_add_node
 */
BOOL dg87_log_file_add_node (dg87_log_file * log_file, char *name,
                             dg87_log_node * parent_node,
                             dg87_log_node * result_node);

/*
 * dg87_log_file_get_first_child
 */
BOOL dg87_log_file_get_first_child (dg87_log_file * log_file,
                                    dg87_log_node * parent_node,
                                    dg87_log_node * first_child_node);

/*
 * dg87_log_file_get_next_node
 */
BOOL dg87_log_file_get_next_node (dg87_log_file * log_file,
                                  dg87_log_node * node);

/*******************************************************************************
 * entries functions
 ******************************************************************************/

/*
 * dg87_log_file_add_entry
 */
BOOL dg87_log_file_add_entry (dg87_log_file * log_file,
                              time_t date, int type, char *data,
                              dg87_log_node * node);

/*
 * dg87_log_file_get_first_entry
 */
BOOL dg87_log_file_get_first_entry (dg87_log_file * log_file,
                                    dg87_log_node * parent_node,
                                    dg87_log_entry * first_entry);

/*
 * dg87_log_file_get_next_entry
 */
BOOL dg87_log_file_get_next_entry (dg87_log_file * log_file,
                                   dg87_log_entry * entry);

#endif /*__DG87_LOG_l_H__*/
/* End of file */
