/* file r_caller.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <config.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <assert.h>

#include "context.h"
#include "r_caller.h"
#include "r_file.h"
#include "x_job.h"
#include "x_task.h"
#include "x_day.h"
#include "l_day.h"
#include <libdg87/daemon.h>
#include <libdg87/server.h>

/******************************************************************************/
BOOL
dg87_r_caller_transmit_state (dg87_context * context, dg87_r_job * job)
{
  char *buffer;
  int size;
	dg87_exp_day *exp_day;
	dg87_exp_task *exp_task;
	dg87_exp_job *exp_job;
	dg87_exp_job_state new_state;

  assert (context != NULL && job != NULL);
  if (context == NULL || job == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
	if (strcmp(job->hostname, "")  == 0 && job->ip_port == 0)
		{
			/* execution on the same daemon */
			/* find the day */
			exp_day = dg87_context_exp_day_get_by_name(context, job->day_name);
			if (exp_day == NULL)
				return FALSE;

			/* find the task */
			exp_task = dg87_exp_day_task_get_by_name(exp_day, job->task_name);
			if (exp_task == NULL)
				return FALSE;

			/* find the job */
			exp_job = dg87_exp_task_job_get_by_name(exp_task, job->job_name);
			if (exp_job == NULL)
				return FALSE;

			/* update the state */
			new_state = dg87_exp_job_ascii_to_state (dg87_r_job_state_to_ascii (job->state));
			dg87_exp_job_set_state(exp_job, new_state, job->status);

		}
	else
		{
			/* execution on a remote daemon : send a xju command message */

			/*xju "DayName" "TaskName" "JobName" "State" "[exit_code]" */
			size = strlen (job->day_name) + strlen (job->task_name) +
					strlen (job->job_name) +
					strlen (dg87_r_job_state_to_ascii (job->state)) + 50;
			if ((buffer = dg87_malloc (size, DG87_CHAR)) == NULL)
				{
					dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
					return FALSE;
				}
			snprintf (buffer, size, "xju \"%s\" \"%s\" \"%s\" \"%s\" \"%i\"",
										job->day_name, job->task_name, job->job_name,
										dg87_r_job_state_to_ascii (job->state), job->status);
			if (dg87_server_post_message (context->server,
																job->hostname, job->ip_port, buffer) == FALSE)
				{
					dg87_free (buffer, DG87_CHAR);
					return FALSE;
				}
			dg87_free (buffer, DG87_CHAR);
		}
  return TRUE;

}

/****************************************************************************/
BOOL
dg87_r_caller_send_log_entry (dg87_context * context,
                              dg87_r_job * job, 
															dg87_log_day_entry_type entry_type,
                              char *message)
{
  char *p;
  char *str_date;
  dg87_strbuf *strbuf;
  dg87_log_day *log_day;
	dg87_log_node task_node;
	dg87_log_node job_node;
  assert (context != NULL && job != NULL && message != NULL);
  if (context == NULL || job == NULL || message == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* replace bad characters by spaces */
  p = message;
  while (*p != '\0')
    {
      if (*p < 32)
        *p = ' ';
      if (*p == '\"')
        *p = '\'';
      p++;
    }


	if (strcmp(job->hostname, "")  == 0 && job->ip_port == 0)
		{
			/* log on the same daemon */
  		log_day = dg87_context_log_day_get_by_name (context, job->day_name);
  		if (log_day == NULL)
							return FALSE;
  		if (dg87_log_day_task_get_by_name (log_day, 
											job->task_name, &task_node) == FALSE)
							return FALSE;
  		if (dg87_log_day_job_get_by_name (log_day, 
											&task_node, job->job_name, &job_node) == FALSE)
							return FALSE;
  		if (dg87_log_day_job_entry_add (log_day, &job_node, entry_type,
                                  dg87_current_time, message) == FALSE)
							return FALSE;
		}
	else
		{
  		if ((strbuf = dg87_strbuf_new ()) == NULL)
    		return FALSE;
  		str_date = dg87_date_to_ascii (dg87_current_time);
  		dg87_strbuf_printf (strbuf,
                      "ljea \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
                      job->day_name, job->task_name, job->job_name,
                      str_date, 
											dg87_log_day_entry_type_to_ascii(entry_type), 
											message);
  		dg87_server_post_message (context->server,
                            job->hostname, job->ip_port, strbuf->buffer);
  		dg87_free (str_date, DG87_CHAR);
  		dg87_strbuf_delete (strbuf);
		}
  return FALSE;
}

/******************************************************************************/
BOOL
dg87_r_caller_transmit_file (dg87_context * context, dg87_r_file * file)
{
  char *buffer;
  int size;

  assert (context != NULL && file != NULL);
  if (context == NULL || file == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
	/*xtcv "DayName" "TaskName" file "Path" */
	size = strlen (file->day_name) + strlen (file->task_name) +
	strlen (file->path) + 50;
	buffer = dg87_malloc (size, DG87_CHAR);
	if (buffer == NULL)
	{
		dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
		return FALSE;
	}
	snprintf (buffer, size, "xtcv \"%s\" \"%s\" file \"%s\"",
					file->day_name, file->task_name, file->path);
	if (dg87_server_post_message (context->server,
																file->hostname, file->ip_port,
																buffer) == FALSE)
	{
		dg87_free (buffer, DG87_CHAR);
		return FALSE;
	}

	dg87_free (buffer, DG87_CHAR);
  return TRUE;
}

/******************************************************************************/
/* End of file */
