/* file r_cmdjob.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include <libdg87/daemon.h>
#include <libdg87/command.h>
#include <libdg87/daemon.h>     /* log function */

#include "context.h"
#include "util.h"
#include "r_cmdjob.h"

/******************************************************************************/
/* jk
 * Job Kill
 * Usage : jk TaskName JobName DayName
 */
int
cmd_jk (dg87_connection * connection)
{
  dg87_r_job *job;
  dg87_context *context;
  dg87_command *command;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "jk TaskName JobName DayName\n");
      return 1;
    }

  /* find the task */
  job = dg87_r_util_find_job (connection, command->argv[0],     /* task name */
                              command->argv[1], /* job name */
                              command->argv[2]);        /* day name */
  if (job == NULL)
    return 1;

  /* delete the job */
  dg87_err_clear_error ();
  context = connection->server->data;
  if (dg87_r_context_job_delete (context, job) == FALSE)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't delete the job '%s' '%s' '%s' ! %s\n"),
                              command->argv[0],
                              command->argv[1],
                              command->argv[2], dg87_err_get_error_str ());
      return 1;
    }

  return 0;
}

/******************************************************************************
 * cmd_ja
 * Job Add
 * Usage : ja DayName TaskName JobName Path Argument HostName ip_port
 */
int
cmd_ja (dg87_connection * connection)
{
  dg87_context *context;
  dg87_command *command;
  dg87_r_job *job;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 7)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "ja DayName TaskName JobName Path Argument HostName ip_port");
      return 1;
    }

  /* TODO control if the same job is already submited :
   * same day, same task, same job*/

  /* add the job to the task */
  dg87_err_clear_error ();
  context = connection->server->data;
  job = dg87_r_context_job_add (context, 
																command->argv[0],       /* day name */
                                command->argv[1],       /* task name */
                                command->argv[2],       /* job name */
                                command->argv[3],       /* path */
                                command->argv[4],       /* argument */
                                command->argv[5],       /* HostName */
                                atoi (command->argv[6]));       /* ip Port */
  if (job == NULL)
    {
      dg87_connection_printf (connection,
                              gettext ("Can't add the job '%s' '%s' '%s' ! %s\n"),
                              command->argv[0],
                              command->argv[1],
                              command->argv[2], dg87_err_get_error_str ());
      return 1;
    }
  return 0;
}

/******************************************************************************/
/* cmd_jc
 * Usage : rjc
 * Job Count
 * return the number of jobs.
 */
int
cmd_jc (dg87_connection * connection)
{
  dg87_context *context;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  context = connection->server->data;
  dg87_connection_printf (connection, "%i\n", context->job_list->count);
  return 0;
}

/******************************************************************************/
/* cmd_jl
 * Usage : jl [options]
 * Job List
 * Options
 * -d : print description of job 
 * -t : print task name of job
 * -h : print hostname and ip_port
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first job to display (first index is 0)
 * -Nnumber : number of job to display (no limit if -N < 1)
 * -s : state
 */
int
cmd_jl (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  int start = 0;                /* index of the first task to display */
  int number = 0;               /* number of jobs to display */
  int count = 0;                /* number of job actually display */
  dg87_item *item = NULL;
  dg87_r_job *job;
  dg87_context *context;
  dg87_command *command;
  BOOL show_state = FALSE;
  BOOL show_host = FALSE;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  context = connection->server->data;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "jl [options]");
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        case 'I':              /* index of the first job to display */
          start = atoi (optarg);
          break;
        case 'N':              /* number of job to display */
          number = atoi (optarg);
          break;
        case 's':              /* show state */
          show_state = TRUE;
          break;
        case 'h':              /* show hostname and IP port */
          show_host = TRUE;
          break;

        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }

  /* find the first job to display */
  item = NULL;
  if (start > 0)
    item = dg87_list_get_item (context->job_list, start);
  else
    item = context->job_list->first;

  /* create the list */
  count = 0;
  while (item != NULL)
    {
      /* reach the limit of job to display */
      count++;
      if (number > 0 && count > number)
        break;
      job = item->object;

      /* print DayName, TaskName, JobName */
      dg87_connection_printf (connection, "%s%s%s%s%s%s",
                              beg, job->day_name,
                              sep, job->task_name, sep, job->job_name);

      /* print host name and IP port */
      if (show_host == TRUE)
        {
          dg87_connection_printf (connection, "%s%s%s%i",
                                  sep, job->hostname, sep, job->ip_port);
        }

      /* print state */
      if (show_state == TRUE)
        {
          dg87_connection_printf (connection, "%s%s", sep,
                                  dg87_r_job_state_to_ascii (job->state));
        }

      dg87_connection_printf (connection, "%s\n", end);
      item = item->next;
    }
  /* free memory */
  free (sep);
  free (beg);
  free (end);
  return 0;
}


/******************************************************************************/
/* cmd_ji
 * job information 
 * get information for a job
 * Usage : ji [options] DayName TaskName JobName 
 * Options :
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int
cmd_ji (dg87_connection * connection)
{
  char *sep, *beg, *end;        /* separator, begin, end */
  int optch, iopt;
  dg87_r_job *job;
  dg87_command *command;

  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 3)
    {
      dg87_connection_printf (connection,
                              gettext ("Incorrect call.\nUsage : %s\n"),
                              "ji [options] DayNameTaskName JobName");
      return 1;
    }

  /* find the job */
  job = dg87_r_util_find_job (connection, command->argv[0],
                              command->argv[1], command->argv[2]);
  if (job == NULL)
    return 1;

  /* default separator */
  sep = strdup (":\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      switch (optch)
        {
        case 'S':              /* separator */
          free (sep);
          sep = strdup (optarg);
          break;
        case 'B':              /* begin separator */
          free (beg);
          beg = strdup (optarg);
          break;
        case 'E':              /* end separator */
          free (end);
          end = strdup (optarg);
          break;
        default:               /* unknown option */
          free (sep);
          free (beg);
          free (end);
          return 1;
        }
    }
  /* day name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Day Name", sep, job->day_name, end);
  /* Task name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Task Name", sep, job->task_name, end);
  /* job name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Job Name", sep, job->job_name, end);
  /* Path */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Path", sep, job->path, end);
  /* Argument */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Argument", sep, job->argument, end);
  /* pid */
  dg87_connection_printf (connection, "%s%s%s%l%s\n",
                          beg, "Pid", sep, job->pid, end);
  /* state */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "State", sep,
                          dg87_r_job_state_to_ascii (job->state), end);
  dg87_connection_printf (connection, "%s%s%s%i%s\n", beg, "State", sep,
                          job->status, end);
  /* Host name */
  dg87_connection_printf (connection, "%s%s%s%s%s\n",
                          beg, "Host name", sep, job->hostname, end);
  /* Ip Port */
  dg87_connection_printf (connection, "%s%s%s%i%s\n",
                          beg, "IP port", sep, job->ip_port, end);

  /* free memory */
  free (sep);
  free (beg);
  free (end);

  return 0;
}

/******************************************************************************/
/* End of file */
