/* file r_file.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <assert.h>
#include <unistd.h>

#include "r_caller.h"
#include "r_file.h"

/****************************************************************************/
dg87_r_file *
dg87_r_file_new (char *day_name, char *task_name, char *path,
                 char *hostname, int ip_port)
{
  dg87_r_file *file;
  assert (day_name != NULL && task_name != NULL && path != NULL
          && hostname != NULL);
  if (day_name == NULL || task_name == NULL || path == NULL
      || hostname == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* path length can not be zero */
  if (strlen (path) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* check task name and day name */
  if (dg87_check_name (task_name, DG87_TASK_NAME_LENGTH) == FALSE)
    return NULL;
  if (dg87_check_name (day_name, DG87_DAY_NAME_LENGTH) == FALSE)
    return NULL;

  /* allocate memory */
  if ((file = dg87_malloc (sizeof (dg87_r_file), DG87_FILE)) == NULL)
    goto alloc_error;
  /* copy path */
  if ((file->path = dg87_strdup (path, DG87_CHAR)) == NULL)
    goto alloc_error;
  if ((file->hostname = dg87_strdup (hostname, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize object */
  dg87_strnzcpy (file->task_name, task_name, DG87_TASK_NAME_LENGTH);
  dg87_strnzcpy (file->day_name, day_name, DG87_DAY_NAME_LENGTH);
  file->ip_port = ip_port;
	DG87_SAVE_NEEDED;
  return file;

alloc_error:
  if (file != NULL)
    dg87_r_file_delete (file);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
BOOL
dg87_r_file_delete (dg87_r_file * file)
{
  assert (file != NULL);
  if (file == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (file->path != NULL)
    dg87_free (file->path, DG87_CHAR);
  if (file->hostname != NULL)
    dg87_free (file->hostname, DG87_CHAR);
  dg87_free (file, DG87_FILE);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_r_file_check_presence (dg87_r_file * file)
{
  struct stat sb;
  assert (file != NULL && file->path != NULL);
  if (file == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  if (stat (file->path, &sb) == 0)
    {
			dg87_daemon_log_printf(dg87_daemon_log_debug,
      	"File '%s' is present !", file->path);
      return TRUE;
    }
  return FALSE;
}

/****************************************************************************/
BOOL
dg87_r_file_load (dg87_r_file * file, dg87_storage * storage)
{
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_r_file_save (dg87_r_file * file, dg87_storage * storage)
{
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_r_file_check (dg87_r_file * file, int (*print) (char *))
{
  if (file->path == NULL)
    {
      print ("File : path is NULL !\n");
      return 1;
    }
  if (strlen (file->path) == 0)
    {
      print ("File : path is zero length !\n");
      return 1;
    }

  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
