/* file r_job.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_JOB_r_H__
#define __DG87_JOB_r_H__

#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <config.h>
#include <libdg87/global.h>
#include <libdg87/storage.h>
#include <libdg87/strbuf.h>
#include "global.h"

typedef enum
{
  dg87_r_job_state_uninitialize,/* job is not started */
  dg87_r_job_state_deleted,     /* job could not be executed */
  dg87_r_job_state_running,     /* job is running */
  dg87_r_job_state_exit,        /* job is finished */
  dg87_r_job_state_old          /* can be remove from the job list */
}
dg87_r_job_state;

struct _dg87_r_job
{
  char job_name[DG87_JOB_NAME_LENGTH + 1];
  char task_name[DG87_TASK_NAME_LENGTH + 1];
  char day_name[DG87_DAY_NAME_LENGTH + 1];
  char *path;                   /* path for the script */
  char *argument;               /* argument for the script */
  char *hostname;
  int ip_port;
  dg87_r_job_state state;
  pid_t pid;                    /* process id */
  int status;
  BOOL transmit_state;
  int fd;                       /* pipe to read stdin and stdout of job */
  dg87_strbuf *output;          /* to store jobs outputs */
};

/*
 * dg87_r_job_new 
 * create a new job and initialize the name property
 */
dg87_r_job *dg87_r_job_new (char *day_name, char *task_name, char *job_name,
                            char *path, char *argument, char *hostname,
                            int ip_port);

/*
 * dg87_r_job_delete
 */
BOOL dg87_r_job_delete (dg87_r_job * job);

/*
 * dg87_r_job_load
 */
BOOL dg87_r_job_load (dg87_r_job * job, dg87_storage * storage);

/*
 * dg87_r_job_save
 */
BOOL dg87_r_job_save (dg87_r_job * job, dg87_storage * storage);

/*
 * dg87_r_job_run
 */
BOOL dg87_r_job_run (dg87_r_job * job, dg87_context * context);

/*
 * dg87_r_job_catch_output
 */
BOOL dg87_r_job_catch_output (dg87_r_job * job, dg87_context * context);

/*
 * dg87_r_job_change_state
 * change the value of the state and tell
 * to transfer this state to diogene
 */
BOOL dg87_r_job_change_state (dg87_r_job * job, dg87_r_job_state state);

/*
 * dg87_r_job_transmit_state
 * contact diogene's server if necessary to update the state of the job
 */
BOOL dg87_r_job_transmit_state (dg87_context * context, dg87_r_job * job);

/*
 * dg87_r_job_state_to_ascii
 * convert a job's state to a string
 */
char *dg87_r_job_state_to_ascii (dg87_r_job_state state);

#ifdef DEBUG
/*
 * dg87_r_job_ckeck
 */
int dg87_r_job_check (dg87_r_job * job, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_JOB_r_H__*/
/* End of file */
