/* file s_ref.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_REF_s_H__
#define __DG87_REF_s_H__

#include <stdio.h>

#include <config.h>
#include "global.h"
#include "s_stat.h"

struct _dg87_sta_ref
{
  char name[DG87_REF_NAME_LENGTH + 1];
  dg87_statfile *stat_file;
};

/*
 * dg87_sta_ref_new
 * create a new ref and return a pointer on this.
 */
dg87_sta_ref *dg87_sta_ref_new (char *name, char *filename, uid_t uid,
                                uid_t gid);

/*
 * dg87_sta_ref_load
 * create a new ref by loading a data file
 */
dg87_sta_ref *dg87_sta_ref_load (char *file);

/*
 * dg87_sta_ref_delete
 */
BOOL dg87_sta_ref_delete (dg87_sta_ref * ref, BOOL delete_file);

/******************************************************************************/
/*   tasks functions                                                          */
/******************************************************************************/

/*
 * dg87_sta_ref_task_add
 */
BOOL dg87_sta_ref_task_add (dg87_sta_ref * ref, char *task_name);

/*
 * dg87_sta_ref_task_read_first
 */
BOOL dg87_sta_ref_task_read_first (dg87_sta_ref * ref,
                                   dg87_statnode * task_node);


/*
 * dg87_sta_ref_task_read_next
 */
BOOL dg87_sta_ref_task_read_next (dg87_sta_ref * ref,
                                  dg87_statnode * task_node);


/*
 * dg87_sta_ref_task_get_count
 */
BOOL dg87_sta_ref_task_get_count (dg87_sta_ref * ref, dg87uint * number);

/*
 * dg87_sta_ref_task_get_by_name
 */
BOOL dg87_sta_ref_task_get_by_name (dg87_sta_ref * ref, char *name,
                                    dg87_statnode * task_node);

/*
 * dg87_sta_ref_task_update
 */
BOOL dg87_sta_ref_task_update (dg87_sta_ref * ref, dg87_statnode * task_node,
                               time_t start_time, dg87uint duration);

/*
 * dg87_sta_ref_task_stat
 */
BOOL dg87_sta_ref_task_stat (dg87_sta_ref * ref, dg87_statnode * task_node,
                             dg87uint * average, dg87uint * mini,
                             dg87uint * maxi);

/******************************************************************************/
/*   jobs functions                                                          */
/******************************************************************************/

/*
 * dg87_sta_ref_job_add
 */
BOOL dg87_sta_ref_job_add (dg87_sta_ref * ref, dg87_statnode * task_node,
                           char *job_name);

/*
 * dg87_sta_ref_job_read_first
 */
BOOL dg87_sta_ref_job_read_first (dg87_sta_ref * ref,
                                  dg87_statnode * task_node,
                                  dg87_statnode * job_node);

/*
 * dg87_sta_ref_job_read_next
 */
BOOL dg87_sta_ref_job_read_next (dg87_sta_ref * ref,
                                 dg87_statnode * job_node);


/*
 * dg87_sta_ref_job_get_count
 */
BOOL dg87_sta_ref_job_get_count (dg87_sta_ref * ref,
                                 dg87_statnode * task_node,
                                 dg87uint * number);

/*
 * dg87_sta_ref_job_get_by_name
 */
BOOL dg87_sta_ref_job_get_by_name (dg87_sta_ref * ref,
                                   dg87_statnode * task_node, char *name,
                                   dg87_statnode * job_node);

/*
 * dg87_sta_ref_job_update
 */
BOOL dg87_sta_ref_job_update (dg87_sta_ref * ref, dg87_statnode * job_node,
                              time_t start_time, dg87uint duration);

/*
 * dg87_sta_ref_job_stat
 */
BOOL dg87_sta_ref_job_stat (dg87_sta_ref * ref, dg87_statnode * job_node,
                            dg87uint * average, dg87uint * mini,
                            dg87uint * maxi);

#ifdef DEBUG
/*
 * dg87_sta_ref_ckeck
 */
int dg87_sta_ref_check (dg87_sta_ref * ref, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_REF_s_H__*/
/* End of file */
