/* file s_stat.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_STAT_s_H__
#define __DG87_STAT_s_H__

#include <time.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <libdg87/global.h>

#define DG87_FILEPOS long

#define DG87S_STATNODE_NAME_LENGTH 127
#define DG87S_ROOT_NODE_POSITION 10
#define DG87_STATFILE_STATENTRYCOUNT 30

/* 
 * STAT FILE FORMAT
 *
 * header file (7 bytes)
 *
 * 01-05 (05) : signature (dg87s)
 * 06-07 (02) : version number (int)
 * 08-09      : NULL character (reserved)
 * 10...      : root node
 *
 * node ( > 25 bytes)
 *
 * 01-04 (04) : file position of next node (brother)
 * 05-06 (02) : CHILDREN number of children nodes
 * 07-10 (04) :          file position of first child node
 * 11-14 (04) :          file position of last child node
 * 15    (01) : name size ( < 255 characters)
 * 16 ...     : name
 */


/*
 * dg87_statfile_statentry
 */
typedef struct dg87_statfile_statentry
{
  time_t start_time;
  dg87uint duration;
}
dg87_statfile_statentry;

/*
 * dg87_statfile object structure
 */
typedef struct _dg87_statfile
{
	FILE* file_stream;
  char *filename;
  DG87_FILEPOS root_node;
}
dg87_statfile;

/*
 * dg87_statnode object structure
 */
typedef struct _dg87_statnode
{
  DG87_FILEPOS position;        /* position of this node in the file */
  DG87_FILEPOS next_node;
  /* CHILDREN NODES */
  dg87uint children_number;
  DG87_FILEPOS first_child_node;
  DG87_FILEPOS last_child_node;
  char name[DG87S_STATNODE_NAME_LENGTH + 1];
  dg87_statfile_statentry stat_entries[DG87_STATFILE_STATENTRYCOUNT];
}
dg87_statnode;

/*
 * dg87_statfile_create
 */
dg87_statfile *dg87_statfile_create (char *filename, uid_t uid, uid_t gid);

/*
 * dg87_statfile_open
 */
dg87_statfile *dg87_statfile_open (char *filename);

/*
 * dg87_statfile_close
 */
BOOL dg87_statfile_close (dg87_statfile * stat_file, BOOL delete_file);

/*******************************************************************************
 * nodes functions
 ******************************************************************************/

/*
 * dg87_statfile_add_node
 */
BOOL dg87_statfile_add_node (dg87_statfile * stat_file, char *name,
                             dg87_statnode * parent_node,
                             dg87_statnode * result_node);

/*
 * dg87_statfile_get_first_child
 */
BOOL dg87_statfile_get_first_child (dg87_statfile * stat_file,
                                    dg87_statnode * parent_node,
                                    dg87_statnode * first_child_node);

/*
 * dg87_statfile_get_next_node
 */
BOOL dg87_statfile_get_next_node (dg87_statfile * stat_file,
                                  dg87_statnode * node);

/*
 * dg87_statfile_update_node
 */
BOOL dg87_statfile_update_node (dg87_statfile * stat_file,
                                dg87_statnode * node, time_t start_time,
                                dg87uint duration);

/* 
 * dg87_statfile_stat_node
 */
BOOL dg87_statfile_stat_node (dg87_statfile * stat_file, dg87_statnode * node,
                              dg87uint * average, dg87uint * min,
                              dg87uint * max);


#endif /*__DG87_STAT_s_H__*/
/* End of file */
