/* file util.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_UTIL_H__
#define __DG87_UTIL_H__

#include "global.h"
#include <libdg87/connection.h>
#include "target.h"
#include "queue.h"
#include "d_job.h"
#include "d_ref.h"
#include "variable.h"
#include <stdlib.h>
#include "s_ref.h"
#include "l_day.h"

/******************************************************************************/
/* dg87_util_find_ref_model
 * return the Ref which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find_.
 */
dg87_ref_model *dg87_util_find_ref_model (dg87_connection * connection,
                                          char *name);

/******************************************************************************/
/* dg87_util_find_target
 * return the Target which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find_.
 */
dg87_target *dg87_util_find_target (dg87_connection * connection, char *name);

/******************************************************************************/
/* dg87_util_find_queue
 * return the Queue which correspond to the name given in parameter.
 * The Target must be pass as parameter.
 * Print an error message and return NULL if it can't be find_.
 */
dg87_queue *dg87_util_find_queue (dg87_connection * connection,
                                  dg87_target * target, char *queue_name);

/******************************************************************************/
/* dg87_util_find_ref_model_task 
 * find_ the task which has the given name (name) in the given ref (ref).
 * Print an error message and return NULL if it can't be find_.
 */
dg87_ref_task *dg87_util_find_ref_model_task (dg87_connection * connection,
                                              dg87_ref_model * ref,
                                              char *name);
/******************************************************************************/
/* dg87_util_find_ref_job
 * return the job which correspond to the name given in parameter.
 * The Task must be pass as parameter.
 * Print an error message and return NULL if it can't be find_.
 */
dg87_ref_job *dg87_util_find_ref_job (dg87_connection * connection,
                                      dg87_ref_task * task, char *job_name);
/******************************************************************************/
/* dg87_util_find_ref_rule
 * find_ the rule which has the given name (name) in the given ref (ref).
 * Print an error message and return NULL if it can't be find_.
 */
dg87_ref_rule *dg87_util_find_ref_rule (dg87_connection * connection,
                                        dg87_ref_model * ref, char *name);

/******************************************************************************/
/* dg87_util_find_variable
 * find_ the variable which has the given name (name).
 * Print an error message and return NULL if it can't be find_.
 */
dg87_variable *dg87_util_find_variable (dg87_connection * connection,
                                        char *name);

/******************************************************************************/
/* dg87_util_find_exp_day
 * return the Day which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find_.
 */
dg87_exp_day *dg87_util_find_exp_day (dg87_connection * connection,
                                      char *name);

/******************************************************************************/
/* dg87_util_find_exp_task 
 * find_ the task which has the given name (name) in the given day (day).
 * Print an error message and return NULL if it can't be find_.
 */
dg87_exp_task *dg87_util_find_exp_task (dg87_connection * connection,
                                        dg87_exp_day * day, char *name);

/******************************************************************************/
/* dg87_util_find_exp_job
 * find_ the job which has the given name (name) in the given task (task).
 * Print an error message and return NULL if it can't be find_.
 */
dg87_exp_job *dg87_util_find_exp_job (dg87_connection * connection,
                                      dg87_exp_task * task, char *name);

/******************************************************************************/
/* dg87_util_find_sta_referential
 * return the Ref which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find_.
 */
dg87_sta_ref *dg87_util_find_sta_referential (dg87_connection * connection,
                                              char *name);

/******************************************************************************/
/* dg87_util_find_sta_task_node
 * return the task node which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find_.
 */
BOOL dg87_util_find_sta_task_node (dg87_connection * connection,
                                   dg87_sta_ref * ref, char *name,
                                   dg87_statnode * task_node);

/******************************************************************************/
/* dg87_util_find_sta_job_node
 * return the job node which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find_.
 */
BOOL dg87_util_find_sta_job_node (dg87_connection * connection,
                                  dg87_sta_ref * ref,
                                  dg87_statnode * task_node, char *name,
                                  dg87_statnode * job_node);


/******************************************************************************/
/* dg87_util_find_log_day
 * return the Day which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find_.
 */
dg87_log_day *dg87_util_find_log_day (dg87_connection * connection,
                                      char *name);

/******************************************************************************/
/* dg87_util_find_log_task_node
 * return the task node which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find_.
 */
BOOL dg87_util_find_log_task_node (dg87_connection * connection,
                                   dg87_log_day * day, char *name,
                                   dg87_log_node * task_node);

/******************************************************************************/
/* dg87_util_find_log_job_node
 * return the job node which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find_.
 */
BOOL dg87_util_find_log_job_node (dg87_connection * connection,
                                  dg87_log_day * day,
                                  dg87_log_node * task_node, char *name,
                                  dg87_log_node * job_node);

/******************************************************************************/
/* dg87_r_util_find_job
 * return the job which correspond to the name given in parameter.
 * The Task must be pass as parameter.
 * Print an error message and return NULL if it can't be find_.
 */
dg87_r_job *dg87_r_util_find_job (dg87_connection * connection,
                                  char *task_name, char *job_name,
                                  char *day_name);

/******************************************************************************/
/* dg87_r_util_find_file
 * return the file which correspond to the name given in parameter.
 * The Path of the file must be pass as parameter.
 * Print an error message and return NULL if it can't be find_.
 */
dg87_r_file *dg87_r_util_find_file (dg87_connection * connection,
                                    char *day_name, char *task_name,
                                    char *path);




#endif /*__DG87_UTIL_H__*/
/* End of file */
