/* file x_condition.c */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <libdg87/global.h>
#include <libdg87/strbuf.h>
#include <assert.h>

#include "x_condition.h"
#include "x_task.h"
#include "x_day.h"
#include "variable.h"
#include "x_caller.h"

/****************************************************************************/
dg87_exp_condition *
dg87_exp_condition_hmin_new (int minutes, dg87ubyte day_delta,
                             dg87_exp_task * task)
{
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (minutes < 0 || minutes >= 24 * 60)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_exp_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_hmin = dg87_malloc (sizeof (struct
                                                             dg87_exp_condition_h),
                                                     DG87_OTHER)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->data.condition_hmin->minutes = minutes;
  condition->data.condition_hmin->day_delta = day_delta;
  condition->task = task;
  condition->state = dg87_exp_condition_waiting;
  condition->type = condition_hmin;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_exp_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_condition_hmax_new (int minutes, dg87ubyte day_delta,
                             dg87_exp_task * task)
{
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (minutes < 0 || minutes >= 24 * 60)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_exp_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_hmax = dg87_malloc (sizeof (struct
                                                             dg87_exp_condition_h),
                                                     DG87_OTHER)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->task = task;
  condition->state = dg87_exp_condition_waiting;
  condition->type = condition_hmax;
  condition->data.condition_hmax->minutes = minutes;
  condition->data.condition_hmin->day_delta = day_delta;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_exp_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_condition_hlate_new (int minutes, dg87ubyte day_delta,
                              dg87_exp_task * task)
{
  dg87_exp_condition *condition;
  assert (task != NULL);
  if (task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (minutes < 0 || minutes >= 24 * 60)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_exp_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_hlate = dg87_malloc (sizeof (struct
                                                              dg87_exp_condition_h),
                                                      DG87_OTHER)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->task = task;
  condition->state = dg87_exp_condition_waiting;
  condition->type = condition_hlate;
  condition->data.condition_hlate->minutes = minutes;
  condition->data.condition_hmin->day_delta = day_delta;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_exp_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_condition_file_new (char *file_path, dg87_exp_task * task)
{
  dg87_exp_condition *condition;
  assert (file_path != NULL && task != NULL);
  if (file_path == NULL || file_path[0] == 0 || task == NULL)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_exp_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_File = dg87_malloc (sizeof (struct
                                                             dg87_exp_condition_file),
                                                     DG87_OTHER)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_File->path =
       dg87_strdup (file_path, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->task = task;
  condition->state = dg87_exp_condition_waiting;
  condition->type = condition_file;
  condition->data.condition_File->command_sended = FALSE;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_exp_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_condition_manual_new (char *message, dg87_exp_task * task)
{
  dg87_exp_condition *condition;
  assert (message != NULL && task != NULL);
  if (message == NULL || message[0] == 0 || task == NULL)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_exp_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_manual = dg87_malloc (sizeof (struct
                                                               dg87_exp_condition_manual),
                                                       DG87_OTHER)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_manual->message =
       dg87_strdup (message, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->task = task;
  condition->state = dg87_exp_condition_waiting;
  condition->type = condition_manual;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_exp_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_condition_ifequal_new (dg87_variable * variable, char *value,
                                dg87_exp_task * task)
{
  dg87_exp_condition *condition;
  assert (variable != NULL && value != NULL && task != NULL);
  if (variable == NULL || value == NULL || task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_exp_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_ifequal = dg87_malloc (sizeof (struct
                                                                dg87_exp_condition_eq),
                                                        DG87_OTHER)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_ifequal->value =
       dg87_strdup (value, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->task = task;
  condition->state = dg87_exp_condition_waiting;
  condition->type = condition_ifequal;
  /* increment variable usage count */
  variable->linked_exp_conditions++;
  condition->data.condition_ifequal->variable = variable;

	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_exp_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
dg87_exp_condition *
dg87_exp_condition_ifnotequal_new (dg87_variable * variable, char *value,
                                   dg87_exp_task * task)
{
  dg87_exp_condition *condition;
  assert (variable != NULL && value != NULL && task != NULL);
  if (variable == NULL || value == NULL || task == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  /* allocate memory */
  if ((condition = dg87_malloc (sizeof (dg87_exp_condition),
                                DG87_CONDITION)) == NULL)
    goto alloc_error;
  if ((condition->data.condition_ifnotequal = dg87_malloc (sizeof (struct
                                                                   dg87_exp_condition_eq),
                                                           DG87_OTHER)) ==
      NULL)
    goto alloc_error;
  if ((condition->data.condition_ifnotequal->value =
       dg87_strdup (value, DG87_CHAR)) == NULL)
    goto alloc_error;
  /* initialize structure */
  condition->task = task;
  condition->state = dg87_exp_condition_waiting;
  condition->type = condition_ifnequal;
  /* increment variable usage count */
  variable->linked_exp_conditions++;
  condition->data.condition_ifnotequal->variable = variable;
	DG87_SAVE_NEEDED;
  return condition;

alloc_error:
  if (condition != NULL)
    dg87_exp_condition_delete (condition);
  dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
  return NULL;
}

/****************************************************************************/
BOOL
dg87_exp_condition_delete (dg87_exp_condition * condition)
{
  assert (condition != NULL);
  if (condition == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* delete memory for structure */
  switch (condition->type)
    {
    case condition_hmin:
      if (condition->data.condition_hmin != NULL)
        dg87_free (condition->data.condition_hmin, DG87_OTHER);
      break;
    case condition_hmax:
      if (condition->data.condition_hmax != NULL)
        dg87_free (condition->data.condition_hmax, DG87_OTHER);
      break;
    case condition_hlate:
      if (condition->data.condition_hlate != NULL)
        dg87_free (condition->data.condition_hlate, DG87_OTHER);
      break;
    case condition_file:
      if (condition->data.condition_File != NULL)
        {
          if (condition->data.condition_File->path != NULL)
            dg87_free (condition->data.condition_File->path, DG87_CHAR);
          dg87_free (condition->data.condition_File, DG87_OTHER);
        }
      break;
    case condition_manual:
      if (condition->data.condition_manual != NULL)
        {
          if (condition->data.condition_manual->message != NULL)
            dg87_free (condition->data.condition_manual->message, DG87_CHAR);
          dg87_free (condition->data.condition_manual, DG87_OTHER);
        }
      break;
    case condition_ifequal:
      if (condition->data.condition_ifequal != NULL)
        {
          if (condition->data.condition_ifequal->variable != NULL)
            {
              /* decrement usage count */
              condition->data.condition_ifequal->variable->
                linked_exp_conditions--;
              dg87_free (condition->data.condition_ifequal->value, DG87_CHAR);
            }
          dg87_free (condition->data.condition_ifequal, DG87_OTHER);
        }
      break;
    case condition_ifnequal:
      if (condition->data.condition_ifnotequal != NULL)
        {
          if (condition->data.condition_ifnotequal->variable != NULL)
            {
              /* decrement usage count */
              condition->data.condition_ifnotequal->variable->
                linked_exp_conditions--;
              dg87_free (condition->data.condition_ifnotequal->value,
                         DG87_CHAR);
            }
          dg87_free (condition->data.condition_ifnotequal, DG87_OTHER);

        }
      break;

    case x_condition_error:
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      break;
    }
  dg87_free (condition, DG87_CONDITION);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_condition_load (dg87_exp_condition * condition,
                         dg87_storage * storage)
{
  dg87ubyte b;
  assert (condition != NULL && storage != NULL);
  if (condition == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* read 'state' property */
  if (dg87_storage_read_ubyte (storage, &b) == FALSE)
    return FALSE;
  condition->state = b;

  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_condition_save (dg87_exp_condition * condition,
                         dg87_storage * storage)
{
  assert (condition != NULL && storage != NULL);
  if (condition == NULL || storage == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* 'state' property */
  if (dg87_storage_write_ubyte (storage, condition->state) == FALSE)
    return FALSE;
  return TRUE;
}

/****************************************************************************/
dg87_exp_condition_type
dg87_exp_condition_ascii_to_type (char *condition)
{
  if (condition == NULL)
    return condition_error;

  if (strcasecmp (condition, "hmin") == 0)
    return condition_hmin;

  if (strcasecmp (condition, "hmax") == 0)
    return condition_hmax;

  if (strcasecmp (condition, "hlate") == 0)
    return condition_hlate;

  if (strcasecmp (condition, "file") == 0)
    return condition_file;

  if (strncmp (condition, "manual", 6) == 0)
    return condition_manual;

  if (strncmp (condition, "if_equal", 8) == 0)
    return condition_ifequal;
  if (strncmp (condition, "if_not_equal", 13) == 0)
    return condition_ifnequal;

  return condition_error;
}

/****************************************************************************/
char *
dg87_exp_condition_type_to_ascii (dg87_exp_condition_type condition_type)
{
  switch (condition_type)
    {
    case condition_hmin:
      return "hmin";
    case condition_hmax:
      return "hmax";
    case condition_hlate:
      return "hlate";
    case condition_file:
      return "file";
    case condition_manual:
      return "manual";
    case condition_ifequal:
      return "if_equal";
    case condition_ifnequal:
      return "if_not_equal";
    case condition_error:
      return "error";
    }
  return "error";
}

/****************************************************************************/
char *
dg87_exp_condition_to_ascii (dg87_exp_condition * condition)
{
  dg87_strbuf *strbuf;
  char *string;

  if ((strbuf = dg87_strbuf_new ()) == NULL)
    return NULL;
  dg87_strbuf_printf (strbuf, "%s ",
                      dg87_exp_condition_type_to_ascii (condition->type));

  switch (condition->type)
    {
    case condition_hmin:
      string =
        dg87_exp_condition_minute_to_hour (condition->data.condition_hmin->
                                           minutes);
      dg87_strbuf_printf (strbuf, "%s", string);
      dg87_free (string, DG87_CHAR);
      break;
    case condition_hmax:
      string =
        dg87_exp_condition_minute_to_hour (condition->data.condition_hmax->
                                           minutes);
      dg87_strbuf_printf (strbuf, "%s", string);
      dg87_free (string, DG87_CHAR);
      break;
    case condition_hlate:
      string =
        dg87_exp_condition_minute_to_hour (condition->data.condition_hlate->
                                           minutes);
      dg87_strbuf_printf (strbuf, "%s", string);
      dg87_free (string, DG87_CHAR);
      break;
    case condition_file:
      dg87_strbuf_printf (strbuf, "%s", condition->data.condition_File->path);
      break;
    case condition_manual:
      dg87_strbuf_printf (strbuf, "%s",
                          condition->data.condition_manual->message);
      break;
    case condition_ifequal:
      dg87_strbuf_printf (strbuf, "%s %s",
                          condition->data.condition_ifequal->variable->name,
                          condition->data.condition_ifequal->value);
      break;
    case condition_ifnequal:
      dg87_strbuf_printf (strbuf, "%s %s",
                          condition->data.condition_ifnotequal->variable->
                          name, condition->data.condition_ifnotequal->value);
      break;

    default:
      dg87_strbuf_printf (strbuf, "Unknown type of condition !\n");
      break;
    }

  string = dg87_strbuf_get_buffer (strbuf);
  dg87_strbuf_delete (strbuf);
  return string;
}

/****************************************************************************/
int
dg87_exp_condition_hour_to_minute (char *hour)
{
  char *str_hour, *str_min;
  int result = -1;
  if (hour == NULL)
    return -1;
  str_hour = strdup (hour);
  if (str_hour == NULL)
    return -1;
  str_min = strchr (str_hour, ':');
  if (str_min == NULL)
    result = atoi (str_hour);
  else
    {
      *str_min = '\0';
      result = atoi (str_hour) * 60 + atoi (str_min + 1);
    }
  free (str_hour);
  return result;
}

/****************************************************************************/
char *
dg87_exp_condition_minute_to_hour (int minutes)
{
  int h, m;
  char *str_res;
  if (minutes < 0 || minutes >= 24 * 60)
    return NULL;
  str_res = dg87_malloc (sizeof (char) * 5, DG87_CHAR);
  h = minutes / 60;
  m = minutes % 60;
  sprintf (str_res, "%.2i:%.2i", h, m);
  return str_res;
}

/****************************************************************************/
char *
dg87_exp_condition_state_to_ascii (dg87_exp_condition_state depstate)
{
  switch (depstate)
    {
    case dg87_exp_condition_waiting:
      return "waiting";
    case dg87_exp_condition_finished:
      return "finished";
    default:
      return "unknown";         /* to remove warning */
    }
  return NULL;
}

/****************************************************************************/
BOOL
_dg87_exp_condition_check_time (int minutes, dg87ubyte day_delta,
								time_t day_start_time)
{
  time_t dt_condition;
  time_t dt_now;
  struct tm dc;

  /* get current time */
  time (&dt_now);

  /* get midnight time */
  dc = *localtime (&day_start_time);
  dc.tm_hour = 0;
  dc.tm_min = 0;
  dc.tm_sec = 0;
  dt_condition = mktime (&dc);

  /* calculate the time of the condition */
  dt_condition += day_delta * 24 * 3600 + minutes * 60;

  return (dt_condition <= dt_now) ? TRUE : FALSE;
}

/****************************************************************************/
dg87_exp_condition_state
dg87_exp_condition_get_state (dg87_exp_condition * condition)
{
  struct stat sf;

  assert (condition != NULL);
  if (condition == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  /* nothing to do if condition is finished or canceled */
  if (condition->state != dg87_exp_condition_waiting)
    return condition->state;

  switch (condition->type)
    {
    case condition_hmin:
      /* if h > hlate then state=finished */
      if (_dg87_exp_condition_check_time
          (condition->data.condition_hmin->minutes,
           condition->data.condition_hmin->day_delta,
					 condition->task->day->start_time) == TRUE)
        dg87_exp_condition_set_state (condition, dg87_exp_condition_finished);
      break;
    case condition_hmax:
      /* if h > hlate then state=finished */
      if (_dg87_exp_condition_check_time
          (condition->data.condition_hmax->minutes,
           condition->data.condition_hmax->day_delta,
					 condition->task->day->start_time) == TRUE)
        dg87_exp_condition_set_state (condition, dg87_exp_condition_finished);
      break;
    case condition_hlate:
      /* if h > hlate then state=finished */
      if (_dg87_exp_condition_check_time
          (condition->data.condition_hmax->minutes,
           condition->data.condition_hmax->day_delta,
					 condition->task->day->start_time) == TRUE)
        dg87_exp_condition_set_state (condition, dg87_exp_condition_finished);
      break;
    case condition_file:
      /* if tasks' queue is not set, test file presence on
       * the local server (where dg87x is running) */
      if (condition->task->queue == NULL)
        {
          /* if file is present then state = finished */
          if (stat (condition->data.condition_File->path, &sf) == 0)
            {
              dg87_exp_condition_set_state (condition,
                                            dg87_exp_condition_finished);
            }
        }
      else
        {
          if (condition->data.condition_File->command_sended == FALSE)
            {
              /* send a `fa' command to dg87r */
              if (dg87_caller__send_file_test (condition) == TRUE)
                condition->data.condition_File->command_sended = TRUE;
            }
        }
      break;
    case condition_manual:
      /* nothing to test : this type of condition is
       * validated by a user with 'xtcv' command */
      break;
    case condition_ifequal:
      if (strcmp (condition->data.condition_ifequal->variable->value,
                  condition->data.condition_ifequal->value) == 0)
        dg87_exp_condition_set_state (condition, dg87_exp_condition_finished);
      break;
    case condition_ifnequal:
      if (strcmp (condition->data.condition_ifnotequal->variable->value,
                  condition->data.condition_ifnotequal->value) != 0)
        dg87_exp_condition_set_state (condition, dg87_exp_condition_finished);
      break;
    default:
      break;
    }
  return condition->state;
}

/****************************************************************************/
BOOL
dg87_exp_condition_set_state (dg87_exp_condition * condition,
                              dg87_exp_condition_state new_state)
{
  assert (condition != NULL);
  if (condition->state == new_state)
    return TRUE;
  condition->state = new_state;
  /* log message */
  dg87_caller__send_condition_state (condition);
	DG87_SAVE_NEEDED;
  return TRUE;
}

/****************************************************************************/
BOOL
dg87_exp_condition_validate (dg87_exp_condition * condition)
{
  assert (condition != NULL);
  if (condition == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return FALSE;
    }
  dg87_exp_condition_set_state (condition, dg87_exp_condition_finished);
  return TRUE;
}

/****************************************************************************/
#ifdef DEBUG
int
dg87_exp_condition_check (dg87_exp_condition * condition,
                          int (*print) (char *))
{
  int minutes;
  if (condition->task == NULL)
    {
      print ("xCondition : task is null !\n");
      return 1;
    }
  switch (condition->type)
    {
    case condition_hmin:
      minutes = condition->data.condition_hmin->minutes;
      if (minutes < 0 || minutes >= 24 * 60)
        {
          print ("xCondition : incorrect value hmin !\n");
          return 1;
        }
      break;
    case condition_hmax:
      minutes = condition->data.condition_hmax->minutes;
      if (minutes < 0 || minutes >= 24 * 60)
        {
          print ("xCondition : incorrect value hmax !\n");
          return 1;
        }
      break;
    case condition_hlate:
      minutes = condition->data.condition_hlate->minutes;
      if (minutes < 0 || minutes >= 24 * 60)
        {
          print ("xCondition : incorrect value hlate !\n");
          return 1;
        }
      break;
    case condition_file:
      if (condition->data.condition_File->path == NULL)
        {
          print ("xCondition : Path file is NULL !\n");
          return 1;
        }
      break;
    case condition_manual:
      if (condition->data.condition_manual->message == NULL)
        {
          print ("xCondition : Message is NULL !\n");
          return 1;
        }
      break;
    default:
      print ("xCondition : incorrect type !\n");

    }
  return 0;
}
#endif /*DEBUG*/
/****************************************************************************/
/* End of file */
