/* file x_day.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_EXP_DAY_H__
#define __DG87_EXP_DAY_H__

#include <time.h>
#include "global.h"
#include <libdg87/vector.h>
#include "x_task.h"
#include "x_taskdep.h"


struct _dg87_exp_day
{
  char name[DG87_DAY_NAME_LENGTH + 1];
  dg87_context *context;
  BOOL enabled;
  BOOL auto_purged;
  dg87_vector *task_vector;
  dg87_list *taskdep_list;
  char *referential_name;
  time_t start_time;
};

/*
 * dg87_exp_day_new
 * create a new day and return a pointer on this.
 */
dg87_exp_day *dg87_exp_day_new (char *name, time_t start_time,
                                dg87_context * context);

/*
 * dg87_exp_day_delete
 */
BOOL dg87_exp_day_delete (dg87_exp_day * day);

/*
 * dg87_exp_day_load
 */
BOOL dg87_exp_day_load (dg87_exp_day * day, dg87_storage * storage);

/*
 * dg87_exp_day_save
 */
BOOL dg87_exp_day_save (dg87_exp_day * day, dg87_storage * storage);

/*
 * dg87_exp_day_set_enabled
 */
BOOL dg87_exp_day_set_enabled (dg87_exp_day * day, BOOL value);

/*
 * dg87_exp_day_set_referential_name
 */
BOOL dg87_exp_day_set_referential_name (dg87_exp_day * day,
                                        char *referential_name);

/*
 * dg87_exp_day_set_auto_purged
 */
BOOL dg87_exp_day_set_auto_purged (dg87_exp_day * day, BOOL auto_purged);

/*************************************************************************/
/*  functions for tasks                                                    */
/*************************************************************************/

/*
 * dg87_exp_day_task_add
 * create an TASK object, add this object in the TaskList
 * of the day and return a pointer on this object.
 * Return NULL if the TASK object can't be create.
 */
dg87_exp_task *dg87_exp_day_task_add (dg87_exp_day * day, char *task_name);

/*
 * dg87_exp_day_task_count
 * return the number of TASK.
 * return -1 if day is NULL.
 */
int dg87_exp_day_task_count (dg87_exp_day * day);

/*
 * dg87_exp_day_task_get_by_name
 * return a pointer on the TASK which has the name
 * given in parameter. Return NULL if it can't be find_.
 */
dg87_exp_task *dg87_exp_day_task_get_by_name (dg87_exp_day * day, char *name);

/*
 * dg87_exp_task_get_by_index
 * return a pointer on the TASK at the given position.
 */
dg87_exp_task *dg87_exp_day_task_get_by_index (dg87_exp_day * day, int index);

/*
 * dg87_exp_day_task_delete
 * remove from vector and delete the task given in parameter.
 */
BOOL dg87_exp_day_task_delete (dg87_exp_day * day, dg87_exp_task * task);

/*
 * dg87_exp_day_task_delete_by_name
 * delete the task which has the name given in parameter.
 */
BOOL dg87_exp_day_task_delete_by_name (dg87_exp_day * day, char *name);

/*
 * dg87_exp_day_task_delete_by_index
 * delete the task at the given index.
 */
BOOL dg87_exp_day_task_delete_by_index (dg87_exp_day * day, int index);

/*************************************************************************/
/*  functions for taskdep                                                  */
/*************************************************************************/

/*
 * dg87_exp_day_taskdep_add
 * add a dependence between two tasks of the day. The name of the tasks must
 * be pass in parameters.
 */
dg87_exp_taskdep *dg87_exp_day_taskdep_add (dg87_exp_day * day,
                                            dg87_exp_task * upstream_task,
                                            dg87_exp_task * downstream_task,
                                            dg87_exp_taskdep_type type);

/*
 * dg87_exp_day_taskdep_count
 * return the number of taskdep the day.
 * return -1 if day is NULL.
 */
int dg87_exp_day_taskdep_count (dg87_exp_day * day);

/*
 * dg87_exp_day_taskdep_get
 * return a pointer on the taskdep at the given index.
 */
dg87_exp_taskdep *dg87_exp_day_taskdep_get (dg87_exp_day * day, int index);

/*
 * dg87_exp_day_taskdep_delete_by_index
 * delete the taskdep at the given index.
 */
BOOL dg87_exp_day_taskdep_delete_by_index (dg87_exp_day * day, int index);

/*
 * dg87_exp_day_taskdep_delete
 * delete the taskdep given in parameter 
 */
BOOL dg87_exp_day_taskdep_delete (dg87_exp_day * day,
                                  dg87_exp_taskdep * taskdep);

#ifdef DEBUG
/*
 * dg87_exp_day_ckeck
 */
int dg87_exp_day_check (dg87_exp_day * day, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_EXP_DAY_H__*/
/* End of file */
