/* file x_jobdep.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_EXP_JOB_DEP_H__
#define __DG87_EXP_JOB_DEP_H__
#include <libdg87/global.h>
#include <libdg87/storage.h>
#include "global.h"

typedef enum
{
  /* when adding a new state,
   * don't forget to update dg87_exp_jobdep_state_to_ascii */
  dg87_exp_jobdep_state_unknown,
  dg87_exp_jobdep_state_waiting,
  dg87_exp_jobdep_state_validated,
  dg87_exp_jobdep_state_canceled
}
dg87_exp_jobdep_state;

/* type of jobs dependencies */
typedef enum
{
  dg87_exp_jobdep_type_unknown,
  dg87_exp_jobdep_type_always,
  dg87_exp_jobdep_type_on_success,
  dg87_exp_jobdep_type_on_error
}
dg87_exp_jobdep_type;

/* structure for jobdep objects */
struct _dg87_exp_jobdep
{
  dg87_exp_job *upstream_job;
  dg87_exp_job *downstream_job;
  dg87_exp_jobdep_type type;
  dg87_exp_jobdep_state state;
};


/*
 * dg87_exp_jobdep_new
 * create a new jobdep and return a pointer on it.
 */
dg87_exp_jobdep *dg87_exp_jobdep_new (dg87_exp_job * upstream_job,
                                      dg87_exp_job * downstream_job,
                                      dg87_exp_jobdep_type type);

/*
 * dg87_exp_jobdep_delete
 * free memory allocate for the object.
 */
BOOL dg87_exp_jobdep_delete (dg87_exp_jobdep * job_dep);

/*
 * dg87_exp_jobdep_load
 */
BOOL dg87_exp_jobdep_load (dg87_exp_jobdep * job_dep, dg87_storage * storage);

/*
 * dg87_exp_jobdep_save
 */
BOOL dg87_exp_jobdep_save (dg87_exp_jobdep * job_dep, dg87_storage * storage);

/*
 * dg87_exp_jobdep_set_type
 * set the type of dependence for the jobdep.
 */
BOOL dg87_exp_jobdep_set_type (dg87_exp_jobdep * job_dep,
                               dg87_exp_jobdep_type type);

/*
 * dg87_exp_jobdep_set_state
 * set the state of the jobdep.
 */
BOOL dg87_exp_jobdep_set_state (dg87_exp_jobdep * job_dep,
                                dg87_exp_jobdep_state state);


/*
 * dg87_exp_jobdep_evaluate
 * evaluate if the dependence can be apply or not
 */
BOOL dg87_exp_jobdep_evaluate (dg87_exp_jobdep * job_dep);

/*
 * dg87_exp_jobdep_ascii_to_type
 * convert a string into a dependence type.
 * return -1 if error.
 */
dg87_exp_jobdep_type dg87_exp_jobdep_ascii_to_type (char *str_type);

/*
 * dg87_exp_jobdep_type_to_ascii
 * convert a dependence type into a string
 */
char *dg87_exp_jobdep_type_to_ascii (dg87_exp_jobdep_type dep_type);

/*
 * dg87_exp_jobdep_ascii_to_state
 * convert a string into a dependence state.
 * return -1 if error.
 */
dg87_exp_jobdep_state dg87_exp_jobdep_ascii_to_state (char *str_state);

/*
 * dg87_exp_jobdep_state_to_ascii
 * convert a dependence state into a string
 */
char *dg87_exp_jobdep_state_to_ascii (dg87_exp_jobdep_state state);

#ifdef DEBUG
/*
 * dg87_exp_jobdep_ckeck
 */
int dg87_exp_jobdep_check (dg87_exp_jobdep * jobdep, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_EXP_JOB_DEP_H__*/
/* End of file */
