/* file cmdmessage.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "daemon.h"
#include "command.h"
#include "cmdmessage.h"

/******************************************************************************/
int
cmd_mc (dg87_connection * connection)
{
  dg87_list *message_list;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  message_list = connection->server->message_list;
  dg87_connection_printf (connection, "%i\n", message_list->count);
  return 0;
}

/******************************************************************************/
int
cmd_mk (dg87_connection * connection)
{
  dg87_message *message;
  dg87_command *command;
  int index;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;
  if (command->argc != 1)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n", "mk index");
      return 1;
    }
  /* find the connection */
  index = atoi (command->argv[0]);
  message =
    dg87_list_get_object_by_index (connection->server->message_list,
				   index - 1);
  if (message == NULL)
    {
      dg87_connection_printf (connection,
			      "Incorrect value for index '%i'\n", index);
      return 1;
    }
  dg87_message_cancel (message);
  return 0;
}

/******************************************************************************/
/*
 * cmd_ml
 * Usage : ml [options] 
 * options :
 * -s : print message state
 * -m : print message
 */
int
cmd_ml (dg87_connection * connection)
{
  BOOL show_state = FALSE;
  BOOL show_message = FALSE;
  char *sep, *beg, *end;	/* separator, begin, end */
  dg87_item *item;
  dg87_command *command;
  dg87_message *message_item;
  int optch, iopt;
  char *optarg;
  assert (connection != NULL);
  if (connection == NULL)
    return 1;
  command = connection->command;

  /* control number of arguments */
  if (command->argc != 0)
    {
      dg87_connection_printf (connection,
			      "Incorrect call.\nUsage : %s\n",
			      "ml [options]");
      return 1;
    }

  /* default separator */
  sep = strdup ("\t");
  beg = strdup ("");
  end = strdup ("");

  /* interpret options */
  for (iopt = 0; iopt < command->optc; iopt++)
    {
      optch = command->optv[iopt];
      optarg = command->optargv[iopt];
      switch (optch)
	{
	case 'S':		/* separator */
	  free (sep);
	  sep = strdup (optarg);
	  break;
	case 'B':		/* begin separator */
	  free (beg);
	  beg = strdup (optarg);
	  break;
	case 'E':		/* end separator */
	  free (end);
	  end = strdup (optarg);
	  break;
	case 's':		/* state */
	  show_state = TRUE;
	  break;
	case 'm':		/* show message */
	  show_message = TRUE;
	  break;
	default:		/* unknown option */
	  dg87_connection_printf (connection, "Unknown option");
	  free (sep);
	  free (beg);
	  free (end);
	  return 1;
	}
    }

  item = connection->server->message_list->first;
  while (item != NULL)
    {
      message_item = item->object;
      item = item->next;
      dg87_connection_printf (connection, "%s%s%s%i", beg,
			      message_item->hostname, sep,
			      message_item->ip_port);
      if (show_message == TRUE)
	dg87_connection_printf (connection, "%s%s", sep,
				message_item->message_string);
      if (show_state == TRUE)
	dg87_connection_printf (connection, "%s%s", sep,
				dg87_message_state_to_ascii (message_item->
							     state));
      dg87_connection_printf (connection, "%s\n", end);
    }
  return 0;
}

/******************************************************************************/
/* End of file */
