/* file daemon.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_DAEMON_H__
#define __DG87_DAEMON_H__

#include "global.h"

typedef enum 
dg87_daemon_log_priority_struct
{
	dg87_daemon_log_alert    = 0,
	dg87_daemon_log_critical = 1,
	dg87_daemon_log_error    = 2,
	dg87_daemon_log_warning  = 3,
	dg87_daemon_log_notice   = 4,
	dg87_daemon_log_info     = 5,
	dg87_daemon_log_debug    = 6
} 
dg87_daemon_log_priority;


/*
 * dg87_daemon object structure
 */
typedef struct
{
  char *name;      /* name of the daemon for the log */
  char *user_name; /* name of the user account to use (can be NULL) */
}
dg87_daemon;

/*
 * dg87_daemon_start
 * create a new daemon objet an return a reference on this object.
 */
dg87_daemon *dg87_daemon_new (char *name, char *user_name);

/*
 * dg87_daemon_start
 * make a fork of the current process
 * return 0 to the parent process.
 * return 1 to the child process.
 * return -1 if an error occures
 */
int dg87_daemon_start (dg87_daemon * daemon);

/*
 * dg87_daemon_stop
 */
void dg87_daemon_stop (dg87_daemon * daemon);

/*
 * dg87_daemon_delete
 * close and delete the lock file.
 * free memory of the object.
 */
void dg87_daemon_delete (dg87_daemon * daemon);

/*
 * dg87_daemon_log_printf
 * This function generates a log message.
 * The first parameter is the priority of the message : info, warning, error...
 * The second parameter is a format string (like printf).
 * The other optionnal parameters are used to replace the % in format string
 * by their value.
 */
void dg87_daemon_log_printf(dg87_daemon_log_priority priority, 
								char* message, ...);

/*
 * dg87_daemon_set_log_priority 
 * Defines the log level priority.
 * Possible values are WARNING, NOTICE, INFO, DEBUG.
 * The function returns FALSE if the log_level value is incorrect.
 */
BOOL dg87_daemon_set_log_priority (char* log_level);

/*
 * dg87_daemon_get_log_priority 
 * return a string which is the log priority
 */
char* dg87_daemon_get_log_priority (void);

#endif /*__DG87_DAEMON_H__*/
/* End of file */
