/* file dgerr.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "dgerr.h"
#include "global.h"
#include <libintl.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "strbuf.h"

static int dg87_err = 0;
static char* dg87_err_str = NULL;

#define gettext_noop(String) (String)

char *dg87_errstr[] = {
  gettext_noop ("No error"),
  gettext_noop ("Out of memory"),
  gettext_noop ("Key not unique"),
  gettext_noop ("Parameter is NULL"),
  gettext_noop ("Incorrect value for parameter"),
  gettext_noop ("Incorrect value for key"),
  gettext_noop ("Jobs must be in the same task to have a dependence"),
  gettext_noop ("Can't delete target because tasks use this target."),
  gettext_noop ("Can't delete target because it has queue."),
  gettext_noop ("Can't delete task because it has dependences with other tasks"),
  gettext_noop ("Index out of bound"),
  gettext_noop ("Can't find an object associate to this key"),
  gettext_noop ("Can't find an object with this key"),
  gettext_noop ("The description is to long"),
  gettext_noop ("The name is to long"),
  gettext_noop ("Can't delete task because it has dependences with other tasks"),
  gettext_noop ("Jobs must be in the same task to have a dependence"),
  gettext_noop ("Can't find job before"),
  gettext_noop ("Can't find job after"),
  gettext_noop ("Incorrect Condition Type"),
  gettext_noop ("Can't find task before"),
  gettext_noop ("Can't find task after"),
  gettext_noop ("Plan is in progress"),
  gettext_noop ("No Plan is in progress"),
  gettext_noop ("Circular reference with rules"),
  gettext_noop ("Rule used by other rules"),
  gettext_noop ("Rule used by tasks"),
  gettext_noop ("Operator must be 'add' with 'always' rule"),
  gettext_noop ("Operator must be 'add' or 'remove' with complex rule."),
  gettext_noop ("Circular dependence."),
  gettext_noop ("The day is unlocked"),
  gettext_noop ("Connection is not waiting request."),
  gettext_noop ("Connection is not waiting response."),
  gettext_noop ("Connection error"),
  gettext_noop ("Can't get hostname"),
  gettext_noop ("Socket connection error"),
  gettext_noop ("Connection message not available"),
  gettext_noop ("Connection is not waiting message"),
  gettext_noop ("Unknown command"),
  gettext_noop ("Interpret error"),
  gettext_noop ("User connected"),
  gettext_noop ("Can't fork"),
  gettext_noop ("A similair job is already in the list"),
  gettext_noop ("Syntax error in argument string"),
  gettext_noop ("Password to long."),
  gettext_noop ("Permission denied"),
  gettext_noop ("Can't delete a system account."),
  gettext_noop ("Can't change privilege of a system account."),
  gettext_noop ("Variable used by rules"),
  gettext_noop ("Variable used by conditions."),
  gettext_noop ("Variable used by tasks."),
  gettext_noop ("Can't open configuration file."),
  gettext_noop ("Syntax error in configuration file."),
  gettext_noop ("Unknown field name in configuration file."),
  gettext_noop ("Wrong mode"),
  gettext_noop ("Can't open storage file"),
  gettext_noop ("Can't close storage file"),
  gettext_noop ("Wrong storage file signature"),
  gettext_noop ("Wrong storage file format version"),
  gettext_noop ("Log file already exits !"),
  gettext_noop ("Log file doesn't exist !"),
  gettext_noop ("Can't open log file !"),
  gettext_noop ("Can't close log file !"),
  gettext_noop ("Wrong log file signature !"),
  gettext_noop ("Wrong log file version !"),
  gettext_noop ("Can't write in log file !"),
  gettext_noop ("Can't read in log file !"),
  gettext_noop ("Can't delete log file !"),
  gettext_noop ("Can't access directoy !"),
  gettext_noop ("A similar file is already in the list !"),
  gettext_noop ("Invalid operation on a system target."),
  gettext_noop ("Invalid operation on a system queue."),
  gettext_noop ("Incorrect day name '%s' !")
};

/***************************************************************************/
void
dg87_err_set_error (int error_number)
{
  dg87_err = error_number;
	if (dg87_err_str != NULL)
		dg87_free(dg87_err_str, DG87_CHAR);
	dg87_err_str = dg87_strdup(gettext(dg87_errstr[dg87_err]), DG87_CHAR);
}
/***************************************************************************/
void
dg87_err_set_error_with_arg (int error_number, ...)
{
	char buffer[60];
  char c[2];
  va_list ap;
  int ival;
  long lval;
  char *string;
	char *format;
	dg87_strbuf* strbuf;

	strbuf = dg87_strbuf_new();
	if (strbuf == NULL)
		return;

	format = gettext(dg87_errstr[error_number]); 

  va_start (ap, error_number); /* ap points on the first optional argument */
  while (*format)
    {
      if (*format == '\n')
			{
	  		/* print CR LF */
	  		dg87_strbuf_cat (strbuf, "\n\r");
			}
      else if (*format != '%')
			{
	  		/* not a format character (%), print it */
	  		c[0] = *format;
	  		c[1] = 0;
	  		dg87_strbuf_cat (strbuf, c);
			}
      /* interpret character after % */
      else
				switch (*++format)
	  		{
	  			case 's':		/* replace '%s' by a string */
	    			string = va_arg (ap, char *);
	    			if (string != NULL)
	      			dg87_strbuf_cat (strbuf, string);
	    			break;
	  			case 'c':		/* replace '%c' by a character */
	    			ival = va_arg (ap, int);
	    			sprintf (buffer, "%c", ival);
	    			dg87_strbuf_cat (strbuf, buffer);
	    			break;
	  			case 'i':		/* replace '%i' by a integer */
	    			ival = va_arg (ap, int);
	    			sprintf (buffer, "%i", ival);
	    			dg87_strbuf_cat (strbuf, buffer);
	    			break;
	  			case 'l':		/* replace '%l' by a long */
	    			lval = va_arg (ap, long);
	    			sprintf (buffer, "%li", lval);
	    			dg87_strbuf_cat (strbuf, buffer);
	    			break;
	  			default:
	    			c[0] = *format;
	    			c[1] = 0;
	    			dg87_strbuf_cat (strbuf, c);
	  			}			/* end case */
      format++;			/* next character */
    }				/* end while */
  va_end (ap);


	if (dg87_err_str != NULL)
		dg87_free(dg87_err_str, DG87_CHAR);
	dg87_err_str = dg87_strbuf_get_buffer(strbuf);
	dg87_strbuf_delete(strbuf);
}


/***************************************************************************/
void
dg87_err_clear_error (void)
{
	dg87_err_set_error(0);
}

/***************************************************************************/
char *
dg87_err_get_error_str ()
{
  return dg87_err_str;
}

/***************************************************************************/
/* End of file */
