/* file dgerr.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#ifndef __DG87_ERROR_H__
#define __DG87_ERROR_H__

#define DG87_ERR_BASE 1000;

#define DG87_ERR_OUT_OF_MEMORY 1
#define DG87_ERR_KEY_NOT_UNIQUE 2
#define DG87_ERR_NULL_PARAMETER 3
#define DG87_ERR_INCORRECT_VALUE 4
#define DG87_ERR_INCORRECT_KEY 5
#define DG87_ERR_JOBS_NOT_IN_SAME_TASK 6
#define DG87_ERR_TARGET_USE_BY_TASK 7
#define DG87_ERR_TARGET_HAS_QUEUE 8
#define DG87_ERR_TASK_HAS_DEPENDENCES 9
#define DG87_ERR_INDEX_OUT_OF_BOUND 10
#define DG87_ERR_CANNOT_FIND_OBJECT 11
#define DG87_ERR_CAN_NOT_FIND_OBJECT 12
#define DG87_ERR_DESCRIPTION_TOO_LONG 13
#define DG87_ERR_NAME_TOO_LONG 14
#define DG87_ERR_XTASK_HAS_DEPENDENCES 15
#define DG87_ERR_XJOBS_NOT_IN_SAME_XTASK 16
#define DG87_ERR_CAN_NOT_FIND_JOB_BEFORE 17
#define DG87_ERR_CAN_NOT_FIND_JOB_AFTER 18
#define DG87_ERR_INCORRECT_CONDITION_TYPE 19
#define DG87_ERR_CANNOT_FIND_TASK_BEFORE 20
#define DG87_ERR_CANNOT_FIND_TASK_AFTER 21
#define DG87_ERR_PLAN_IN_PROGRESS 22
#define DG87_ERR_NO_PLAN_IN_PROGRESS 23
#define DG87_ERR_CIRCULAR_RULE_REFERENCE 24
#define DG87_ERR_RULE_USED_BY_ANOTER_RULE 25
#define DG87_ERR_RULE_USED_BY_TASK 26
#define DG87_ERR_RULE_WRONG_OPERATOR_ALWAYS 27
#define DG87_ERR_RULE_WRONG_OPERATOR_COMPLEX 28
#define DG87_ERR_CIRCULAR_DEPENDENCE 29
#define DG87_ERR_DAY_IS_UNLOCKED 30
#define DG87_ERR_CNX_IS_NOT_WAITING_REQUEST 31
#define DG87_ERR_CNX_IS_NOT_WAITING_RESPONSE 32
#define DG87_ERR_CNX_CAN_NOT_CONNECT 33
#define DG87_ERR_CNX_CAN_NOT_GET_HOST_BY_NAME 34
#define DG87_ERR_CNX_SOCKET 35
#define DG87_ERR_CNX_MESSAGE_NOT_AVAILABLE 36
#define DG87_ERR_CNX_NOT_WAITING_MESSAGE 37
#define DG87_ERR_UNKNOWN_COMMAND 38
#define DG87_ERR_INTERPRET_ERROR 39
#define DG87_ERR_USER_CONNECTED 40
#define DG87_ERR_CAN_NOT_FORK 41
#define DG87_ERR_SAME_JOB_ALREADY_EXISTS 42
#define DG87_ERR_ARGUMENT_SYNTAX_ERROR 43
#define DG87_ERR_PASSWORD_TOO_LONG	44
#define DG87_ERR_PERM_DENIED 45
#define DG87_ERR_CAN_NOT_DELETE_SYSTEM_ACCOUNT 46
#define DG87_ERR_CAN_NOT_CHANGE_SYSTEM_ACCOUNT 47
#define DG87_ERR_VARIABLE_USED_BY_RULES	48
#define DG87_ERR_VARIABLE_USED_BY_CONDITIONS 49
#define DG87_ERR_VARIABLE_USED_BY_TASKS 50
#define DG87_ERR_CAN_NOT_OPEN_CONFIG_FILE 51
#define DG87_ERR_CONF_SYNTAX_ERROR 52
#define DG87_ERR_CONF_UNKNOWN_FIELD_NAME 53
#define DG87_ERR_STORAGE_WRONG_MODE 54
#define DG87_ERR_STORAGE_CAN_NOT_OPEN 55
#define DG87_ERR_STORAGE_CAN_NOT_CLOSE 56
#define DG87_ERR_STORAGE_WRONG_SIGN 57
#define DG87_ERR_STORAGE_WRONG_VERSION 58
#define DG87_ERR_LOGFILE_ALREADY_EXISTS 59
#define DG87_ERR_LOGFILE_DONT_EXIT 60
#define DG87_ERR_LOGFILE_CAN_NOT_OPEN 61
#define DG87_ERR_LOGFILE_CAN_NOT_CLOSE 62
#define DG87_ERR_LOGFILE_WRONG_SIGN 63
#define DG87_ERR_LOGFILE_WRONT_VERSION 64
#define DG87_ERR_LOG_CAN_NOT_WRITE 65
#define DG87_ERR_LOG_CAN_NOT_READ 66
#define DG87_ERR_LOG_CAN_NOT_DELETE_FILE 67
#define DG87_ERR_CAN_NOT_ACCESS_DIRECTORY 68
#define DG87_ERR_SAME_FILE_ALREADY_EXISTS 69
#define DG87_ERR_CAN_NOT_CHANGE_SYSTEM_TARGET 70
#define DG87_ERR_CAN_NOT_CHANGE_SYSTEM_QUEUE 71
#define DG87_ERR_OPER_INCORRECT_DAY_NAME 72



void dg87_err_set_error (int error_number);

/*
 * dg87_err_set_error_with_arg
 * the first argument is the error number. The other arguments are
 * string or integer which are used by the error message associated
 * with the error number.
 */
void dg87_err_set_error_with_arg (int error_number, ...);

void dg87_err_clear_error (void);
char *dg87_err_get_error_str ();

#endif /*__DG87__ERROR_H__*/
