/* file element.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "element.h"

/****************************************************************************/
dg87_element *
dg87_element_new (void *object, char *key)
{
  dg87_element *element;
  assert (object != NULL && key != NULL);
  if (object == NULL || key == NULL)
    {
      dg87_err_set_error (DG87_ERR_NULL_PARAMETER);
      return NULL;
    }
  if (strlen (key) == 0)
    {
      dg87_err_set_error (DG87_ERR_INCORRECT_KEY);
      return NULL;
    }
  if ((element = dg87_malloc (sizeof (dg87_element), DG87_ELEMENT)) == NULL)
    {
      dg87_err_set_error (DG87_ERR_OUT_OF_MEMORY);
      return NULL;
    }
  element->key = key;
  element->object = object;
  element->next = NULL;
  element->prev = NULL;
  return element;
}

/****************************************************************************/
BOOL
dg87_element_delete (void *element)
{
  assert (element != NULL);
  if (element == NULL)
    return FALSE;

  dg87_free (element, DG87_ELEMENT);

  return TRUE;
}

/****************************************************************************/


/* End of file */
