/* file global.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_GLOBAL_H__
#define __DG87_GLOBAL_H__

#define TRUE 1
#define FALSE 0
#define BOOLERROR 2
#define BOOL unsigned char

#define dg87uint unsigned short int
#define dg87int short int
#define dg87long int
#define dg87ubyte unsigned char
#define dg87byte char

#include <stdlib.h>
#include <time.h>
#include "dgerr.h"

/*
 * dg87_save_needed : 
 * 0 -> save is not needed
 * 1 -> save is needed after the save tempo
 * 2 -> save is needed immediatly
 */
extern dg87ubyte dg87_save_needed;


/*
 * dg87_current_time
 * this variable is used to store the current time : it reduces the number
 * of calls to time(NULL). this variable is updated one time by cycle of
 * the engine
 */
extern time_t dg87_current_time;


#define DG87_SAVE_DONE dg87_save_needed=0;
#define DG87_SAVE_NEEDED dg87_save_needed=1;
#define DG87_SAVE_NEEDED_NOW dg87_save_needed=2;

/* command families */
/* don't forget to update dg87_command_ascii_to_family and 
 * dg87_command_family_to_ascii when adding a new commands family.
 */
#define DG87_COMMAND_FAM_MAX 11
typedef enum
{
  dg87_command_generic = 0,	/* user, password, help, memory ... */
  dg87_command_communication = 1,	/* connection, message */
  dg87_command_user = 2,
  dg87_command_engine = 3,	/* shutdown, ese */
  dg87_command_target = 4,	/* target, queue */
  dg87_command_ref = 5,		/* dg87d referential, task, job... */
  dg87_command_job = 6,		/* dg87r jobs */
  dg87_command_day = 7,		/* dg87x day, task, job */
  dg87_command_variable = 8,	/* variables */
  dg87_command_log = 9,		/* dg87l log for days, tasks and jobs */
  dg87_command_stat = 10	/* dg87s stat for days, tasks and jobs */
}
dg87_command_family;

/* privilege to access to functions */
typedef enum
{
  dg87_command_access_none,
  dg87_command_access_consult,
  dg87_command_access_manage,
  dg87_command_access_admin
}
dg87_command_privilege;


struct _dg87_context;
typedef struct _dg87_context dg87_context;

struct _dg87_connection;
typedef struct _dg87_connection dg87_connection;

struct _dg87_engine;
typedef struct _dg87_engine dg87_engine;

struct _dg87_interpret;
typedef struct _dg87_interpret dg87_interpret;

struct _dg87_server;
typedef struct _dg87_server dg87_server;

struct _dg87_user;
typedef struct _dg87_user dg87_user;

struct _dg87_command;
typedef struct _dg87_command dg87_command;

/*
 * dg87_update_current_time
 * this function is call to update the dg87_current_time variable.
 * It's called at the begining of the main function, and at the
 * begining of each cycle of the dg87server_run function.
 */
void dg87_update_current_time(void);

/*
 * dg87_ascii_to_date
 */
time_t dg87_ascii_to_date (char *string);

/*
 * dg87_date_to_ascii
 */
char *dg87_date_to_ascii (time_t time);

/*
 * dg87_bool_to_ascii
 */
char *dg87_bool_to_ascii (BOOL value);

/* 
 * dg87_ascii_to_bool
 */
BOOL dg87_ascii_to_bool (char *value);

/*
 * dg87_duration_to_ascii
 */
void dg87_duration_to_ascii (dg87uint secondes, char *buffer);

/*
 * dg87_strnzcpy 
 * this function is the same as 'strncpy' but it adds a \0 at the end.
 */
void dg87_strnzcpy (char *dest, char *src, int dest_size);

/* 
 * dg87_check_name
 * check that characters in buffer are valid : a-z A-Z _- 0-9
 */
BOOL dg87_check_name (char *buffer, int buffer_size);

/* 
 * dg87_check_description
 * check that characters in buffer are valid : a-z A-Z _- 0-9 space
 */
BOOL dg87_check_description (char *buffer, int buffer_size);


#ifdef DEBUG

#define DG87_OTHER	0
#define DG87_CHAR	1
#define DG87_VECTOR	2
#define DG87_LIST	3
#define DG87_ITEM	4
#define DG87_ELEMENT	5
#define DG87_ENGINE	6
#define DG87_SERVER	7
#define DG87_INTERPRET	8
#define DG87_CONNECTION	9
#define DG87_DAEMON	10
#define DG87_MESSAGE	11
#define DG87_COMMAND	12
#define DG87_CONTEXT	13
#define DG87_TASK 	14
#define DG87_JOB 	15
#define DG87_TARGET	16
#define DG87_QUEUE 	17
#define DG87_TASKDEP	18
#define DG87_JOBDEP	19
#define DG87_CONDITION	20
#define DG87_REF	21
#define DG87_DAY	22
#define DG87_RULE	23
#define DG87_OPERATION	24
#define DG87_STRBUF	25
#define DG87_USER	26
#define DG87_VARIABLE	27
#define DG87_STORAGE	28
#define DG87_LOGFILE	29
#define DG87_FILE	30

#define DG87_ARRAYSIZE	31
/* DON'T FORGET TO UPDATE dg87_refcptobj and dg87_objname in memory.c !!! */

/*void* dg87_memory_malloc(size_t A);*/
void *dg87_memory_malloc (size_t A, int B);

/*void* dg87_memory_strdup(const char* A);*/
void *dg87_memory_strdup (const char *A, int B);

/*extern void dg87_memory_free(void *ptr);*/
extern void dg87_memory_free (void *ptr, int B);

void dg87_memory_report (void);
int dg87_memory_usage (void);
char *dg87_memory_stat (void);

#define dg87_malloc(A,B) dg87_memory_malloc(A,B)
#define dg87_strdup(A,B) dg87_memory_strdup(A,B)
#define dg87_free(A,B) dg87_memory_free(A,B)

#define dg87_usage() dg87_memory_usage()
#define dg87_report() dg87_memory_report()
#define dg87_stat() dg87_memory_stat()

#else

#define dg87_malloc(A,B) malloc(A)
#define dg87_strdup(A,B) strdup(A)
#define dg87_free(A,B) free(A)

#define dg87_report() {}
#define dg87_usage() {}
#define dg87_stat() {}

#endif /*DEBUG*/
#endif /*__DG87_GLOBAL_H__*/
/* End of file */
