/* file list.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_LIST_H__
#define __DG87_LIST_H__
#include "global.h"
#include "item.h"

typedef struct _dg87_list
{
  dg87_item *first;		/* pointer on the first item */
  dg87_item *last;		/* pointer on the last item  */
  dg87uint count;		/* number of element in the list */
}
dg87_list;

/*
 *  dg87_list_new: 
 *  create a new list and return a pointer on this object.
 */
dg87_list *dg87_list_new (void);

/*
 * dg87_list_delete : delete a list
 */
BOOL dg87_list_delete (dg87_list * list);

/*
 * dg87_list_clear
 * delete of item of the list (but not the object !)
 * return FALSE if list is NULL.
 */
BOOL dg87_list_clear (dg87_list * list);

/*
 * dg87_list_add
 * add an item at the end of the list
 */
BOOL dg87_list_add_object (dg87_list * list, void *object);

/*
 * dg87_list_insert
 * insert the object at the given index in the list.
 * Return FALSE if the object can't be add
 */
BOOL dg87_list_insert_object (dg87_list * list, void *object, dg87uint index);

/*
 * dg87_list_count
 * return the number of elements in the list.
 * Return NULL if the list is empty.
 */
dg87uint dg87_list_count (dg87_list * list);

/*
 * dg87_list_item
 * return the element of the list which the given index.
 * Return NULL if index is out of bound.
 */
void *dg87_list_get_object_by_index (dg87_list * list, dg87uint index);

/*
 * dg87_list_remove_object
 * This function find the object in the list and remove it.
 * It return FALSE if it can't be find.
 */
BOOL dg87_list_remove_object (dg87_list * list, void *object);

/*
 * dg87_list_remove_by_index
 * remove the element of the list and return a pointer
 * on the object.
 */
void *dg87_list_remove_object_by_index (dg87_list * list, dg87uint index);

/*
 * dg87_list_get_item
 * return the item associated to an object at the given index.
 */
dg87_item *dg87_list_get_item (dg87_list * list, dg87uint index);

/*
 * dg87_list_move_item
 * This function is used to move an item in the list.
 * The delta value is -1 or +1
 */
BOOL dg87_list_move_item (dg87_list * list, dg87uint index, int delta);

/*
 * dg87_remove_item
 * remove the item at the given index in the list and
 * return a pointer on the object.
 */
void *dg87_list_remove_item (dg87_list * list, dg87_item * item);

/*
 * dg87_list_find_item
 * find the item associate with the given Object.
 * Return NULL if Object is not in the list.
 */
dg87_item *dg87_list_find_item (dg87_list * list, void *object);

/*
 * dg87_list_intersection
 * this function return TRUE if an object is present if the two lists.
 */
BOOL dg87_list_intersection (dg87_list * list1, dg87_list * list2);

#ifdef DEBUG
/*
 * dg87_list_ckeck
 */
int dg87_list_check (dg87_list * list, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_LIST_H__*/
/* End of file */
