/* file memory.c */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <syslog.h>
#include <time.h>
#include <assert.h>
#include "global.h"

#ifdef DEBUG

dg87ubyte dg87_save_needed = 0;
time_t dg87_current_time;

int dg87_refcptnew = 0;
int dg87_refcptfre = 0;
int dg87_refcptobj[DG87_ARRAYSIZE] =
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

char *dg87_objname[DG87_ARRAYSIZE] =
  { "Other", "Char", "Vector", "List", "Item", "Element", "Engine",
  "Server", "Interpret", "Connection", "Daemon", "Message", "Command",
  "Context", "Task", "Job", "Target", "Queue", "TaskDep", "JobDep",
  "Condition", "Ref", "Day", "Rule", "Operation", "Strbuf", "User",
  "Variable", "Storage", "LogFile", "File"
};

/******************************************************************************/
void dg87_update_current_time(void)
{
	dg87_current_time = time(NULL);
}
/******************************************************************************/
void *
dg87_memory_malloc (size_t A, int B)
{
  dg87_refcptobj[B]++;
  dg87_refcptnew++;
  return malloc (A);
}

/******************************************************************************/
void *
dg87_memory_strdup (const char *A, int B)
{
  assert (A != NULL);
  dg87_refcptobj[B]++;
  dg87_refcptnew++;
  return strdup (A);
}

/******************************************************************************/
void
dg87_strnzcpy (char *dest, char *src, int dest_size)
{
  assert (dest != NULL && src != NULL);
  strncpy (dest, src, dest_size);
  dest[dest_size] = '\0';
}

/******************************************************************************/
extern void
dg87_memory_free (void *ptr, int B)
{
  assert (ptr != NULL);
  dg87_refcptobj[B]--;
  dg87_refcptfre++;
  free (ptr);
}

/******************************************************************************/
int
dg87_memory_usage (void)
{
  return dg87_refcptnew - dg87_refcptfre;
}

/******************************************************************************/
char *
dg87_memory_stat (void)
{
  int i;
  char str[20];
  char *newbuffer;
  int size;
  char *buffer;

  buffer = strdup ("Memory usage:\n");
  if (buffer == NULL)
    return NULL;
  size = strlen (buffer) + 1;
  for (i = 0; i < DG87_ARRAYSIZE; i++)
    {
      sprintf (str, "%i", dg87_refcptobj[i]);
      size = size + strlen (dg87_objname[i]) + strlen (str) + 4;
      newbuffer = realloc (buffer, size);
      if (newbuffer == NULL)
	{
	  free (buffer);
	  return NULL;
	}
      buffer = newbuffer;
      strcat (buffer, dg87_objname[i]);
      strcat (buffer, " : ");
      strcat (buffer, str);
      strcat (buffer, "\n");
    }
  return buffer;
}

/******************************************************************************/
void
dg87_memory_report (void)
{
  syslog (LOG_ALERT, "reference count: new(%i) - free(%i)=%i\n",
	  dg87_refcptnew, dg87_refcptfre, dg87_refcptnew - dg87_refcptfre);
}
#endif /*DEBUG*/
/******************************************************************************/
/* dg87_ascii_to_date
 * convert a string to a time_t.
 * The formats of the string are :
 *  - YYYY-MM-DD           (10)
 *  - YYYY-MM-DD hh        (13)
 *  - YYYY-MM-DD hh:mm     (16)
 *  - YYYY-MM-DD hh:mm:ss  (19)
 * where YYYY is year, * MM is month, DD is month day, hh are hour, 
 * mm are minutes and ss are seconds.
 * The function return -1 on error.
 */
  time_t dg87_ascii_to_date (char *string)
{
  int len;
  time_t td;
  struct tm dc;
  int year, month, day, hour = 0, minute = 0, second = 0;
  char buffer[21];
  assert (string != NULL);
  if (string == NULL)
    return -1;
  len = strlen (string);
  if (len != 19 && len != 16 && len != 13 && len != 10)
    return -1;
  strcpy (buffer, string);
  /* control separators */
  if (buffer[4] != '-' || buffer[7] != '-')
    return -1;
  if (len >= 13 && buffer[10] != ' ')
    return -1;
  if (len >= 16 && buffer[13] != ':')
    return -1;
  if (len == 19 && buffer[16] != ':')
    return -1;
  /* interpret year, month and day */
  buffer[4] = '\0';
  buffer[7] = '\0';
  buffer[10] = '\0';
  year = atoi (buffer);
  month = atoi (buffer + 5);
  day = atoi (buffer + 8);
  /* interpret hours */
  if (len >= 13)
    {
      buffer[13] = '\0';
      hour = atoi (buffer + 11);
    }
  /* interpret minutes */
  if (len >= 16)
    {
      buffer[16] = '\0';
      minute = atoi (buffer + 14);
    }
  /* interpret seconds */
  if (len == 19)
    {
      buffer[19] = '\0';
      second = atoi (buffer + 17);
    }
  dc.tm_year = year - 1900;
  dc.tm_mon = month - 1;
  dc.tm_mday = day;
  dc.tm_hour = hour;
  dc.tm_min = minute;
  dc.tm_sec = second;
  dc.tm_isdst = -1;
  td = mktime (&dc);
  return td;
}

/******************************************************************************/
/* dg87_date_to_ascii
 * return a string witch represent the time given in parameter. 
 * The format of the string is YYYY-MM-DD hh:mm:ss where YYYY is year,
 * MM is month, DD is month day, hh are hour,mm are minutes and ss are seconds
 * The function return -1 on error.
 * The string must be free with a call to dg87_free
 */
char *
dg87_date_to_ascii (time_t time)
{
  struct tm dc;
  char format[] = "%Y-%m-%d %H:%M:%S";
  char *string = dg87_malloc (sizeof (char) * 21, DG87_CHAR);
  if (string == NULL)
    return NULL;
  dc = *localtime (&time);
  strftime (string, 21, format, &dc);
  return string;
}

/******************************************************************************/
/* dg87_bool_to_ascii
 * return a string witch represents the boolean value. 
 * user don't have to free memory.
 */
char *
dg87_bool_to_ascii (BOOL value)
{
  return (value == TRUE) ? "true" : "false";
}

/******************************************************************************/
/* dg87_ascii_to_bool
 * convert a string to a BOOL.
 * this function returns BOOLERROR if value is incorrect.
 */
BOOL
dg87_ascii_to_bool (char *value)
{
  assert (value != NULL);
  if (value == NULL)
    return BOOLERROR;
  if (strcmp (value, "true") == 0)
    return TRUE;
  if (strcmp (value, "false") == 0)
    return FALSE;
  return BOOLERROR;
}

/******************************************************************************/
/* 
 * dg87_duration__to_ascii
 * convert a duration in secondes in a string with `h:mm:ss' format.
 * The minimal length for the buffer is 15 characters. 
 */
void
dg87_duration_to_ascii (dg87uint secondes, char *buffer)
{
  dg87uint hours, minutes;
  assert (buffer != NULL);
  hours = secondes / 3600;
  secondes -= hours * 3600;
  minutes = secondes / 60;
  secondes -= minutes * 60;
  snprintf (buffer, 15, "%i:%.2i:%.2i", hours, minutes, secondes);

}

/******************************************************************************/
BOOL
dg87_check_name (char *buffer, int buffer_size)
{
  char *c;
  int i;
  assert (buffer != NULL && buffer_size > 0);
  if (buffer == NULL)
    return FALSE;
  if (*buffer == '\0')
    {
      /* name can't be an empty string */
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  /* valid character : a-z A-Z _- 0-9 */
  for (i = 0, c = buffer; i < buffer_size && *c != '\0'; i++, c++)
    {
      if (*c >= 'a' && *c <= 'z')
	continue;
      if (*c >= 'A' && *c <= 'Z')
	continue;
      if (*c >= '0' && *c <= '9')
	continue;
      if (*c == '_' || *c == '-')
	continue;
      /* invalid character */
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  if (*c != '\0')
    {
      /* name is too long */
      dg87_err_set_error (DG87_ERR_NAME_TOO_LONG);
      return FALSE;
    }
  return TRUE;
}

/******************************************************************************/
BOOL
dg87_check_description (char *buffer, int buffer_size)
{
  char *c;
  int i;
  assert (buffer != NULL && buffer_size > 0);
  if (buffer == NULL)
    return FALSE;
  /* description can be an empty string */
  /* valid character : a-z A-Z _- 0-9 space */
  for (i = 0, c = buffer; i < buffer_size && *c != '\0'; i++, c++)
    {
      if (*c >= 'a' && *c <= 'z')
	continue;
      if (*c >= 'A' && *c <= 'Z')
	continue;
      if (*c >= '0' && *c <= '9')
	continue;
      if (*c == '_' || *c == '-' || *c == ' ')
	continue;
      /* invalid character */
      dg87_err_set_error (DG87_ERR_INCORRECT_VALUE);
      return FALSE;
    }
  if (*c != '\0')
    {
      /* description is too long */
      dg87_err_set_error (DG87_ERR_DESCRIPTION_TOO_LONG);
      return FALSE;
    }
  return TRUE;
}

/******************************************************************************/
/* End of file */
