/* file message.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_MESSAGE_H__
#define __DG87_MESSAGE_H__
#include <time.h>
#include "global.h"
#include "connection.h"
#include "server.h"


typedef enum
{
  dg87_message_state_not_connected,
  dg87_message_state_read_banner,
  dg87_message_state_send_user,
  dg87_message_state_read_user_response,
  dg87_message_state_send_message,
  dg87_message_state_read_reponse,
  dg87_message_state_cancel,
  dg87_message_state_banner_error,
  dg87_message_state_authentification_error,
  dg87_message_state_connection_error
}
dg87_message_state;

typedef struct
{
  char *message_string;
  char *hostname;
  int ip_port;
  dg87_connection *connection;
  dg87_message_state state;
  time_t last_connection_retry;
  char *user;
  char *password;
  char *response;
}
dg87_message;

/*
 * dg87_message_new
 */
dg87_message *dg87_message_new (char *hostname, int ip_port,
				char *message_string);

/*
 * dg87_message_delete
 */
BOOL dg87_message_delete (dg87_message * message);

/*
 * dg87_message_send
 */
BOOL dg87_message_send (dg87_message * message,
			char *user_name, char *password);

/*
 * dg87_message_cancel
 */
BOOL dg87_message_cancel (dg87_message * message);

/*
 * dg87_message_is_aborted
 */
BOOL dg87_message_is_aborted (dg87_message * message);

/*
 * dg87_message_state_to_ascii
 */
char *dg87_message_state_to_ascii (dg87_message_state state);

#endif /*__DG87_MESSAGE_H__*/
/* End of file */
