/* file server.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_SERVER_H__
#define __DG87_SERVER_H__

#include <fcntl.h>
#include "interpret.h"
#include "global.h"
#include "connection.h"
#include "user.h"
#include "list.h"
#include "vector.h"

struct _dg87_server
{
  int ip_port;
  char* ip_addr;
    BOOL (*work_function) (dg87_server *);
  int (*signal_function) (dg87_server *, int signo);
  void *data;
  BOOL quit;
  BOOL shutdown;
  dg87_interpret *interpret;
  dg87_list *connection_list;
  dg87_list *message_list;
  dg87_vector *user_vector;
  char *program_name;
  char *program_version;
  char *package_name;
  int fd_pid_file;		/* file descriptor of PID file */
  char *pid_file_str;		/* path of the lock file */
};

/*
 * dg87_server_new
 */
dg87_server *dg87_server_new (char *package_name,
			      char *program_name,
			      char *program_version,
			      int ip_port,
			      char* ip_addr,
			      BOOL (*work_function) (dg87_server *),
			      int (*signal_function) (dg87_server *,
						      int signo), void *data,
			      dg87_interpret_cmd * cmd_array,
			      char *pid_file_str);

/*
 * dg87_server_run
 */
BOOL dg87_server_run (dg87_server * server);

/*
 * dg87_server_delete
 */
BOOL dg87_server_delete (dg87_server * server);

/*
 * dg87_server_post_message
 */
BOOL dg87_server_post_message (dg87_server * server,
			       char *hostname, int ip_port, char *message);

/* 
 * dg87_server_reload
 * This function is used to change PID file or IP port or
 * a running server.
 * pid_file_str is the string of the PID file. If pid_file_str is NULL,
 * no PID file is used.
 */
BOOL dg87_server_reload (dg87_server * server, char *pid_file_str,
			 char* ip_addr, int ip_port);

/*
 * dg87_server_get_diogene_password
 */
char *dg87_server_get_diogene_password (dg87_server * server);

/*************************************************************************/
/*  functions for users                                                  */
/*************************************************************************/

/*
 * dg87_server_user_add
 * create an user object, add this object in the UserVector 
 * of the server and return a pointer on this object.
 * Return NULL if the user object can't be create.
 */
dg87_user *dg87_server_user_add (dg87_server * server, char *name);

/*
 * dg87_server_user_count
 * return the number of users.
 * return -1 if server is NULL.
 */
int dg87_server_user_count (dg87_server * server);

/*
 * dg87_server_user_get_by_name
 * return a pointer on the user which has the name
 * given in parameter. Return NULL if it can't be find.
 */
dg87_user *dg87_server_user_get_by_name (dg87_server * server, char *name);

/*
 * dg87_server_user_get_by_index
 * return a pointer on the user at the given position.
 */
dg87_user *dg87_server_user_get_by_index (dg87_server * server, int index);

/*
 * dg87_server_user_delete_by_name
 * delete the user which has the name given in parameter.
 */
BOOL dg87_server_user_delete_by_name (dg87_server * server, char *name);

/*
 * dg87_server_user_delete_by_index
 * delete the user at the given index.
 */
BOOL dg87_server_user_delete_by_index (dg87_server * server, int index);

#endif /*__DG87_SERVER_H__*/

/* End of file */
