/* file storage.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_STORAGE_H__
#define __DG87_STORAGE_H__

#include <time.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include "global.h"

typedef enum
{
  dg87_storage_mode_closed,
  dg87_storage_mode_read,
  dg87_storage_mode_write
}
dg87_storage_mode;

typedef struct
{
  char *data_file;
  char *backup_file;
  char *signature;
  int min_version, max_version,	/* supported version */
    actual_version;		/* version of file which is reading */
  dg87_storage_mode mode;
  FILE* file_stream;
  uid_t uid, gid;		/* owner and group of the file if daemon is 
				   running in root account */
}
dg87_storage;

/*
 * dg87_storage_new
 */
dg87_storage *dg87_storage_new (char *data_file, char *backup_file,
				char *signature, int min_version,
				int max_version, char *user_account);

/*
 * dg87_storage_delete
 */
BOOL dg87_storage_delete (dg87_storage * storage);

/*
 * dg87_storage_open
 */
BOOL dg87_storage_open (dg87_storage * storage, dg87_storage_mode mode);

/*
 * dg87_storage_close
 */
BOOL dg87_storage_close (dg87_storage * storage);

/*
 * dg87_storage_write_string
 */
BOOL dg87_storage_write_string (dg87_storage * storage, char *string);

/*
 * dg87_storage_read_string
 */
BOOL dg87_storage_read_string (dg87_storage * storage, char **pstring);

/*
 * dg87_storage_write_long
 * 32 bits
 */
BOOL dg87_storage_write_long (dg87_storage * storage, dg87long value);

/*
 * dg87_storage_read_long
 */
BOOL dg87_storage_read_long (dg87_storage * storage, dg87long * value);

/*
 * dg87_storage_write_uint
 * 16 bits
 */
BOOL dg87_storage_write_uint (dg87_storage * storage, dg87uint value);

/*
 * dg87_storage_read_uint
 */
BOOL dg87_storage_read_uint (dg87_storage * storage, dg87uint * value);

/*
 * dg87_storage_write_int
 * 16 bits
 */
BOOL dg87_storage_write_int (dg87_storage * storage, dg87int value);

/*
 * dg87_storage_read_int
 */
BOOL dg87_storage_read_int (dg87_storage * storage, dg87int * value);

/*
 * dg87_storage_write_ubyte
 */
BOOL dg87_storage_write_ubyte (dg87_storage * storage, dg87ubyte value);

/*
 * dg87_storage_read_ubyte
 */
BOOL dg87_storage_read_ubyte (dg87_storage * storage, dg87ubyte * value);

/*
 * dg87_storage_write_byte
 */
BOOL dg87_storage_write_byte (dg87_storage * storage, dg87byte value);

/*
 * dg87_storage_read_byte
 */
BOOL dg87_storage_read_byte (dg87_storage * storage, dg87byte * value);


#endif /*__DG87_STORAGE__*/
/* End of file */
