/* file strbuf.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_STRBUF_H__
#define __DG87_STRBUF_H__
#include "global.h"

#define DG87_STRBUF_BUFFERSIZE 512

typedef struct _dg87_strbuf
{
  char *buffer;
  int size;
  int buffer_size;
}
dg87_strbuf;

/*
 * dg87_strbuf_new
 */
dg87_strbuf *dg87_strbuf_new (void);

/*
 * dg87_strbuf_delete
 */
BOOL dg87_strbuf_delete (dg87_strbuf * strbuf);

/*
 * dg87_strbuf_get_buffer
 * return a pointer on a string (terminated by a null character)
 * and reinitialize the string buffer (empty string).
 */
char *dg87_strbuf_get_buffer (dg87_strbuf * strbuf);

/*
 * dg87_strbuf_cat
 * concat a string to the buffer string. 
 */
BOOL dg87_strbuf_cat (dg87_strbuf * strbuf, char *string);

/*
 * dg87_strbuf_ncat
 * concat 'size' characters of a string to a string buffer.
 */
BOOL dg87_strbuf_ncat (dg87_strbuf * strbuf, char *string, int size);

/*
 * dg87_strbuf_cpy
 * copy a string at the end of a string buffer.
 */
BOOL dg87_strbuf_cpy (dg87_strbuf * strbuf, char *string);

/*
 * dg87_strbuf_ncpy
 * copy 'size' characters of a string at the end of a string buffer.
 */
BOOL dg87_strbuf_ncpy (dg87_strbuf * strbuf, char *string, int size);

/*
 * dg87_strbuf_printf
 * this function is an equivalent of `printf' function.
 */
BOOL dg87_strbuf_printf (dg87_strbuf * strbuf, char *format, ...);

#endif /*__DG87_STRBUF_H__*/

/* End of file */
