/* file user.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_USER_H__
#define __DG87_USER_H__

#include <config.h>
#include "global.h"
#include "storage.h"

/*
 * dg87_user object
 */
struct _dg87_user
{
  char name[DG87_USER_NAME_LENGTH + 1];
  char password[DG87_USER_PASSWORD_LENGTH + 1];
  int connection_count;		/* number of active connections */
  dg87_command_privilege privileges[DG87_COMMAND_FAM_MAX];
  BOOL system_account;
};

/*
 * dg87_user_new
 */
dg87_user *dg87_user_new (char *name);

/*
 * dg87_user_delete
 */
BOOL dg87_user_delete (dg87_user * user);

/*
 * dg87_user_load
 */
BOOL dg87_user_load (dg87_user * user, dg87_storage * storage);

/*
 * dg87_user_save
 */
BOOL dg87_user_save (dg87_user * user, dg87_storage * storage);


/*
 * dg87_user_set_password
 */
BOOL dg87_user_set_password (dg87_user * user, char *password);

/*
 * dg87_user_verify_password
 */
BOOL dg87_user_verify_password (dg87_user * user, char *password);

/*
 * dg87_user_update_connection_count
 */
BOOL dg87_user_update_connection_count (dg87_user * user, int delta);

/*
 * dg87_user_set_privilege
 */
BOOL dg87_user_set_privilege (dg87_user * user,
			      dg87_command_family family,
			      dg87_command_privilege privilege);

/*
 * dg87_user_set_administrator
 */
BOOL dg87_user_set_administrator (dg87_user * user);

/*
 * dg87_user_verify_privilege
 */
BOOL dg87_user_verify_privilege (dg87_user * user,
				 dg87_command_family family,
				 dg87_command_privilege privilege);

#endif /*__DG87_USER_H__*/
/* End of file */
