/* file vector.h */
/*
 *      libdg87 (Diogene87)
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_VECTOR_H__
#define __DG87_VECTOR_H__

#include "global.h"
#include "element.h"

typedef struct
{
  dg87_element *first;
  dg87_element *last;
  dg87uint count;
}
dg87_vector;

/*
 * dg87_vector_new
 */
dg87_vector *dg87_vector_new (void);

/*
 * dg87_vector_delete
 */
BOOL dg87_vector_delete (dg87_vector * vector);

/*
 * dg87_vector_clear
 */
BOOL dg87_vector_clear (dg87_vector * vector);

/*
 * dg87_vector_add_object
 */
BOOL dg87_vector_add_object (dg87_vector * vector, void *object, char *key);

/*
 * dg87_vector_remove_object
 * This function remove the object from the vector.
 * It return FALSE if the object can't be find.
 */
BOOL dg87_vector_remove_object (dg87_vector * vector, void *object);

/*
 * dg87_vector_remove_object_by_key
 * remove the object which correspond to the key given in parameter.
 * This function return a pointer on the object which was remove.
 * It return NULL if no object has a key witch match the given key.
 */
void *dg87_vector_remove_object_by_key (dg87_vector * vector, char *key);

/*
 * dg87_vector_remove_object_by_index
 * remove the object at the given index in parameter.
 * This function return a pointer on the object which was remove.
 */
void *dg87_vector_remove_object_by_index (dg87_vector * vector,
					  dg87uint index);
/*
 * dg87_vector_count
 */
dg87uint dg87_vector_count (dg87_vector * vector);

/*
 * dg87_vecotr_get_object_by_index
 */
void *dg87_vector_get_object_by_index (dg87_vector * vector, dg87uint index);

/*
 * dg87_vector_get_object_by_key
 */
void *dg87_vector_get_object_by_key (dg87_vector * vector, char *key);

/*
 * dg87_vector_get_element_by_index
 */
dg87_element *dg87_vector_get_element_by_index (dg87_vector * vector,
						dg87uint index);

/*
 * dg87_vector_get_element_by_key
 */
dg87_element *dg87_vector_get_element_by_key (dg87_vector * vector,
					      char *key);


/*
 * dg87_vector_find_element
 * find the element associate with the given Object.
 * Return NULL if Object is not in the list.
 */
dg87_element *dg87_vector_find_element (dg87_vector * vector, void *object);

#ifdef DEBUG
/*
 * dg87_vector_ckeck
 */
int dg87_vector_check (dg87_vector * vector, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_VECTOR_H__*/
/* End of file */
