#!/bin/sh
#
# file tst03.sh 
# (test dio87d functions)
#
#      Diogene87
#	Copyright (C) 2003,2004,2005 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" = "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "test exploitation commands"

startDio87d || exit 1

if [ $? != 0 ] ; then echo "error"; exit 1; fi

################################################## TARGET 
echo "Test target commands"
mem1=`dio87c -- memory`

# add a target 
dio87c -- ta tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a target"; exit 1; fi
# verify target count = 2
n=`dio87c -- tc`
if [ $n != 2 ] ; then echo "Target count isn't 2"; exit 1; fi
# verify target list
if [ "`dio87c -- tl | grep tg1`" != "tg1" ] ; then
	echo "Target is not in list."; exit 1
fi

# change target description
dio87c -- tsd tg1 newDesc
if [ $? != 0 ] ; then echo "Can't change target description !"; exit 1; fi
if [ "`dio87c -- ti -S! tg1 | grep Desc`" != "Description!newDesc" ] ; then
	echo "Target description is not changed"; exit 1
fi
# change open property
dio87c -- tso tg1 true 
if [ $? != 0 ] ; then echo "Can't change target open property !"; exit 1; fi
if [ "`dio87c -- ti -S! tg1 | grep Open`" != "Opened!true" ] ; then
	echo "Target max property is not changed"; exit 1
fi
# change ip port property
dio87c -- tsp tg1 8888 
if [ $? != 0 ] ; then echo "Can't change IP Port property !"; exit 1; fi
if [ "`dio87c -- ti -S! tg1 | grep Port`" != "IP Port!8888" ] ; then
	echo "Target IP Port property is not changed"; exit 1
fi

# try to add a second target with same name
dio87c -- ta tg1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add two target with same name"; exit 1; fi

# delete target
dio87c -- tk tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the target"; exit 1; fi

# verify target count = 1
n=`dio87c -- tc`
if [ $n != 1 ] ; then echo "Target count isn't 1"; exit 1; fi

# try to delete a target which doesn't exist
dio87c -- tk tg1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete the target which doesn't exist"; exit 1; fi

waitMessageSending
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi


################################################## QUEUE 
echo "Test queue commands"

# add a target 
dio87c -- ta tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a target"; exit 1; fi
# add a queue 
dio87c -- qa tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a queue"; exit 1; fi

# verify queue count = 2
n=`dio87c -- qc tg1`
if [ $n != 2 ] ; then echo "Queue count isn't 2"; exit 1; fi
# verify queue list
if [ "`dio87c -- ql tg1 | grep q1`" != "q1" ]; then
	echo "Queue is not in queue list of the target"; exit 1
fi

# change queue description
dio87c -- qsd tg1 q1 newDesc
if [ $? != 0 ] ; then echo "Can't change queue description !"; exit 1; fi
if [ "`dio87c -- qi -S! tg1 q1 | grep Desc`" != "Description!newDesc" ] ; then
	echo "Queue description is not changed"; exit 1
fi

# change queue max 
dio87c -- qsm tg1 q1 55
if [ $? != 0 ] ; then echo "Can't change max property of Queue"; exit 1; fi
# verify max property change
if [ "`dio87c -- qi tg1 q1 | grep Max|awk '{print $2}'`" != "55" ] ; then
	echo "Queue max property is not changed"; exit 1;
fi

# change open property
dio87c -- qso tg1 q1 true
if [ $? != 0 ] ; then echo "Can't change queue open property !"; exit 1; fi
if [ "`dio87c -- qi -S! tg1 q1 | grep Open`" != "Opened!true" ] ; then
	echo "Queue max property is not changed"; exit 1
fi

# try to add a second queue with same name
dio87c -- qa tg1 q1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can add two queue with same name"; exit 1; fi

# delete queue 
dio87c -- qk tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the queue"; exit 1; fi

# verify queue count = 1
n=`dio87c -- qc tg1`
if [ $n != 1 ] ; then echo "Queue count isn't 1"; exit 1; fi

# try to delete a queue which doesn't exist
dio87c -- qk tg1 q1 > /dev/null 2>&1
if [ $? = 0 ] ; then echo "Can delete a queue which doesn't exist"; exit 1; fi

# delete the target
dio87c -- tk tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a target"; exit 1; fi

waitMessageSending
mem2=`dio87c -- memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a target with queues
mem1=`dio87c -- memory`
dio87c -- ta tg1 > /dev/null 2>&1
dio87c -- qa tg1 q1 > /dev/null 2>&1
dio87c -- qa tg1 q2 > /dev/null 2>&1
dio87c -- qa tg1 q3 > /dev/null 2>&1
dio87c -- tk tg1 > /dev/null 2>&1

waitMessageSending
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## DAY
echo "Test exploitation days commands"

mem1=`dio87c -- memory`
# add a day
dio87c -- ra r1 || exit 1
dio87c -- rp -d r1 "$dayDate" d1 >/dev/null || exit 1

# verify day count
if [ "`dio87c -- xdc`" -ne 1 ] ; then echo "Day count is not 1."; exit 1; fi
# verify day list
if [ "`dio87c -- xdl`" != "d1" ] ; then echo "Day not in day list."; exit 1;fi

# verify start time
if [ "`dio87c -- xdi -S! d1 | grep Start`" != "Start Time!$dayDate" ] ; then
	echo "Day start time is not set ($dayDate)."; exit 1
fi

# verify that day is disabled
if [ "`dio87c -- xdi -S! d1 | grep Enabled`" != "Enabled!false" ] ; then
	echo "Day is not disabled."; exit 1
fi
# enable day
dio87c -- xdse d1 true 
if [ $? -ne 0 ] ; then echo "Can't enable day."; exit 1; fi
# verify that day is enabled
if [ "`dio87c -- xdi -S! d1 | grep Enabled`" != "Enabled!true" ] ; then
	echo "Day is not enabled."; exit 1
fi


# delete day
dio87c -- xdk d1
if [ $? != 0 ]; then echo "Can't delete day"; exit 1; fi


dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- ldk d1 || exit 1
waitMessageSending
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi


# try to delete an inexistant day
dio87c -- xdk jx > /dev/null 2>&1
if [ $? = 0 ]; then echo "Can delete an inexistant day"; exit 1; fi

################################################## TASK 
echo "Test tasks commands"
mem1=`dio87c -- memory`

# add a target and a queue
dio87c -- ta tg1
if [ $? != 0 ]; then echo "Can't add target"; exit 1; fi
dio87c -- qa tg1 q1
if [ $? != 0 ]; then echo "Can't add queue"; exit 1; fi

# add a day
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtsd r1 t1 "task1" || exit 1
dio87c -- rtst r1 t1 tg1 q1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1
if [ $? != 0 ]; then echo "Can't add day"; exit 1; fi

# verify task count and list
if [ `dio87c -- xtc d1` -ne 1 ] ; then echo "Task count isn't 1."; exit 1; fi
if [ "`dio87c -- xtl d1`" != "t1" ]; then 
	echo "Task is not in task list of day"; exit 1
fi

if [ "`dio87c -- xti -S! d1 t1 | grep \"Day Name\"`" != "Day Name!d1" ] ; then
	echo "Task day name is not set." ; exit 1
fi
# verify task description
if [ "`dio87c -- xti -S! d1 t1 | grep Description`" != "Description!task1" ] ; then
	echo "Task description is not set"; exit 1
fi
# verify queue 
if [ "`dio87c -- xti -S! d1 t1 | grep Queue`" != "Queue!q1" ] ; then
	echo "Queue is not set"; exit 1
fi


# delete day (with a task)
dio87c -- xdk d1
if [ $? != 0 ]; then echo "Can't delete a day with tasks"; exit 1; fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- ldk d1 || exit 1
dio87c -- tk tg1 || exit 1

waitMessageSending
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi


################################################## JOB 
echo "Test jobs commands"
mem1=`dio87c -- memory`

# add a target and a queue
dio87c -- ta tg1 || exit 1
dio87c -- qa tg1 q1 || exit 1

# add a day
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtst r1 t1 tg1 q1 || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rjsd r1 t1 j1 job1 || exit 1
dio87c -- rjsp r1 t1 j1 "/bin/ls" || exit 1
dio87c -- rjsa r1 t1 j1 "arg" || exit 1
dio87c -- rjst r1 t1 j1 25 35 || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# verify job fields
if [ "`dio87c -- xji -S! d1 t1 j1 | grep \"Task Name\"`" != "Task Name!t1" ] ; then
	echo "Job task is not set." ; exit 1
fi
if [ "`dio87c -- xji -S! d1 t1 j1 | grep \"Day Name\"`" != "Day Name!d1" ] ; then
	echo "Job day is not set." ; exit 1
fi
if [ "`dio87c -- xji -S! d1 t1 j1 | grep Description`" != "Description!job1" ] ; then
	echo "Job description is not set." ; exit 1
fi
if [ "`dio87c -- xji -S! d1 t1 j1 | grep Queue`" != "Target Queue!q1" ] ; then
	echo "Job queue is not set." ; exit 1
fi
if [ "`dio87c -- xji -S! d1 t1 j1 | grep Path`" != "Path!/bin/ls" ] ; then
	echo "Job path is not set." ; exit 1
fi
if [ "`dio87c -- xji -S! d1 t1 j1 | grep Argument`" != "Argument!arg" ] ; then
	echo "Job argument is not set." ; exit 1
fi
if [ "`dio87c -- xji -S! d1 t1 j1 | grep \"Start Tempo\"`" != "Start Tempo!25" ]; then
	echo "Job start tempo is not set." ; exit 1
fi
if [ "`dio87c -- xji -S! d1 t1 j1 | grep \"End Tempo\"`" != "End Tempo!35" ] ; then
	echo "Job end tempo is not set." ; exit 1
fi


# delete day (with tasks and jobs)
dio87c -- xdk d1
if [ $? != 0 ]; then echo "Can't delete a day with jobs"; exit 1; fi

dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1
dio87c -- ldk d1 || exit 1
dio87c -- tk tg1 || exit 1
waitMessageSending
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## TASK DEP 
echo "Test task dependencies commands" 

mem1=`dio87c -- memory`
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rta r1 t2 || exit 1
dio87c -- rtoa r1 t2 add always || exit 1
dio87c -- rta r1 t3 || exit 1
dio87c -- rtoa r1 t3 add always || exit 1
dio87c -- rta r1 t4 || exit 1
dio87c -- rtoa r1 t4 add always || exit 1
dio87c -- rta r1 t5 || exit 1
dio87c -- rtoa r1 t5 add always || exit 1
dio87c -- rtda r1 t1 t2 always || exit 1
dio87c -- rtda r1 t2 t3 onerror || exit 1
dio87c -- rtda r1 t3 t4 onsuccess || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# verify task dependence list
if [ "`dio87c -- xtdc d1`" != "3" ] ; then 
	echo "Task dependence count isn't 3"; exit 1
fi
if [ "`dio87c -- xtdl -tS! d1 | grep always`" != "t1!t2!always" ] ; then
	echo "Task dependence is not in list (always)"; exit 1
fi
if [ "`dio87c -- xtdl -tS! d1 | grep onerror`" != "t2!t3!onerror" ] ; then
	echo "Task dependence is not in list (onerror)"; exit 1
fi
if [ "`dio87c -- xtdl -tS! d1 | grep onsuccess`" != "t3!t4!onsuccess" ] ; then
	echo "Task dependence is not in list (onsuccess)"; exit 1
fi

# delete day (with task dependencies)
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1

waitMessageSending
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## JOB DEP
echo "Test job dependencies commands" 


mem1=`dio87c -- memory`

dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rja r1 t1 j1 || exit 1
dio87c -- rja r1 t1 j2 || exit 1
dio87c -- rja r1 t1 j3 || exit 1
dio87c -- rja r1 t1 j4 || exit 1
dio87c -- rjda r1 t1 j1 j2 always
dio87c -- rjda r1 t1 j2 j3 onerror 
dio87c -- rjda r1 t1 j3 j4 onsuccess
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# verify task dependence list
if [ "`dio87c -- xjdc d1 t1`" != "3" ] ; then 
	echo "Job dependence count isn't 3"; exit 1
fi
if [ "`dio87c -- xjdl -tS! d1 t1 | grep always`" != "j1!j2!always" ] ; then
	echo "Job dependence is not in list (always)"; exit 1
fi
if [ "`dio87c -- xjdl -tS! d1 t1 | grep onerror`" != "j2!j3!onerror" ] ; then
	echo "Job dependence is not in list (onerror)"; exit 1
fi
if [ "`dio87c -- xjdl -tS! d1 t1 | grep onsuccess`" != "j3!j4!onsuccess" ] ; then
	echo "Job dependence is not in list (onsuccess)"; exit 1
fi


# delete day (with task dependencies)
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1

waitMessageSending
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## CONDITIONS 
echo "Test conditions commands" 

mem1=`dio87c -- memory`
dio87c -- ra r1 || exit 1
dio87c -- rta r1 t1 || exit 1
dio87c -- rtoa r1 t1 add always || exit 1
dio87c -- rtca r1 t1 file "/tmp/fic" || exit 1
dio87c -- rtca r1 t1 manual "ok"  || exit 1
dio87c -- rtca r1 t1 hmin "08:00" 0 || exit 1
dio87c -- rtca r1 t1 hmax "09:00" 0 || exit 1
dio87c -- rtca r1 t1 hlate "12:00" 0 || exit 1
dio87c -- rp r1 "$dayDate" d1 >/dev/null || exit 1

# verify condition list
if [ "`dio87c -- xtcc d1 t1`" != "5" ]; then
	echo "Condition count is not 5."; exit 1
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep file`" != "file!/tmp/fic" ]; then
	echo "File condition is not in list."; exit 1
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep manual`" != "manual!ok" ]; then
	echo "File condition is not in list."; exit 1
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep hmin`" != "hmin!08:00!+0" ]; then
	echo "HMin condition is not in list."; exit 1
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep hmax`" != "hmax!09:00!+0" ]; then
	echo "HMax condition is not in list."; exit 1
fi
if [ "`dio87c -- xtcl -S! d1 t1 | grep hlate`" != "hlate!12:00!+0" ]; then
	echo "HLate condition is not in list."; exit 1
fi
	
dio87c -- xdk d1 || exit 1
dio87c -- ldk d1 || exit 1
dio87c -- rk r1 || exit 1
dio87c -- srk r1 || exit 1

waitMessageSending
mem2=`dio87c -- memory`
if [ $mem1 != $mem2 ] ; then
	echo "dio87d : memory error $mem1 -> $mem2"
	exit 1
fi

stopDio87d || exit 1


