/* file queue.h */
/*
 *      dio87d (Diogene87) - a powerful job scheduling system.
 *	Copyright (C) 2003,2004,2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef __DG87_QUEUE_H__
#define __DG87_QUEUE_H__

#include "global.h"
#include "target.h"

struct _dg87_queue
{
  char name[DG87_QUEUE_NAME_LENGTH + 1];
  char description[DG87_QUEUE_DESC_LENGTH + 1];
  dg87_target *target;
  BOOL open;
	/* can't be deleted if true (`batch' queue) */
  BOOL system;                  
	/* maxiumum of jobs which can be run at the same time */
  int max;                      
	/* referential tasks which are linked with this queue */
  dg87_list *ref_task_list;     
	/* exploitation tasks which are linked with this queue */
  dg87_list *exp_task_list;     
	/* jobs which are actually running */
  dg87_list *running_job_list;  
};

/*
 * dg87_queue_new 
 * create a new queue and initialize the name property.
 * if the `system' argument is TRUE, the target can't be deleted.
 */
dg87_queue *dg87_queue_new (dg87_target * target, char *name, BOOL system);

/*
 * dg87_queue_delete
 * if the `force' argument is TRUE, a system queue can be deleted.
 */
BOOL dg87_queue_delete (dg87_queue * queue, BOOL force);

/*
 * dg87_queue_load
 */
BOOL dg87_queue_load (dg87_queue * queue, dg87_storage * storage);

/*
 * dg87_queue_save
 */
BOOL dg87_queue_save (dg87_queue * queue, dg87_storage * storage);

/*
 * dg87_queue_set_name
 * change the name of the queue.
 */
BOOL dg87_queue_set_name(dg87_queue * queue, char *new_name);


/*
 * dg87_queue_set_description
 * set the description of the queue.
 */
BOOL dg87_queue_set_description (dg87_queue * queue, char *description);

/*
 * dg87_queue_set_open
 * set the value for the "open" property.
 * return FALSE if the open's value can't be change.
 */
BOOL dg87_queue_set_open (dg87_queue * queue, BOOL value);

/*
 * dg87_queue_set_max
 * set the value of the "max" property.
 * return FALSE if the value can't be change.
 */
BOOL dg87_queue_set_max (dg87_queue * queue, int value);

/*
 * dg87_queue_add_running_job
 */
BOOL dg87_queue_add_running_job (dg87_queue * queue, dg87_exp_job * job);
/*
 * dg87_queue_remove_running_job
 */
BOOL dg87_queue_remove_running_job (dg87_queue * queue, dg87_exp_job * job);
/*
 * dg87_queue_manage
 * This function is call by the Engine's object.
 */
BOOL dg87_queue_manage (dg87_context * context, dg87_queue * queue);



#ifdef DEBUG
/*
 * dg87_queue_ckeck
 */
int dg87_queue_check (dg87_queue * queue, int (*print) (char *));
#endif /*DEBUG*/
#endif /*__DG87_QUEUE_H__*/
/* End of file */
